@@ -23,9 +23,19 @@
 
 """
 Main application
+
+Hierarchy of the whole thing:
+
+Pitivi
+    InteractivePitivi
+    GuiPitivi
+        ProjectCreatorGuiPitivi
+        ProjectLoaderGuiPitivi
+        StartupWizardGuiPitivi
 """
 import os
 import sys
+
 from gi.repository import GES
 from gi.repository import Gio
 from gi.repository import GLib
@@ -34,10 +44,8 @@ from gi.repository import Gtk
 from gettext import gettext as _
 from optparse import OptionParser
 
-import pitivi.instance as instance
-
 from pitivi.effects import EffectsHandler
-from pitivi.configure import APPNAME, pitivi_version, RELEASES_URL
+from pitivi.configure import VERSION, RELEASES_URL
 from pitivi.settings import GlobalSettings
 from pitivi.utils.threads import ThreadMaster
 from pitivi.mainwindow import PitiviMainWindow
@@ -54,18 +62,6 @@ import pitivi.utils.loggable as log
 #from pitivi.undo.timeline import TimelineLogObserver
 
 
-"""
-Hierarchy of the whole thing:
-
-Pitivi
-    InteractivePitivi
-    GuiPitivi
-        ProjectCreatorGuiPitivi
-        ProjectLoaderGuiPitivi
-        StartupWizardGuiPitivi
-"""
-
-
 class Pitivi(Loggable, Signallable):
     """
     Pitivi's main application class.
@@ -76,10 +72,6 @@ class Pitivi(Loggable, Signallable):
      - C{new-project-loading} : Pitivi is attempting to load a new project.
      - C{new-project-loaded} : A new L{Project} has been loaded, and the UI should refresh it's view.
      - C{new-project-failed} : A new L{Project} failed to load.
-     - C{closing-project} :  pitivi would like to close a project. handlers should return false
-     if they do not want this project to close. by default, assumes
-     true. This signal should only be used by classes that might want to abort
-     the closing of a project.
      - C{project-closed} : The project is closed, it will be freed when the callback returns.
      Classes should connect to this instance when they want to know that
      data related to that project is no longer going to be used.
@@ -97,20 +89,16 @@ class Pitivi(Loggable, Signallable):
         "new-project-created": ["project"],
         "new-project-loaded": ["project"],
         "new-project-failed": ["uri", "exception"],
-        "closing-project": ["project"],
         "project-closed": ["project"],
         "missing-uri": ["formatter", "uri"],
         "version-info-received": ["versions"],
         "shutdown": None}
 
     def __init__(self):
-        """
-        initialize pitivi with the command line arguments
-        """
         Loggable.__init__(self)
 
-        # init logging as early as possible so we can log startup code
-        enable_color = os.environ.get('PITIVI_DEBUG_NO_COLOR', '0') in ('', '0')
+        # Init logging as early as possible so we can log startup code
+        enable_color = not os.environ.get('PITIVI_DEBUG_NO_COLOR', '0') in ('', '1')
         # Let's show a human-readable pitivi debug output by default, and only
         # show a crazy unreadable mess when surrounded by gst debug statements.
         enable_crack_output = "GST_DEBUG" in os.environ
@@ -118,24 +106,12 @@ class Pitivi(Loggable, Signallable):
 
         self.info('starting up')
 
-        # store ourself in the instance global
-        if instance.Pitivi:
-            raise RuntimeWarning(_("There is already a %s instance, please inform "
-                "the developers by filing a bug at "
-                "http://bugzilla.gnome.org/enter_bug.cgi?product=pitivi")
-                % APPNAME)
-        instance.Pitivi = self
-
-        self.current_project = None
-
-        # get settings
         self.settings = GlobalSettings()
         self.threads = ThreadMaster()
-        #self.screencast = False
-
         self.effects = EffectsHandler()
         self.system = getSystem()
 
+        self.current_project = None
         self.projectManager = ProjectManager(self)
         self._connectToProjectManager(self.projectManager)
 
@@ -146,7 +122,7 @@ class Pitivi(Loggable, Signallable):
         #self.timelineLogObserver = TimelineLogObserver(self.action_log)
         self.projectLogObserver = ProjectLogObserver(self.action_log)
 
-        self.version_information = {}
+        self._version_information = {}
         self._checkVersion()
 
     def shutdown(self):
@@ -165,7 +141,6 @@ class Pitivi(Loggable, Signallable):
         self.threads.stopAllThreads()
         self.settings.storeSettings()
         self.current_project = None
-        instance.Pitivi = None
         self.emit("shutdown")
         return True
 
@@ -175,20 +150,19 @@ class Pitivi(Loggable, Signallable):
         pm.connect("new-project-created", self._projectManagerNewProjectCreated)
         pm.connect("new-project-loaded", self._projectManagerNewProjectLoaded)
         pm.connect("new-project-failed", self._projectManagerNewProjectFailed)
-        pm.connect("closing-project", self._projectManagerClosingProject)
         pm.connect("project-closed", self._projectManagerProjectClosed)
 
-    def _projectManagerNewProjectLoading(self, projectManager, uri):
+    def _projectManagerNewProjectLoading(self, unused_project_manager, uri):
         self.emit("new-project-loading", uri)
 
-    def _projectManagerNewProjectCreated(self, projectManager, project):
+    def _projectManagerNewProjectCreated(self, unused_project_manager, project):
         self.current_project = project
         self.emit("new-project-created", project)
 
-    def _newProjectLoaded(self, project):
+    def _newProjectLoaded(self, unused_project):
         pass
 
-    def _projectManagerNewProjectLoaded(self, projectManager, project, unused_fully_loaded):
+    def _projectManagerNewProjectLoaded(self, unused_project_manager, project, unused_fully_loaded):
         self.current_project = project
         self.action_log.clean()
         #self.timelineLogObserver.startObserving(project.timeline)
@@ -196,13 +170,10 @@ class Pitivi(Loggable, Signallable):
         self._newProjectLoaded(project)
         self.emit("new-project-loaded", project)
 
-    def _projectManagerNewProjectFailed(self, projectManager, uri, exception):
+    def _projectManagerNewProjectFailed(self, unused_project_manager, uri, exception):
         self.emit("new-project-failed", uri, exception)
 
-    def _projectManagerClosingProject(self, projectManager, project):
-        return self.emit("closing-project", project)
-
-    def _projectManagerProjectClosed(self, projectManager, project):
+    def _projectManagerProjectClosed(self, unused_project_manager, project):
         #self.timelineLogObserver.stopObserving(project.timeline)
         self.projectLogObserver.stopObserving(project)
         self.current_project = None
@@ -214,7 +185,7 @@ class Pitivi(Loggable, Signallable):
         self.info("Requesting version information")
         giofile.load_contents_async(None, self._versionInfoReceivedCb, None)
 
-    def _versionInfoReceivedCb(self, giofile, result, data):
+    def _versionInfoReceivedCb(self, giofile, result, user_data):
         try:
             raw = giofile.load_contents_finish(result)[1]
             raw = raw.split("\n")
@@ -224,21 +195,41 @@ class Pitivi(Loggable, Signallable):
 
             # search newest version and status
             status = "UNSUPPORTED"
+            current_version = None
             for version, version_status in data:
-                if pitivi_version == version:
+                if VERSION == version:
                     status = version_status
                 if version_status.upper() == "CURRENT":
+                    # This is the latest.
                     current_version = version
 
             self.info("Latest software version is %s", current_version)
-            if status is "UNSUPPORTED":
-                self.warning("Using an outdated version of Pitivi (%s)" % pitivi_version)
+            # Python is magical... comparing version *strings* always works,
+            # even with different major.minor.nano version number schemes!
+            if VERSION > current_version:
+                status = "CURRENT"
+                self.info("Running version %s, which is newer than the latest known version. Considering it as the latest current version.", VERSION)
+            elif status is "UNSUPPORTED":
+                self.warning("Using an outdated version of Pitivi (%s)", VERSION)
 
-            self.version_information["current"] = current_version
-            self.version_information["status"] = status
-            self.emit("version-info-received", self.version_information)
+            self._version_information["current"] = current_version
+            self._version_information["status"] = status
+            self.emit("version-info-received", self._version_information)
         except Exception, e:
-            self.warning("Version info could not be read: %s" % e)
+            self.warning("Version info could not be read: %s", e)
+
+    def isLatest(self):
+        """
+        Whether the app's version is the latest as far as we know.
+        """
+        status = self._version_information.get("status")
+        return status is None or status.upper() == "CURRENT"
+
+    def getLatest(self):
+        """
+        Get the latest version of the app or None.
+        """
+        return self._version_information.get("current")
 
 
 class InteractivePitivi(Pitivi):
@@ -249,33 +240,16 @@ class InteractivePitivi(Pitivi):
     def __init__(self, debug=False):
         Pitivi.__init__(self)
         self.mainloop = GLib.MainLoop()
-        self.actioner = None
         self.gui = None
         if debug:
             sys.excepthook = self._excepthook
 
-    def _excepthook(self, exc_type, value, tback):
+    def _excepthook(self, unused_exc_type, unused_value, tback):
         import traceback
         import pdb
         traceback.print_tb(tback)
         pdb.post_mortem(tback)
 
-    def _setActioner(self, actioner):
-        self.actioner = actioner
-        if self.actioner:
-            self.actioner.connect("eos", self._eosCb)
-            # On error, all we need to do is shutdown which
-            # is the same as we do for EOS
-            self.actioner.connect("error", self._eosCb)
-            # Configure the actioner and start acting!
-            self.actioner.startAction()
-
-    def _eosCb(self, unused_obj):
-        raise NotImplementedError()
-
-    def _loadProject(self, project_filename):
-        self.projectManager.loadProject(quote_uri(os.path.abspath(project_filename)))
-
     def run(self):
         """Runs the main loop."""
         self.mainloop.run()
@@ -294,15 +268,12 @@ class GuiPitivi(InteractivePitivi):
         self._showGui()
 
     def _showStartupError(self, message, detail):
-        dialog = Gtk.MessageDialog(type=Gtk.MessageType.ERROR,
+        dialog = Gtk.MessageDialog(message_type=Gtk.MessageType.ERROR,
                                    buttons=Gtk.ButtonsType.OK)
         dialog.set_markup("<b>" + message + "</b>")
         dialog.format_secondary_text(detail)
         dialog.run()
 
-    def _eosCb(self, unused_obj):
-        self.shutdown()
-
     def _createGui(self):
         """Returns a Gtk.Widget which represents the UI."""
         return PitiviMainWindow(self)
@@ -338,7 +309,7 @@ class ProjectCreatorGuiPitivi(GuiPitivi):
         lib.connect("discovery-error", self._discoveryErrorCb, uris)
         lib.addUris(uris)
 
-    def _sourceAddedCb(self, medialibrary, info, startup_uris, add_to_timeline):
+    def _sourceAddedCb(self, unused_media_library, info, startup_uris, add_to_timeline):
         if self._maybePopStartupUri(startup_uris, info.get_uri()) \
                 and add_to_timeline:
             self.action_log.begin("add clip")
@@ -347,7 +318,7 @@ class ProjectCreatorGuiPitivi(GuiPitivi):
             self.current_project.timeline.get_layers()[0].add_clip(src)
             self.action_log.commit()
 
-    def _discoveryErrorCb(self, medialibrary, uri, error, debug, startup_uris):
+    def _discoveryErrorCb(self, unused_media_library, uri, unused_error, unused_debug, startup_uris):
         self._maybePopStartupUri(startup_uris, uri)
 
     def _maybePopStartupUri(self, startup_uris, uri):
@@ -376,8 +347,7 @@ class ProjectLoaderGuiPitivi(GuiPitivi):
         if not os.path.exists(project_filename):
             self.error("Project file does not exist: %s" % project_filename)
             sys.exit(1)
-        else:
-            self._loadProject(project_filename)
+        self.projectManager.loadProject(quote_uri(os.path.abspath(project_filename)))
 
 
 class StartupWizardGuiPitivi(GuiPitivi):
@@ -392,10 +362,7 @@ class StartupWizardGuiPitivi(GuiPitivi):
         self.projectManager.newBlankProject(emission=False)
 
     def _createGui(self):
-        # Prevent the main window to go fullscreen because at least
-        # the Metacity window manager will refuse to bring
-        # the startup wizard window in front of the main window.
-        return PitiviMainWindow(self, allow_full_screen=False)
+        return PitiviMainWindow(self)
 
     def _showGui(self):
         GuiPitivi._showGui(self)
