@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       clipproperties.py
+#       pitivi/clipproperties.py
 #
 # Copyright (C) 2010 Thibault Saunier <tsaunier@gnome.org>
 #
@@ -31,7 +31,6 @@ from gi.repository import GES
 
 from gettext import gettext as _
 
-from pitivi.check import missing_soft_deps
 from pitivi.configure import get_ui_dir
 
 from pitivi.dialogs.depsmanager import DepsManager
@@ -69,7 +68,7 @@ def compare_type(track, effect_type):
 
 class ClipProperties(Gtk.ScrolledWindow, Loggable):
     """
-    Widget for configuring clips properties
+    Widget for configuring the selected clip.
     """
 
     def __init__(self, instance, uiman):
@@ -80,8 +79,8 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
         self._project = None
 
         self.infobar_box = Gtk.VBox()
-        effect_properties_handling = EffectsPropertiesManager(instance)
-        self.effect_expander = EffectProperties(instance, effect_properties_handling, self)
+        effects_properties_manager = EffectsPropertiesManager(instance)
+        self.effect_expander = EffectProperties(instance, effects_properties_manager, self)
         self.effect_expander.set_vexpand(False)
         # Transformation boxed DISABLED
         #self.transformation_expander = TransformationProperties(instance, instance.action_log)
@@ -138,11 +137,14 @@ class ClipProperties(Gtk.ScrolledWindow, Loggable):
 class EffectProperties(Gtk.Expander, Loggable):
     """
     Widget for viewing and configuring effects
+
+    @type instance: C{Pitivi}
+    @type effects_properties_manager: C{EffectsPropertiesManager}
     """
     # Note: This should be inherited from Gtk.Expander when we get other things
     # to put in ClipProperties, that is why this is done this way
 
-    def __init__(self, instance, effect_properties_handling, clip_properties):
+    def __init__(self, instance, effects_properties_manager, clip_properties):
         # Set up the expander widget that will contain everything:
         Gtk.Expander.__init__(self)
         self.set_expanded(True)
@@ -156,14 +158,14 @@ class EffectProperties(Gtk.Expander, Loggable):
         self.selected_effects = []
         self.clips = []
         self._effect_config_ui = None
-        self.effect_props_handling = effect_properties_handling
+        self.effects_properties_manager = effects_properties_manager
         self.clip_properties = clip_properties
         self._config_ui_h_pos = None
         self._timeline = None
 
         # The toolbar that will go between the list of effects and properties
         self._toolbar = Gtk.Toolbar()
-        self._toolbar.get_style_context().add_class("inline-toolbar")
+        self._toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
         self._toolbar.set_icon_size(Gtk.IconSize.SMALL_TOOLBAR)
         removeEffectButton = Gtk.ToolButton()
         removeEffectButton.set_icon_name("list-remove-symbolic")
@@ -178,7 +180,7 @@ class EffectProperties(Gtk.Expander, Loggable):
         self.treeview_scrollwin.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         self.storemodel = Gtk.ListStore(bool, str, str, str, object)
-        self.treeview = Gtk.TreeView(self.storemodel)
+        self.treeview = Gtk.TreeView(model=self.storemodel)
         self.treeview_scrollwin.add(self.treeview)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_property("has_tooltip", True)
@@ -223,7 +225,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
         # Prepare the main container widgets and lay out everything
         self._vcontent = Gtk.VPaned()
-        self._table = Gtk.Table(3, 1, False)
+        self._table = Gtk.Table(n_rows=3, n_columns=1, homogeneous=False)
         self._table.attach(self.treeview_scrollwin, 0, 1, 0, 1)
         self._table.attach(self._toolbar, 0, 1, 2, 3, yoptions=Gtk.AttachOptions.FILL)
         self._vcontent.pack1(self._table, resize=True, shrink=False)
@@ -241,7 +243,7 @@ class EffectProperties(Gtk.Expander, Loggable):
         self.treeview.connect("drag-motion", self._dragMotionCb)
         self.treeview.connect("query-tooltip", self._treeViewQueryTooltipCb)
         self._vcontent.connect("notify", self._vcontentNotifyCb)
-        removeEffectButton.connect("clicked", self._removeEffectClicked)
+        removeEffectButton.connect("clicked", self._removeEffectCb)
         self.app.connect("new-project-loaded", self._newProjectLoadedCb)
         self.connect('notify::expanded', self._expandedCb)
         self.connected = False
@@ -306,26 +308,23 @@ class EffectProperties(Gtk.Expander, Loggable):
     def _connectTimelineSelection(self, timeline):
         self.timeline = timeline
 
-    def _removeEffectClicked(self, toolbutton):
+    def _removeEffectCb(self, toolbutton):
         if not self.selection.get_selected()[1]:
+            # Cannot remove nothing,
             return
-        else:
-            effect = self.storemodel.get_value(self.selection.get_selected()[1],
-                                               COL_TRACK_EFFECT)
-            self._removeEffect(effect)
+        effect = self.storemodel.get_value(self.selection.get_selected()[1],
+                                           COL_TRACK_EFFECT)
+        self._removeEffect(effect)
 
     def _removeEffect(self, effect):
         self.app.action_log.begin("remove effect")
         if self._effect_config_ui:
             self._effect_config_ui.get_children()[0].get_children()[0].resetKeyframeToggleButtons()
-        self._cleanCache(effect)
+        self.effects_properties_manager.cleanCache(effect)
         effect.get_parent().remove(effect)
         self._updateTreeview()
         self.app.action_log.commit()
 
-    def _cleanCache(self, effect):
-        config_ui = self.effect_props_handling.cleanCache(effect)
-
     def addEffectToClip(self, clip, bin_desc):
         media_type = self.app.effects.getFactoryFromName(bin_desc).media_type
 
@@ -451,10 +450,10 @@ class EffectProperties(Gtk.Expander, Loggable):
                     self._vcontent.remove(widget)
 
             element = effect
-            ui = self.effect_props_handling.getEffectConfigurationUI(element)
+            ui = self.effects_properties_manager.getEffectConfigurationUI(element)
 
             if self._effect_config_ui:
-                self._effect_config_ui.get_children()[0].get_children()[0].resetShowKeyframesButton()
+                self._effect_config_ui.get_children()[0].get_children()[0].resetKeyframeToggleButtons()
 
             self._effect_config_ui = ui
             if self._effect_config_ui:
@@ -488,14 +487,13 @@ class TransformationProperties(Gtk.Expander):
         self.default_values = {}
         self.set_label(_("Transformation"))
 
-        if not "Frei0r" in missing_soft_deps:
-            self.builder = Gtk.Builder()
-            self.builder.add_from_file(os.path.join(get_ui_dir(),
-                        "cliptransformation.ui"))
+        self.builder = Gtk.Builder()
+        self.builder.add_from_file(os.path.join(get_ui_dir(),
+                                                "cliptransformation.ui"))
 
-            self.add(self.builder.get_object("transform_box"))
-            self.show_all()
-            self._initButtons()
+        self.add(self.builder.get_object("transform_box"))
+        self.show_all()
+        self._initButtons()
         self.connect('notify::expanded', self._expandedCb)
         self.hide()
 
@@ -521,13 +519,12 @@ class TransformationProperties(Gtk.Expander):
         self.app.gui.viewer.setZoom(scale.get_value())
 
     def _expandedCb(self, expander, params):
-        if not "Frei0r" in missing_soft_deps:
-            if self._selected_clip:
-                self.effect = self._findOrCreateEffect("frei0r-filter-scale0tilt")
-                self._updateSpinButtons()
-                self.set_expanded(self.get_expanded())
-                self._updateBoxVisibility()
-                self.zoom_scale.set_value(1.0)
+        if self._selected_clip:
+            self.effect = self._findOrCreateEffect("frei0r-filter-scale0tilt")
+            self._updateSpinButtons()
+            self.set_expanded(self.get_expanded())
+            self._updateBoxVisibility()
+            self.zoom_scale.set_value(1.0)
         else:
             if self.get_expanded():
                 DepsManager(self.app)
