@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       ui/clipmediaprops.py
+#       pitivi/dialogs/clipmediaprops.py
 #
 # Copyright (c) 2011, Parthasarathi Susarla <partha@collabora.co.uk>
 #
@@ -15,35 +15,64 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""
-Dialog box displaying the properties of a clip from media library, allowing
-to set those properties as the project settings.
-"""
-
-from gi.repository import Gtk
-from gi.repository import Gst
 import os
 
+from gi.repository import Gdk
+from gi.repository import Gst
+from gi.repository import Gtk
+
 from gettext import gettext as _
+
 from pitivi.configure import get_ui_dir
-from pitivi.utils.ui import frame_rates, audio_rates,\
+from pitivi.utils.ui import frame_rates, audio_rates, \
     audio_channels, pixel_aspect_ratios, get_value_from_model
 
 
-class clipmediapropsDialog():
+class ClipMediaPropsDialog(object):
+    """
+    Displays the properties of an asset, and allows applying them to a project.
 
-    def __init__(self, project, audio_streams, video_streams):
+    @type project: L{Project}
+    @type asset: L{GES.UriClipAsset}
+    """
+
+    def __init__(self, project, asset):
         self.project = project
-        self.audio_streams = audio_streams
-        self.video_streams = video_streams
-        self.has_audio = self.has_video = self.is_image = False
-        self._createUi()
+        info = asset.get_info()
+        self.audio_streams = info.get_audio_streams()
+        self.video_streams = info.get_video_streams()
+        self.has_audio = False
+        self.has_video = False
+        self.is_image = False
+
+        builder = Gtk.Builder()
+        builder.add_from_file(os.path.join(get_ui_dir(), "clipmediaprops.ui"))
+        builder.connect_signals(self)
+        self.dialog = builder.get_object("Import Settings")
+        # Checkbuttons (with their own labels) in the first table column:
+        self.size_checkbutton = builder.get_object("size_checkbutton")
+        self.framerate_checkbutton = builder.get_object("framerate_checkbutton")
+        self.PAR_checkbutton = builder.get_object("PAR_checkbutton")
+        self.channels_checkbutton = builder.get_object("channels_checkbutton")
+        self.samplerate_checkbutton = builder.get_object("samplerate_checkbtn")
+        # These labels are in a separate table col on the right of checkboxes:
+        self.channels = builder.get_object("channels")
+        self.size_height = builder.get_object("size_height")
+        self.size_width = builder.get_object("size_width")
+        self.frame_rate = builder.get_object("frame_rate")
+        self.aspect_ratio = builder.get_object("aspect_ratio")
+        self.sample_rate = builder.get_object("sample_rate")
+        # Various other layout widgets
+        self.frame1 = builder.get_object("frame1")
+        self.frame2 = builder.get_object("frame2")
+        self.hbox2 = builder.get_object("hbox2")
+        self.hbox3 = builder.get_object("hbox3")
+        self.video_header_label = builder.get_object("label2")
 
     def run(self):
+        """Set up widgets and run the dialog"""
         # TODO: in "onApplyButtonClicked", we only use the first stream...
         # If we have multiple audio or video streams, we should reflect that
         # in the UI, instead of acting as if there was only one. But that means
@@ -61,14 +90,30 @@ class clipmediapropsDialog():
             self.size_height.set_text(str(stream.get_height()))
             self.is_image = stream.is_image()
             if not self.is_image:
-                self.frame_rate.set_text(
-                    get_value_from_model(frame_rates, Gst.Fraction(
-                        stream.get_framerate_num(),
-                        stream.get_framerate_denom())))
+                # When gst returns a crazy framerate such as 0/1, that either
+                # means it couldn't determine it, or it is a variable framerate
+                framerate_num = stream.get_framerate_num()
+                framerate_denom = stream.get_framerate_denom()
+                if framerate_num != 0 and framerate_denom != 0:
+                    self.frame_rate.set_text(
+                        get_value_from_model(frame_rates,
+                            Gst.Fraction(framerate_num, framerate_denom)
+                        ))
+                else:
+                    foo = str(framerate_num) + "/" + str(framerate_denom)
+                    # Translators: a label showing an invalid framerate value
+                    self.frame_rate.set_text(_("invalid (%s fps)" % foo))
+                    self.framerate_checkbutton.set_active(False)
+                    # For consistency, insensitize the checkbox AND value labels
+                    self.framerate_checkbutton.set_sensitive(False)
+                    self.frame_rate.set_sensitive(False)
+
+                # Aspect ratio (probably?) doesn't need such a check:
                 self.aspect_ratio.set_text(
                     get_value_from_model(pixel_aspect_ratios, Gst.Fraction(
                         stream.get_par_num(),
                         stream.get_par_denom())))
+
             self.has_video = True
             break
 
@@ -79,53 +124,39 @@ class clipmediapropsDialog():
         if self.is_image:
             self.hbox2.hide()
             self.hbox3.hide()
-            self.label2.set_markup("<b>" + _("Image:") + "</b>")
+            self.video_header_label.set_markup("<b>" + _("Image:") + "</b>")
+
+        self.dialog.connect("key-press-event", self._keyPressCb)
         self.dialog.run()
 
-    def _createUi(self):
-        builder = Gtk.Builder()
-        builder.add_from_file(os.path.join(get_ui_dir(), "clipmediaprops.ui"))
-        builder.connect_signals(self)
-        self.dialog = builder.get_object("Import Settings")
-        self.channels = builder.get_object("channels")
-        self.size_height = builder.get_object("size_height")
-        self.size_width = builder.get_object("size_width")
-        self.frame_rate = builder.get_object("frame_rate")
-        self.aspect_ratio = builder.get_object("aspect_ratio")
-        self.sample_rate = builder.get_object("sample_rate")
-        self.frame1 = builder.get_object("frame1")
-        self.frame2 = builder.get_object("frame2")
-        self.hbox2 = builder.get_object("hbox2")
-        self.hbox3 = builder.get_object("hbox3")
-        self.label2 = builder.get_object("label2")
-        self.checkbutton1 = builder.get_object("checkbutton1")
-        self.checkbutton2 = builder.get_object("checkbutton2")
-        self.checkbutton3 = builder.get_object("checkbutton3")
-        self.checkbutton4 = builder.get_object("checkbutton4")
-        self.checkbutton5 = builder.get_object("checkbutton5")
-        self.checkbutton6 = builder.get_object("checkbutton6")
-
     def _applyButtonCb(self, unused_button):
+        """Apply widget values to the project"""
         project = self.project
         if self.has_video:
             # This also handles the case where the video is a still image
             video = self.video_streams[0]
-            if self.checkbutton1.get_active():
+            if self.size_checkbutton.get_active():
                 project.videowidth = video.get_width()
                 project.videoheight = video.get_height()
-            if (self.checkbutton2.get_active() and not self.is_image):
+            if self.framerate_checkbutton.get_active() and not self.is_image:
                 project.videorate = Gst.Fraction(video.get_framerate_num(),
                                                  video.get_framerate_denom())
-            if (self.checkbutton3.get_active() and not self.is_image):
+            if self.PAR_checkbutton.get_active() and not self.is_image:
                 project.videopar = Gst.Fraction(video.get_par_num(),
                                                 video.get_par_denom())
         if self.has_audio:
             audio = self.audio_streams[0]
-            if self.checkbutton4.get_active():
+            if self.channels_checkbutton.get_active():
                 project.audiochannels = audio.get_channels()
-            if self.checkbutton5.get_active():
+            if self.samplerate_checkbutton.get_active():
                 project.audiorate = audio.get_sample_rate()
         self.dialog.destroy()
 
     def _cancelButtonCb(self, unused_button):
+        """Destroy the dialog"""
         self.dialog.destroy()
+
+    def _keyPressCb(self, unused_widget, event):
+        if event.keyval in (Gdk.KEY_Escape, Gdk.KEY_Q, Gdk.KEY_q):
+            self.dialog.destroy()
+        return True
