@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       ui/prefs.py
+#       pitivi/dialogs/prefs.py
 #
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
@@ -46,8 +46,10 @@ GlobalSettings.addConfigOption('prefsDialogHeight',
     default=400)
 
 
-class PreferencesDialog():
-
+class PreferencesDialog(object):
+    """
+    This dialog displays preferences for pitivi.
+    """
     prefs = {}
     original_values = {}
 
@@ -56,8 +58,21 @@ class PreferencesDialog():
         self.widgets = {}
         self.resets = {}
         self._current = None
-        self._createUi()
-        self._fillContents()
+
+        # Identify the widgets we'll need
+        builder = Gtk.Builder()
+        builder.add_from_file(os.path.join(get_ui_dir(), "preferences.ui"))
+        builder.connect_signals(self)
+        self.dialog = builder.get_object("dialog1")
+        self.model = builder.get_object("liststore1")
+        self.treeview = builder.get_object("treeview1")
+        self.contents = builder.get_object("box1")
+        self.revert_button = builder.get_object("revertButton")
+        self.factory_settings = builder.get_object("resetButton")
+        self.restart_warning = builder.get_object("restartWarning")
+
+        self.sections = {}
+        self.__fillContents()
         req = self.contents.size_request()
         min_width = req.width
         min_height = req.height
@@ -67,27 +82,14 @@ class PreferencesDialog():
         self.dialog.set_default_size(width, height)
 
     def run(self):
+        """Run the internal dialog"""
         self.dialog.run()
 
-    def _createUi(self):
-        builder = Gtk.Builder()
-        builder.add_from_file(os.path.join(get_ui_dir(), "preferences.ui"))
-        builder.connect_signals(self)
-
-        # widgets we'll need
-        self.dialog = builder.get_object("dialog1")
-        self.model = builder.get_object("liststore1")
-        self.treeview = builder.get_object("treeview1")
-        self.contents = builder.get_object("box1")
-        self.revert_button = builder.get_object("revertButton")
-        self.factory_settings = builder.get_object("resetButton")
-        self.restart_warning = builder.get_object("restartWarning")
-
 ## Public API
 
     @classmethod
     def addPreference(cls, attrname, label, description, section=None,
-                    widget_klass=None, **args):
+                    widget_class=None, **args):
         """
         Add a user preference. The preferences dialog will try
         to guess the appropriate widget to use based on the type of the
@@ -95,20 +97,20 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param : user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
-        @param widget_klass: overrides auto-detected widget
-        @type widget_klass: C{class}
+        @param widget_class: overrides auto-detected widget
+        @type widget_class: C{class}
         """
         if not section:
             section = "General"
         if not section in cls.prefs:
             cls.prefs[section] = {}
-        cls.prefs[section][attrname] = (label, description, widget_klass, args)
+        cls.prefs[section][attrname] = (label, description, widget_class, args)
 
     @classmethod
     def addPathPreference(cls, attrname, label, description, section=None):
@@ -118,9 +120,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param section: user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
@@ -138,9 +140,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param section: user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
@@ -161,9 +163,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param section: user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
@@ -180,9 +182,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param choices: a sequence of (<label>, <value>) pairs
         @type choices: C{[(str, pyobject), ...]}
         @param section: user-visible category to which this option
@@ -200,9 +202,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param section: user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
@@ -220,9 +222,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param section: user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
@@ -238,9 +240,9 @@ class PreferencesDialog():
 
         @param label: user-visible name for this option
         @type label: C{str}
-        @param desc: a user-visible description documenting this option
+        @param description: a user-visible description documenting this option
         (ignored unless prefs_label is non-null)
-        @type desc: C{str}
+        @type description: C{str}
         @param section: user-visible category to which this option
         belongs (ignored unless prefs_label is non-null)
         @type section: C{str}
@@ -249,8 +251,7 @@ class PreferencesDialog():
             ptvWidgets.FontWidget)
 
 ## Implementation
-    def _fillContents(self):
-        self.sections = {}
+    def __fillContents(self):
         for section in sorted(self.prefs):
             options = self.prefs[section]
             self.model.append((_(section), section))
@@ -262,8 +263,8 @@ class PreferencesDialog():
 
             prefs = {}
             for attrname in options:
-                label, description, klass, args = options[attrname]
-                widget = klass(**args)
+                label, description, widget_class, args = options[attrname]
+                widget = widget_class(**args)
                 widget.setWidgetValue(getattr(self.settings, attrname))
                 widget.connectValueChanged(self._valueChanged, widget, attrname)
                 self.widgets[attrname] = widget
@@ -312,8 +313,9 @@ class PreferencesDialog():
         self.factory_settings.set_sensitive(self._canReset())
 
     def _treeSelectionChangedCb(self, selection):
-        model, iter = selection.get_selected()
-        new = self.sections[model[iter][1]]
+        """ Update current when selection changed"""
+        model, _iter = selection.get_selected()
+        new = self.sections[model[_iter][1]]
         if self._current != new:
             if self._current:
                 self._current.hide()
@@ -321,6 +323,8 @@ class PreferencesDialog():
             self._current = new
 
     def _clearHistory(self):
+        # Disable missing docstring
+        #pylint: disable=C0111
         self.original_values = {}
         self.revert_button.set_sensitive(False)
 
@@ -354,10 +358,14 @@ class PreferencesDialog():
         self.factory_settings.set_sensitive(self._canReset())
 
     def _acceptButtonCb(self, unused_button):
+        # Disable missing docstring
+        #pylint: disable=C0111
         self._clearHistory()
         self.dialog.hide()
 
-    def _valueChanged(self, fake_widget, real_widget, attrname):
+    def _valueChanged(self, unused_fake_widget, real_widget, attrname):
+        # Disable missing docstring
+        #pylint: disable=C0111
         value = getattr(self.settings, attrname)
         if attrname not in self.original_values:
             self.original_values[attrname] = value
@@ -375,98 +383,16 @@ class PreferencesDialog():
         self.factory_settings.set_sensitive(True)
 
     def _configureCb(self, unused_widget, event):
+        # Disable missing docstring
+        #pylint: disable=C0111
         self.settings.prefsDialogWidth = event.width
         self.settings.prefsDialogHeight = event.height
 
     def _canReset(self):
+        # Disable missing docstring
+        #pylint: disable=C0111
         for section in self.prefs.itervalues():
             for attrname in section:
                 if not self.settings.isDefault(attrname):
                     return True
         return False
-
-## Preference Test Cases
-
-if False:
-
-    from pitivi.settings import GlobalSettings
-
-    options = (
-        ('numericPreference1', 10),
-        ('numericPreference2', 2.4),
-        ('textPreference1', "banana"),
-        ('textPreference2', "42"),
-        ('aPathPreference', "file:///etc/"),
-        ('aChoicePreference', 42),
-        ('aLongChoicePreference', "Mauve"),
-        ('aTogglePreference', True),
-        ('aFontPreference', "Sans 9"),
-    )
-
-    for attrname, default in options:
-        GlobalSettings.addConfigOption(attrname, default=default)
-
-## Numeric
-
-    PreferencesDialog.addNumericPreference('numericPreference1',
-        label="Open Range",
-        section="Test",
-        description="This option has no upper bound",
-        lower=-10)
-
-    PreferencesDialog.addNumericPreference('numericPreference2',
-        label="Closed Range",
-        section="Test",
-        description="This option has both upper and lower bounds",
-        lower=-10,
-        upper=10000)
-
-## Text
-
-    PreferencesDialog.addTextPreference('textPreference1',
-        label="Unfiltered",
-        section="Test",
-        description="Anything can go in this box")
-
-    PreferencesDialog.addTextPreference('textPreference2',
-        label="Numbers only",
-        section="Test",
-        description="This input validates its input with a regex",
-        matches="^-?\d+(\.\d+)?$")
-
-## other
-    PreferencesDialog.addPathPreference('aPathPreference',
-        label="Test Path",
-        section="Test",
-        description="Test the path widget")
-
-    PreferencesDialog.addChoicePreference('aChoicePreference',
-        label="Swallow Velocity",
-        section="Test",
-        description="What is the airspeed velocity of a coconut-laden swallow?",
-        choices=(
-            ("42 Knots", 32),
-            ("9 furlongs per fortnight", 42),
-            ("I don't know that!", None)))
-
-    PreferencesDialog.addChoicePreference('aLongChoicePreference',
-        label="Favorite Color",
-        section="Test",
-        description="What is the color of the parrot's plumage?",
-        choices=(
-            ("Mauve", "Mauve"),
-            ("Chartreuse", "Chartreuse"),
-            ("Magenta", "Magenta"),
-            ("Pink", "Pink"),
-            ("Norwegian Blue", "Norwegian Blue"),
-            ("Yellow Ochre", "Yellow Ochre")))
-
-    PreferencesDialog.addTogglePreference('aTogglePreference',
-        label="Test Toggle",
-        section="Test",
-        description="Test the toggle widget")
-
-    PreferencesDialog.addFontPreference('aFontPreference',
-        label="Foo Font",
-        section="Test",
-        description="Test the font widget")
