@@ -1,3 +1,4 @@
+""" This module implements a startup wizard"""
 # Pitivi video editor
 #
 #       pitivi/dialogs/startupwizard.py
@@ -44,6 +45,16 @@ class StartUpWizard(object):
     - see the quick start manual (User Manual button).
     """
 
+    @staticmethod
+    def _userManualCb(unused_button):
+        """Handle a click on the Help button."""
+        show_user_manual()
+
+    @staticmethod
+    def _cheatsheetCb(unused_button):
+        """Show the cheatsheet section of the user manual"""
+        show_user_manual("cheatsheet")
+
     def __init__(self, app):
         self.app = app
         self.builder = Gtk.Builder()
@@ -55,16 +66,18 @@ class StartUpWizard(object):
         self.recent_chooser = self.builder.get_object("recentchooser2")
         # FIXME: gtk creates a combo box with only one item, but there is no
         # simple way to hide it.
-        filter = Gtk.RecentFilter()
-        filter.set_name(_("Projects"))
+        _filter = Gtk.RecentFilter()
+        _filter.set_name(_("Projects"))
 
         for asset in GES.list_assets(GES.Formatter):
-            filter.add_pattern('*.' + asset.get_meta(GES.META_FORMATTER_EXTENSION))
+            _filter.add_pattern('*.' + asset.get_meta(GES.META_FORMATTER_EXTENSION))
 
-        self.recent_chooser.add_filter(filter)
+        self.recent_chooser.add_filter(_filter)
+
+        missing_button = self.builder.get_object("missing_deps_button")
 
         if not missing_soft_deps:
-            self.builder.get_object("missing_deps_button").hide()
+            missing_button.hide()
 
         self.app.projectManager.connect("new-project-failed", self._projectFailedCb)
         self.app.projectManager.connect("new-project-loaded", self._projectLoadedCb)
@@ -73,8 +86,8 @@ class StartUpWizard(object):
         vbox = self.builder.get_object("topvbox")
         self.infobar = Gtk.InfoBar()
         vbox.pack_start(self.infobar, True, True, 0)
-        if self.app.version_information:
-            self._appVersionInfoReceivedCb(None, self.app.version_information)
+        if self.app.getLatest():
+            self._appVersionInfoReceivedCb(self.app, None)
         else:
             self.app.connect("version-info-received", self._appVersionInfoReceivedCb)
 
@@ -91,7 +104,7 @@ class StartUpWizard(object):
         uri = self.recent_chooser.get_current_uri()
         self.app.projectManager.loadProject(uri)
 
-    def _keyPressCb(self, widget, event):
+    def _keyPressCb(self, unused_widget, event):
         """Handle a key press event on the dialog."""
         if event.keyval == Gdk.KEY_Escape:
             # The user pressed "Esc".
@@ -102,26 +115,21 @@ class StartUpWizard(object):
         self.app.gui.openProject()
 
     def _onMissingDepsButtonClickedCb(self, unused_button):
+        """Handle a click on the Missing Deps button."""
         DepsManager(self.app, parent_window=self.window)
 
-    def _userManualCb(self, unused_button):
-        """Handle a click on the Help button."""
-        show_user_manual()
-
-    def _cheatsheetCb(self, unused_button):
-        """Show the cheatsheet section of the user manual"""
-        show_user_manual("cheatsheet")
-
-    def _deleteCb(self, unused_widget, event):
+    def _deleteCb(self, unused_widget, unused_event):
         """Handle a click on the X button of the dialog."""
         self.app.projectManager.newBlankProject()
 
     def show(self):
+        """Will show the interal window and position the wizard"""
         self.window.set_transient_for(self.app.gui)
         self.window.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
         self.window.show()
 
     def hide(self):
+        """Will hide the internal window"""
         self.window.hide()
 
     def _projectFailedCb(self, unused_project_manager, unused_uri,
@@ -129,7 +137,7 @@ class StartUpWizard(object):
         """Handle the failure of a project open operation."""
         self.show()
 
-    def _projectLoadedCb(self, unused_project_manager, project, fully_loaded):
+    def _projectLoadedCb(self, unused_project_manager, unused_project, fully_loaded):
         """Handle the success of a project load operation.
 
         All the create or load project usage scenarios must generate
@@ -144,23 +152,25 @@ class StartUpWizard(object):
         """Handle the start of a project load operation."""
         self.hide()
 
-    def _appVersionInfoReceivedCb(self, pitivi, version):
-        # current version, don't show message
-        if version["status"].upper() == "CURRENT":
+    def _appVersionInfoReceivedCb(self, app, unused_version_information):
+        """Handle version info"""
+        if app.isLatest():
+            # current version, don't show message
             return
 
-        # new current version, reset counter
-        if self.app.settings.lastCurrentVersion != version["current"]:
-            self.app.settings.lastCurrentVersion = version["current"]
+        latest_version = app.getLatest()
+        if self.app.settings.lastCurrentVersion != latest_version:
+            # new latest version, reset counter
+            self.app.settings.lastCurrentVersion = latest_version
             self.app.settings.displayCounter = 0
 
-        # current version info already showed 5 times, don't show again
         if self.app.settings.displayCounter >= 5:
+            # current version info already showed 5 times, don't show again
             return
 
         # increment counter, create infobar and show info
         self.app.settings.displayCounter = self.app.settings.displayCounter + 1
-        text = _("Pitivi %s is available." % version["current"])
+        text = _("Pitivi %s is available." % latest_version)
         label = Gtk.Label(label=text)
         self.infobar.get_content_area().add(label)
         self.infobar.set_message_type(Gtk.MessageType.INFO)
