@@ -1,5 +1,26 @@
 # -*- coding: utf-8 -*-
-import platform
+# Pitivi video editor
+#
+#       pitivi/mediafilespreviewer.py
+#
+# Copyright (c) 2011, Pier Carteri <pier.carteri@gmail.com>
+# Copyright (c) 2012, Thibault Saunier <tsaunier@gnome.org>
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 2.1 of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this program; if not, write to the
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
+
 from gettext import gettext as _
 from gi.repository import GLib
 from gi.repository import GObject
@@ -8,12 +29,12 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Pango
-from gi.repository.GstPbutils import Discoverer
+from gi.repository import GstPbutils
 
-from pitivi.configure import get_pixmap_dir
 from pitivi.settings import GlobalSettings
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import uri_is_valid
+from pitivi.utils.pipeline import AssetPipeline
 from pitivi.utils.ui import beautify_length, beautify_stream, SPACING
 from pitivi.viewer import ViewerWidget
 
@@ -47,32 +68,35 @@ acceptable_tags = [
     Gst.TAG_COPYRIGHT]
 
 
-class PreviewWidget(Gtk.VBox, Loggable):
+class PreviewWidget(Gtk.Grid, Loggable):
+    """
+    Widget for displaying a GStreamer sink with playback controls.
 
-    def __init__(self, instance):
-        Gtk.VBox.__init__(self)
+    @ivar settings: The settings of the app.
+    @type settings: L{GlobalSettings}
+    """
+
+    def __init__(self, settings, minimal=False):
+        Gtk.Grid.__init__(self)
         Loggable.__init__(self)
 
         self.log("Init PreviewWidget")
         self.connect('destroy', self._destroy_cb)
 
-        self.settings = instance.settings
+        self.settings = settings
         self.preview_cache = {}
         self.preview_cache_errors = {}
 
-        self.discoverer = Discoverer.new(Gst.SECOND)
+        self.discoverer = GstPbutils.Discoverer.new(Gst.SECOND)
 
         #playbin for play pics
-        self.player = Gst.ElementFactory.make("playbin", "preview-player")
-        bus = self.player.get_bus()
-        bus.add_signal_watch()
-        bus.connect('message', self._bus_message_cb)
-        bus.connect('message::tag', self._tag_found_cb)
-        self.__fakesink = Gst.ElementFactory.make("fakesink", "fakesink")
+        self.player = AssetPipeline(clip=None, name="preview-player")
+        self.player.connect('eos', self._pipelineEosCb)
+        self.player.connect('error', self._pipelineErrorCb)
+        self.player._bus.connect('message::tag', self._tag_found_cb)
 
         #some global variables for preview handling
         self.is_playing = False
-        self.time_format = Gst.Format(Gst.Format.TIME)
         self.original_dims = (PREVIEW_WIDTH, PREVIEW_HEIGHT)
         self.countinuous_seek = False
         self.slider_being_used = False
@@ -83,23 +107,23 @@ class PreviewWidget(Gtk.VBox, Loggable):
 
         # Gui elements:
         # Drawing area for video output
-        self.preview_video = ViewerWidget()
-        self.preview_video.connect("realize", self._on_preview_video_realize_cb)
-        self.preview_video.modify_bg(Gtk.StateType.NORMAL, self.preview_video.get_style().black)
-        self.preview_video.set_double_buffered(False)
-        self.pack_start(self.preview_video, False, True, 0)
+        self.preview_video = ViewerWidget(realizedCb=self._on_preview_video_realize_cb)
+        self.preview_video.props.hexpand = minimal
+        self.preview_video.props.vexpand = minimal
+        self.attach(self.preview_video, 0, 0, 1, 1)
 
         # An image for images and audio
         self.preview_image = Gtk.Image()
         self.preview_image.set_size_request(self.settings.FCpreviewWidth, self.settings.FCpreviewHeight)
         self.preview_image.show()
-        self.pack_start(self.preview_image, False, True, 0)
+        self.attach(self.preview_image, 0, 1, 1, 1)
 
         # Play button
         self.bbox = Gtk.HBox()
-        self.play_button = Gtk.ToolButton(Gtk.STOCK_MEDIA_PLAY)
+        self.play_button = Gtk.ToolButton()
+        self.play_button.set_icon_name("media-playback-start")
         self.play_button.connect("clicked", self._on_start_stop_clicked_cb)
-        self.bbox.pack_start(self.play_button, False, True, 0)
+        self.bbox.pack_start(self.play_button, False, False, 0)
 
         #Scale for position handling
         self.pos_adj = Gtk.Adjustment()
@@ -112,39 +136,39 @@ class PreviewWidget(Gtk.VBox, Loggable):
         self.bbox.pack_start(self.seeker, True, True, 0)
 
         # Zoom buttons
-        self.b_zoom_in = Gtk.ToolButton(Gtk.STOCK_ZOOM_IN)
+        self.b_zoom_in = Gtk.ToolButton()
+        self.b_zoom_in.set_icon_name("zoom-in")
         self.b_zoom_in.connect("clicked", self._on_zoom_clicked_cb, 1)
-        self.b_zoom_out = Gtk.ToolButton(Gtk.STOCK_ZOOM_OUT)
+        self.b_zoom_out = Gtk.ToolButton()
+        self.b_zoom_out.set_icon_name("zoom-out")
         self.b_zoom_out.connect("clicked", self._on_zoom_clicked_cb, -1)
         self.bbox.pack_start(self.b_zoom_in, False, True, 0)
         self.bbox.pack_start(self.b_zoom_out, False, True, 0)
         self.bbox.show_all()
-        self.pack_start(self.bbox, False, False, 0)
+        self.attach(self.bbox, 0, 2, 1, 1)
 
         # Label for metadata tags
         self.l_tags = Gtk.Label()
         self.l_tags.set_justify(Gtk.Justification.LEFT)
         self.l_tags.set_ellipsize(Pango.EllipsizeMode.END)
         self.l_tags.show()
-        self.pack_start(self.l_tags, False, False, 0)
+        self.attach(self.l_tags, 0, 3, 1, 1)
 
         # Error handling
         vbox = Gtk.VBox()
         vbox.set_spacing(SPACING)
         self.l_error = Gtk.Label(label=_("Pitivi can not preview this file."))
-        self.b_details = Gtk.Button(_("More info"))
+        self.b_details = Gtk.Button.new_with_label(_("More info"))
         self.b_details.connect('clicked', self._on_b_details_clicked_cb)
         vbox.pack_start(self.l_error, True, True, 0)
         vbox.pack_start(self.b_details, False, False, 0)
         vbox.show()
-        self.pack_start(vbox, False, False, 0)
+        self.attach(vbox, 0, 4, 1, 1)
 
-    def setMinimal(self):
-        self.remove(self.l_tags)
-        self.b_zoom_in.hide()
-        self.b_zoom_out.hide()
-        # Allow expanding/filling and pack the video preview below the controls
-        self.set_child_packing(self.preview_video, True, True, 0, Gtk.PackType.END)
+        if minimal:
+            self.remove(self.l_tags)
+            self.bbox.remove(self.b_zoom_in)
+            self.bbox.remove(self.b_zoom_out)
 
     def add_preview_request(self, dialogbox):
         """add a preview request """
@@ -169,17 +193,15 @@ class PreviewWidget(Gtk.VBox, Loggable):
             try:
                 info = self.discoverer.discover_uri(uri)
             except Exception, e:
-                if e is not None:
-                    self.preview_cache_errors[uri] = e
-                    if self.current_selected_uri == uri:
-                        self.show_error(uri)
-                    return
+                self.preview_cache_errors[uri] = e
+                if self.current_selected_uri == uri:
+                    self.show_error(uri)
+                return
 
             if self.current_selected_uri == uri:
                 self.show_preview(uri, info)
 
     def show_preview(self, uri, info):
-
         if info:
             self.preview_cache[uri] = info
         else:
@@ -215,21 +237,24 @@ class PreviewWidget(Gtk.VBox, Loggable):
             else:
                 self.current_preview_type = 'video'
                 self.preview_image.hide()
-                self.player.set_property("uri", self.current_selected_uri)
-                self.player.set_state(Gst.State.PAUSED)
+                self.player.setClipUri(self.current_selected_uri)
+                self.player.setState(Gst.State.PAUSED)
                 self.pos_adj.props.upper = duration
-                w, h = self.__get_best_size((video.get_par_num() / video.get_par_denom()) * video.get_width(),
-                    video.get_height())
+                video_width = (video.get_par_num() / video.get_par_denom()) * video.get_width()
+                video_height = video.get_height()
+                w, h = self.__get_best_size(video_width, video_height)
                 self.preview_video.set_size_request(w, h)
+                self.preview_video.setDisplayAspectRatio(float(video_width) / video_height)
                 self.preview_video.show()
+                self.player.connectWithViewer(self.preview_video)
                 self.bbox.show()
                 self.play_button.show()
                 self.seeker.show()
                 self.b_zoom_in.show()
                 self.b_zoom_out.show()
-                self.description = _("<b>Resolution</b>: %d×%d") % \
-                    ((video.get_par_num() / video.get_par_denom()) * video.get_width(), video.get_height()) +\
-                    "\n" + _("<b>Duration</b>: %s") % pretty_duration + "\n"
+                self.description = "\n".join([
+                    _("<b>Resolution</b>: %d×%d") % (video_width, video_height),
+                    _("<b>Duration</b>: %s") % pretty_duration])
         else:
             self.current_preview_type = 'audio'
             self.preview_video.hide()
@@ -243,37 +268,43 @@ class PreviewWidget(Gtk.VBox, Loggable):
             self.preview_image.set_from_icon_name("audio-x-generic", Gtk.IconSize.DIALOG)
             self.preview_image.show()
             self.preview_image.set_size_request(PREVIEW_WIDTH, PREVIEW_HEIGHT)
-            self.description = beautify_stream(audio) + "\n" + \
-                _("<b>Duration</b>: %s") % pretty_duration + "\n"
-            self.player.set_state(Gst.State.NULL)
-            self.player.set_property("uri", self.current_selected_uri)
-            self.player.set_property("video-sink", self.__fakesink)
-            self.player.set_state(Gst.State.PAUSED)
+            self.description = "\n".join([
+                beautify_stream(audio),
+                _("<b>Duration</b>: %s") % pretty_duration])
+            self.player.setState(Gst.State.NULL)
+            self.player.setClipUri(self.current_selected_uri)
+            self.player.setState(Gst.State.PAUSED)
             self.play_button.show()
             self.seeker.show()
             self.b_zoom_in.hide()
             self.b_zoom_out.hide()
             self.bbox.show()
 
-    def show_error(self, uri):
+    def show_error(self, unused_uri):
         self.l_error.show()
         self.b_details.show()
 
     def play(self):
-        self.player.set_state(Gst.State.PLAYING)
+        self.player.setState(Gst.State.PLAYING)
         self.is_playing = True
         self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PAUSE)
         GLib.timeout_add(250, self._update_position)
         self.debug("Preview started")
 
     def pause(self):
-        self.player.set_state(Gst.State.PAUSED)
+        self.player.setState(Gst.State.PAUSED)
         self.is_playing = False
         self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
         self.log("Preview paused")
 
+    def togglePlayback(self):
+        if self.is_playing:
+            self.pause()
+        else:
+            self.play()
+
     def clear_preview(self):
-        self.log("Reset PreviewWidget ")
+        self.log("Reset PreviewWidget")
         self.seeker.set_value(0)
         self.bbox.hide()
         self.l_error.hide()
@@ -281,7 +312,7 @@ class PreviewWidget(Gtk.VBox, Loggable):
         self.description = ""
         self.l_tags.set_markup("")
         self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
-        self.player.set_state(Gst.State.NULL)
+        self.player.setState(Gst.State.NULL)
         self.is_playing = False
         self.tags = {}
         self.current_selected_uri = ""
@@ -294,45 +325,40 @@ class PreviewWidget(Gtk.VBox, Loggable):
         if event.type == Gdk.EventType.BUTTON_PRESS:
             self.countinuous_seek = True
             if self.is_playing:
-                self.player.set_state(Gst.State.PAUSED)
+                self.player.setState(Gst.State.PAUSED)
         elif event.type == Gdk.EventType.BUTTON_RELEASE:
             self.countinuous_seek = False
             value = long(widget.get_value())
-            self.player.seek_simple(self.time_format, Gst.SeekFlags.FLUSH, value)
+            self.player.simple_seek(value)
             if self.is_playing:
-                self.player.set_state(Gst.State.PLAYING)
+                self.player.setState(Gst.State.PLAYING)
             # Now, allow gobject timeout to continue updating the slider pos:
             self.slider_being_used = False
 
     def _on_motion_notify_cb(self, widget, event):
         if self.countinuous_seek:
             value = long(widget.get_value())
-            self.player.seek_simple(self.time_format, Gst.SeekFlags.FLUSH, value)
+            self.player.simple_seek(value)
 
-    def _bus_message_cb(self, bus, message):
-        if message.type == Gst.MessageType.EOS:
-            self.player.set_state(Gst.State.NULL)
-            self.is_playing = False
-            self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
-            self.pos_adj.set_value(0)
-        elif message.type == Gst.MessageType.ERROR:
-            self.player.set_state(Gst.State.NULL)
-            self.is_playing = False
-            err, dbg = message.parse_error()
-            self.error("Error: %s %s" % (err, dbg))
+    def _pipelineEosCb(self, unused_pipeline):
+        self.player.setState(Gst.State.NULL)
+        self.is_playing = False
+        self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
+        self.pos_adj.set_value(0)
 
-    def _update_position(self, *args):
+    def _pipelineErrorCb(self, unused_message, unused_detail):
+        self.player.setState(Gst.State.NULL)
+        self.is_playing = False
+
+    def _update_position(self, *unused_args):
         if self.is_playing and not self.slider_being_used:
-            curr_pos = self.player.query_position(self.time_format)[1]
+            curr_pos = self.player.getPosition()
             self.pos_adj.set_value(long(curr_pos))
         return self.is_playing
 
-    def _on_preview_video_realize_cb(self, widget):
-        if platform.system() == 'Windows':
-            xid = widget.get_window().get_handle()
-        else:
-            xid = widget.get_window().get_xid()
-        self.player.set_window_handle(xid)
+    def _on_preview_video_realize_cb(self, unused_drawing_area, unused_widget):
+        if self.current_preview_type == 'video':
+            self.player.connectWithViewer(self.preview_video)
 
     def _on_start_stop_clicked_cb(self, button):
         if self.is_playing:
@@ -399,7 +425,7 @@ class PreviewWidget(Gtk.VBox, Loggable):
         tag_list.foreach(self._appendTag, None)
         keys = self.tags.keys()
         keys.sort()
-        text = self.description + "\n"
+        text = self.description + "\n\n"
         for key in keys:
             text = text + "<b>" + key.capitalize() + "</b>: " + self.tags[key] + "\n"
         self.l_tags.set_markup(text)
@@ -407,22 +433,19 @@ class PreviewWidget(Gtk.VBox, Loggable):
     def _on_b_details_clicked_cb(self, unused_button):
         mess = self.preview_cache_errors.get(self.current_selected_uri, None)
         if mess is not None:
-            dialog = Gtk.MessageDialog(None,
-                Gtk.DialogFlags.MODAL,
-                Gtk.MessageType.WARNING,
-                Gtk.ButtonsType.OK,
-                str(mess))
+            dialog = Gtk.MessageDialog(transient_for=None,
+                modal=True,
+                message_type=Gtk.MessageType.WARNING,
+                buttons=Gtk.ButtonsType.OK,
+                text=str(mess))
             dialog.set_icon_name("pitivi")
             dialog.set_title(_("Error while analyzing a file"))
             dialog.run()
             dialog.destroy()
 
     def _destroy_cb(self, widget):
-        self.player.set_state(Gst.State.NULL)
+        self.player.setState(Gst.State.NULL)
         self.is_playing = False
-        #FIXME: are the following lines really needed?
-        del self.player
-        del self.preview_cache
 
     def __get_best_size(self, width_in, height_in):
         if width_in > height_in:
