@@ -24,6 +24,7 @@
 
 from gi.repository import Gst
 from gi.repository import GES
+from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
@@ -33,19 +34,22 @@ from gi.repository import GdkPixbuf
 
 import os
 import time
+import threading
 
 from urllib import unquote
-from gettext import gettext as _
+from gettext import ngettext, gettext as _
+from urlparse import urlparse
 from hashlib import md5
 from gi.repository.GstPbutils import DiscovererVideoInfo
 
+from pitivi.check import GNOMEDESKTOP_SOFT_DEPENDENCY
 from pitivi.configure import get_ui_dir, get_pixmap_dir
 from pitivi.settings import GlobalSettings
 from pitivi.mediafilespreviewer import PreviewWidget
 from pitivi.dialogs.filelisterrordialog import FileListErrorDialog
-from pitivi.dialogs.clipmediaprops import clipmediapropsDialog
+from pitivi.dialogs.clipmediaprops import ClipMediaPropsDialog
 from pitivi.utils.ui import beautify_length
-from pitivi.utils.misc import PathWalker, quote_uri
+from pitivi.utils.misc import PathWalker, quote_uri, path_from_uri
 from pitivi.utils.signal import SignalGroup
 from pitivi.utils.loggable import Loggable
 import pitivi.utils.ui as dnd
@@ -97,23 +101,14 @@ ui = '''
 # http://en.wikipedia.org/wiki/Comparison_of_container_formats and
 # http://en.wikipedia.org/wiki/List_of_file_formats#Video
 # ...and looking at the contents of /usr/share/mime
-SUPPORTED_FILE_FORMATS = {"video": ("3gpp", "3gpp2", "dv", "mp4", "mpeg", "ogg", "quicktime", "webm", "x-flv", "x-matroska", "x-mng", "x-ms-asf", "x-msvideo", "x-ms-wmp", "x-ms-wmv", "x-ogm+ogg", "x-theora+ogg"),
-    "application": ("mxf"),
+SUPPORTED_FILE_FORMATS = {"video": ("3gpp", "3gpp2", "dv", "mp2t", "mp4", "mpeg", "ogg", "quicktime", "webm", "x-flv", "x-matroska", "x-mng", "x-ms-asf", "x-msvideo", "x-ms-wmp", "x-ms-wmv", "x-ogm+ogg", "x-theora+ogg"),
+    "application": ("mxf",),
     # Don't forget audio formats
     "audio": ("aac", "ac3", "basic", "flac", "mp2", "mp4", "mpeg", "ogg", "opus", "webm", "x-adpcm", "x-aifc", "x-aiff", "x-aiffc", "x-ape", "x-flac+ogg", "x-m4b", "x-matroska", "x-ms-asx", "x-ms-wma", "x-speex", "x-speex+ogg", "x-vorbis+ogg", "x-wav"),
     # ...and image formats
     "image": ("jp2", "jpeg", "png", "svg+xml")}
 # Stuff that we're not too confident about but might improve eventually:
-OTHER_KNOWN_FORMATS = ("video/mp2t")
-
-
-def compare_simple(model, iter1, iter2, user_data):
-    a_basename = GLib.path_get_basename(model[iter1][user_data]).lower()
-    b_basename = GLib.path_get_basename(model[iter2][user_data]).lower()
-    if a_basename < b_basename:
-        return -1
-    # Each element is unique, there is a strict order.
-    return 1
+OTHER_KNOWN_FORMATS = ("video/mp2t",)
 
 
 class MediaLibraryWidget(Gtk.VBox, Loggable):
@@ -123,14 +118,15 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         'play': (GObject.SignalFlags.RUN_LAST, None,
                 (GObject.TYPE_PYOBJECT,))}
 
-    def __init__(self, instance, uiman):
+    def __init__(self, app, uiman):
         Gtk.VBox.__init__(self)
         Loggable.__init__(self)
 
         self.pending_rows = []
 
-        self.app = instance
+        self.app = app
         self._errors = []
+        self._missing_thumbs = []
         self._project = None
         self._draggedPaths = None
         self.dragged = False
@@ -144,7 +140,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         self._warning_label = builder.get_object("warning_label")
         self._view_error_button = builder.get_object("view_error_button")
         toolbar = builder.get_object("medialibrary_toolbar")
-        toolbar.get_style_context().add_class("inline-toolbar")
+        toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
         self._remove_button = builder.get_object("media_remove_button")
         self._clipprops_button = builder.get_object("media_props_button")
         self._insert_button = builder.get_object("media_insert_button")
@@ -153,7 +149,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
 
         # Store
         self.storemodel = Gtk.ListStore(*STORE_MODEL_STRUCTURE)
-        self.storemodel.set_sort_func(COL_URI, compare_simple, user_data=COL_URI)
+        self.storemodel.set_sort_func(COL_URI, MediaLibraryWidget.compare_basename)
         # Prefer to sort the media library elements by URI
         # rather than show them randomly.
         self.storemodel.set_sort_column_id(COL_URI, Gtk.SortType.ASCENDING)
@@ -179,7 +175,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
 
         # TreeView
         # Displays icon, name, type, length
-        self.treeview = Gtk.TreeView(self.modelFilter)
+        self.treeview = Gtk.TreeView(model=self.modelFilter)
         self.treeview_scrollwin.add(self.treeview)
         self.treeview.connect("button-press-event", self._treeViewButtonPressEventCb)
         self.treeview.connect("button-release-event", self._treeViewButtonReleaseEventCb)
@@ -220,7 +216,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         namecol.add_attribute(txtcell, "markup", COL_LENGTH)
 
         # IconView
-        self.iconview = Gtk.IconView(self.modelFilter)
+        self.iconview = Gtk.IconView(model=self.modelFilter)
         self.iconview_scrollwin.add(self.iconview)
         self.iconview.connect("button-press-event", self._iconViewButtonPressEventCb)
         self.iconview.connect("button-release-event", self._iconViewButtonReleaseEventCb)
@@ -281,7 +277,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
             ("InsertEnd", Gtk.STOCK_COPY, _("Insert at _End of Timeline"),
             "Insert", None, self._insertEndCb),
         )
-        self.selection_actions = Gtk.ActionGroup("medialibraryselection")
+        self.selection_actions = Gtk.ActionGroup(name="medialibraryselection")
         self.selection_actions.add_actions(selection_actions)
         self.selection_actions.set_sensitive(False)
         uiman.insert_action_group(self.selection_actions, 0)
@@ -300,6 +296,33 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         self.pack_start(self.treeview_scrollwin, True, True, 0)
         self.pack_start(self._progressbar, False, True, 0)
 
+        self.thumbnailer = MediaLibraryWidget._getThumbnailer()
+
+    @staticmethod
+    def _getThumbnailer():
+        if not GNOMEDESKTOP_SOFT_DEPENDENCY:
+            return None
+        from gi.repository import GnomeDesktop
+        # We need to instanciate the thumbnail factory on the main thread...
+        size_normal = GnomeDesktop.DesktopThumbnailSize.NORMAL
+        return GnomeDesktop.DesktopThumbnailFactory.new(size_normal)
+
+    @staticmethod
+    def compare_basename(model, iter1, iter2, unused_user_data):
+        """
+        Compare the model elements identified by the L{Gtk.TreeIter} elements.
+        """
+        uri1 = model[iter1][COL_URI]
+        uri2 = model[iter2][COL_URI]
+        basename1 = GLib.path_get_basename(uri1).lower()
+        basename2 = GLib.path_get_basename(uri2).lower()
+        if basename1 < basename2:
+            return -1
+        if basename1 == basename2:
+            if uri1 < uri2:
+                return -1
+        return 1
+
     def getAssetForUri(self, uri):
         # Sanitization
         uri = filter(lambda c: c != '\n' and c != '\r', uri)
@@ -329,23 +352,6 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
     def _insertEndCb(self, unused_action):
         self.app.gui.timeline_ui.insertEnd(self.getSelectedAssets())
 
-    def _trackElementAddedCb(self, source, unused_track_element):
-        """ After an object has been added to the first track, position it
-        correctly and request the next source to be processed. """
-        timeline = self.app.current_project.timeline
-        layer = timeline.get_layers()[0]  # FIXME Get the longest layer
-
-        # Handle the case where we just inserted the first clip
-        if len(layer.get_clips()) == 1:
-            source.props.start = 0
-        else:
-            source.props.start = timeline.props.duration
-
-        # We only need one TrackElement to estimate the new duration.
-        # Process the next source.
-        source.disconnect_by_func(self._trackElementAddedCb)
-        self._insertNextSource()
-
     def _searchEntryChangedCb(self, entry):
         # With many hundred clips in an iconview with dynamic columns and
         # ellipsizing, doing needless searches is very expensive.
@@ -366,12 +372,11 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         Used for the search box.
         """
         text = data.get_text().lower()
-        if text == "":
+        if not text:
             return True  # Avoid silly warnings
-        else:
-            # We must convert to markup form to be able to search for &, ', etc.
-            text = GLib.markup_escape_text(text)
-            return text in model.get_value(iter, COL_INFOTEXT).lower()
+        # We must convert to markup form to be able to search for &, ', etc.
+        text = GLib.markup_escape_text(text)
+        return text in model.get_value(iter, COL_INFOTEXT).lower()
 
     def _getIcon(self, iconname, alternate=None, size=48):
         icontheme = Gtk.IconTheme.get_default()
@@ -444,13 +449,14 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         chooser_action = Gtk.FileChooserAction.OPEN
         dialogtitle = _("Select One or More Files")
 
-        close_after = Gtk.CheckButton(_("Close after importing files"))
+        close_after = Gtk.CheckButton(label=_("Close after importing files"))
         close_after.set_active(self.app.settings.closeImportDialog)
 
-        self._importDialog = Gtk.FileChooserDialog(dialogtitle, None,
-                                           chooser_action,
-                                           (Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE,
-                                            Gtk.STOCK_ADD, Gtk.ResponseType.OK))
+        self._importDialog = Gtk.FileChooserDialog(title=dialogtitle, transient_for=None,
+                                           action=chooser_action)
+
+        self._importDialog.add_buttons(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE,
+                                       Gtk.STOCK_ADD, Gtk.ResponseType.OK)
         self._importDialog.set_icon_name("pitivi")
         self._importDialog.props.extra_widget = close_after
         self._importDialog.set_default_response(Gtk.ResponseType.OK)
@@ -460,17 +466,16 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         self._importDialog.set_current_folder(self.app.settings.lastImportFolder)
         self._importDialog.connect('response', self._dialogBoxResponseCb)
         self._importDialog.connect('close', self._dialogBoxCloseCb)
-        pw = PreviewWidget(self.app)
-        self._importDialog.set_preview_widget(pw)
+        previewer = PreviewWidget(self.app.settings)
+        self._importDialog.set_preview_widget(previewer)
         self._importDialog.set_use_preview_label(False)
-        self._importDialog.connect('update-preview', pw.add_preview_request)
+        self._importDialog.connect('update-preview', previewer.add_preview_request)
         # Filter for the "known good" formats by default
         filt_supported = Gtk.FileFilter()
         filt_known = Gtk.FileFilter()
         filt_supported.set_name(_("Supported file formats"))
-        for category in SUPPORTED_FILE_FORMATS:
-            # Category can be "video", "audio", "image", "application"
-            for mime in SUPPORTED_FILE_FORMATS[category]:
+        for category, mime_types in SUPPORTED_FILE_FORMATS.iteritems():
+            for mime in mime_types:
                 filt_supported.add_mime_type(category + "/" + mime)
                 filt_known.add_mime_type(category + "/" + mime)
         # Also allow showing known but not reliable demuxers
@@ -490,17 +495,20 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         """
         Update the _progressbar with the ratio of clips imported vs the total
         """
+        # The clip iter has a +1 offset in the progressbar label (to refer to
+        # the actual # of the clip we're processing), but there is no offset
+        # in the progressbar itself (to reflect the process being incomplete).
         current_clip_iter = self.app.current_project.nb_imported_files
         total_clips = self.app.current_project.nb_remaining_file_to_import + current_clip_iter
 
         progressbar_text = _("Importing clip %(current_clip)d of %(total)d" %
-            {"current_clip": current_clip_iter,
+            {"current_clip": current_clip_iter + 1,
             "total": total_clips})
         self._progressbar.set_text(progressbar_text)
         if current_clip_iter == 0:
             self._progressbar.set_fraction(0.0)
         elif total_clips != 0:
-            self._progressbar.set_fraction((current_clip_iter - 1) / float(total_clips))
+            self._progressbar.set_fraction(current_clip_iter / float(total_clips))
 
     def _getThumbnailInDir(self, dir, hash):
         """
@@ -531,8 +539,31 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
             except GLib.GError:
                 return None, None
 
+    def _generateThumbnails(self, uri):
+        if not self.thumbnailer:
+            # TODO: Use thumbnails generated with GStreamer.
+            return None
+        # This way of getting the mimetype feels awfully convoluted but
+        # seems to be the proper/reliable way in a GNOME context
+        asset_file = Gio.file_new_for_uri(uri)
+        info = asset_file.query_info(attributes="standard::*",
+                                    flags=Gio.FileQueryInfoFlags.NONE,
+                                    cancellable=None)
+        mime = Gio.content_type_get_mime_type(info.get_content_type())
+        mtime = os.path.getmtime(path_from_uri(uri))
+        if not self.thumbnailer.can_thumbnail(uri, mime, mtime):
+            self.debug("Thumbnailer says it can't thumbnail %s", uri)
+            return None
+        pixbuf_128 = self.thumbnailer.generate_thumbnail(uri, mime)
+        if not pixbuf_128:
+            self.debug("Thumbnailer failed thumbnailing %s", uri)
+            return None
+        self.thumbnailer.save_thumbnail(pixbuf_128, uri, mtime)
+        pixbuf_64 = pixbuf_128.scale_simple(64, 64, GdkPixbuf.InterpType.BILINEAR)
+        return pixbuf_128, pixbuf_64
+
     def _addAsset(self, asset):
-        # 128 is the normal thumb size, but for icons it looks insane.
+        # 128 is the normal size for thumbnails, but for *icons* it looks insane
         LARGE_SIZE = 96
         info = asset.get_info()
 
@@ -546,7 +577,8 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
             # $XDG_CACHE_HOME/thumbnails will be used, otherwise
             # $HOME/.cache/thumbnails will be used."
             # Older version of the spec also mentioned $HOME/.thumbnails
-            thumbnail_hash = md5(quote_uri(info.get_uri())).hexdigest()
+            quoted_uri = quote_uri(info.get_uri())
+            thumbnail_hash = md5(quoted_uri).hexdigest()
             try:
                 thumb_dir = os.environ['XDG_CACHE_HOME']
                 thumb_64, thumb_128 = self._getThumbnailInDir(thumb_dir, thumbnail_hash)
@@ -559,13 +591,15 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
                 thumb_dir = os.path.expanduser("~/.thumbnails/")
                 thumb_64, thumb_128 = self._getThumbnailInDir(thumb_dir, thumbnail_hash)
             if thumb_64 is None:
-                # TODO gst discoverer should create missing thumbnails.
                 if asset.is_image():
                     thumb_64 = self._getIcon("image-x-generic")
                     thumb_128 = self._getIcon("image-x-generic", None, LARGE_SIZE)
                 else:
                     thumb_64 = self._getIcon("video-x-generic")
                     thumb_128 = self._getIcon("video-x-generic", None, LARGE_SIZE)
+                # TODO ideally gst discoverer should create missing thumbnails.
+                self.log("Missing a thumbnail for %s, queuing", path_from_uri(quoted_uri))
+                self._missing_thumbs.append(quoted_uri)
         else:
             thumb_64 = self._getIcon("audio-x-generic")
             thumb_128 = self._getIcon("audio-x-generic", None, LARGE_SIZE)
@@ -588,6 +622,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
             self.flush_pending_rows()
 
     def flush_pending_rows(self):
+        self.debug("Flushing %d pending model rows", len(self.pending_rows))
         for row in self.pending_rows:
             self.storemodel.append(row)
         del self.pending_rows[:]
@@ -595,13 +630,13 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
     # medialibrary callbacks
 
     def _assetAddedCb(self, unused_project, asset,
-            current_clip_iter=None, total_clips=None):
+            unused_current_clip_iter=None, unused_total_clips=None):
         """ a file was added to the medialibrary """
         if isinstance(asset, GES.UriClipAsset):
             self._updateProgressbar()
             self._addAsset(asset)
 
-    def _assetRemovedCb(self, unsued_project, asset):
+    def _assetRemovedCb(self, unused_project, asset):
         """ the given uri was removed from the medialibrary """
         # find the good line in the storemodel and remove it
         model = self.storemodel
@@ -612,48 +647,79 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
                 break
         if not len(model):
             self._welcome_infobar.show_all()
-        self.debug("Removing %s", uri)
+        self.debug("Removing: %s", uri)
 
-    def _errorCreatingAssetCb(self, unsued_project, error, id, type):
+    def _errorCreatingAssetCb(self, unused_project, error, id, type):
         """ The given uri isn't a media file """
         if GObject.type_is_a(type, GES.UriClip):
             error = (id, str(error.domain), error)
             self._errors.append(error)
             self._updateProgressbar()
 
-    def _sourcesStartedImportingCb(self, unsued_project):
+    def _sourcesStartedImportingCb(self, unused_project):
         self.import_start_time = time.time()
         self._welcome_infobar.hide()
         self._progressbar.show()
 
-    def _sourcesStoppedImportingCb(self, unsued_project):
-        self.debug("Importing took %.3f seconds" % (time.time() - self.import_start_time))
+    def _sourcesStoppedImportingCb(self, unused_project):
+        self.debug("Importing took %.3f seconds", time.time() - self.import_start_time)
         self.flush_pending_rows()
         self._progressbar.hide()
         if self._errors:
-            if len(self._errors) > 1:
-                self._warning_label.set_text(_("Errors occurred while importing."))
-                self._view_error_button.set_label(_("View errors"))
-            else:
-                self._warning_label.set_text(_("An error occurred while importing."))
-                self._view_error_button.set_label(_("View error"))
+            errors_amount = len(self._errors)
+            btn_text = ngettext("View error", "View errors", errors_amount)
+            # Translators: {0:d} is just like %d (integer number variable)
+            text = ngettext("An error occurred while importing.",
+                            "{0:d} errors occurred while importing.",
+                            errors_amount)
+            # Do the {0:d} (aka "%d") substitution using "format" instead of %,
+            # avoiding tracebacks as %d would be missing in the singular form:
+            text = text.format(errors_amount)
 
+            self._view_error_button.set_label(btn_text)
+            self._warning_label.set_text(text)
             self._import_warning_infobar.show_all()
 
+        missing_thumbs = self._missing_thumbs
+        self._missing_thumbs = []
+        if missing_thumbs:
+            self.info("Generating missing thumbnails: %d", len(missing_thumbs))
+            self._thumbs_process = threading.Thread(target=MediaLibraryWidget._generateThumbnailsThread, args=(self, missing_thumbs))
+            self._thumbs_process.start()
+
+    def _generateThumbnailsThread(self, missing_thumbs):
+        for uri in missing_thumbs:
+            thumbnails = self._generateThumbnails(uri)
+            if not thumbnails:
+                continue
+            pixbuf_128, pixbuf_64 = thumbnails
+            # Search through the model for the row corresponding to the asset.
+            found = False
+            for row in self.storemodel:
+                if uri == row[COL_URI]:
+                    found = True
+                    # Finally, show the new pixbuf in the UI
+                    if pixbuf_128:
+                        row[COL_ICON_128] = pixbuf_128
+                    if pixbuf_64:
+                        row[COL_ICON_64] = pixbuf_64
+                    break
+            if not found:
+                # Can happen if the user removed the asset in the meanwhile.
+                self.log("%s needed a thumbnail, but vanished from storemodel", uri)
+
     ## Error Dialog Box callbacks
 
-    def _errorDialogBoxCloseCb(self, unused_dialog):
-        self._error_dialogbox.destroy()
-        self._error_dialogbox = None
+    def _errorDialogBoxCloseCb(self, dialog):
+        dialog.destroy()
 
-    def _errorDialogBoxResponseCb(self, unused_dialog, unused_response):
-        self._error_dialogbox.destroy()
-        self._error_dialogbox = None
+    def _errorDialogBoxResponseCb(self, dialog, unused_response):
+        dialog.destroy()
 
     ## Import Sources Dialog Box callbacks
 
     def _dialogBoxResponseCb(self, dialogbox, response):
-        self.debug("response:%r", response)
+        self.debug("response: %r", response)
         if response == Gtk.ResponseType.OK:
             lastfolder = dialogbox.get_current_folder()
             self.app.settings.lastImportFolder = lastfolder
@@ -740,13 +806,15 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         Show the clip properties (resolution, framerate, audio channels...)
         and allow setting them as the new project settings.
         """
-        paths = self.getSelectedPaths()[0]  # Only use the first item
-        model = self.treeview.get_model()
-        info = model[paths][COL_ASSET].get_info()
-        d = clipmediapropsDialog(self.app.current_project,
-                                info.get_audio_streams(),
-                                info.get_video_streams())
-        d.run()
+        paths = self.getSelectedPaths()
+        if not paths:
+            self.debug("No item selected")
+            return
+        # Only use the first item.
+        path = paths[0]
+        asset = self.storemodel[path][COL_ASSET]
+        dialog = ClipMediaPropsDialog(self.app.current_project, asset)
+        dialog.run()
 
     def _warningInfoBarDismissedCb(self, unused_button):
         self._resetErrorList()
@@ -765,12 +833,12 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         else:
             msgs = (_("Error while analyzing a file"),
                     _("The following file can not be used with Pitivi."))
-        self._error_dialogbox = FileListErrorDialog(*msgs)
-        self._error_dialogbox.connect("close", self._errorDialogBoxCloseCb)
-        self._error_dialogbox.connect("response", self._errorDialogBoxResponseCb)
+        error_dialogbox = FileListErrorDialog(*msgs)
+        error_dialogbox.connect("close", self._errorDialogBoxCloseCb)
+        error_dialogbox.connect("response", self._errorDialogBoxResponseCb)
         for uri, reason, extra in self._errors:
-            self._error_dialogbox.addFailedFile(uri, reason, extra)
-        self._error_dialogbox.window.show()
+            error_dialogbox.addFailedFile(uri, reason, extra)
+        error_dialogbox.window.show()
         # Reset the error list, since the user has read them.
         self._resetErrorList()
 
@@ -795,7 +863,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
                 selection = view.get_selected_items()
                 return view.path_is_selected(path) and len(selection)
         else:
-            assert False
+            raise RuntimeError("Unknown media library view type: %s" % type(view))
 
         return False
 
@@ -832,22 +900,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
             self.iconview.unselect_all()
 
     def _treeViewButtonPressEventCb(self, treeview, event):
-        chain_up = True
-
-        if event.type == getattr(Gdk.EventType, '2BUTTON_PRESS'):
-            # It is possible to double-click outside of clips:
-            if self.getSelectedPaths() != []:
-                # Here we used to emit "play", but
-                # this is now handled by _itemOrRowActivatedCb instead.
-                pass
-            chain_up = False
-        elif not event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK):
-            chain_up = not self._rowUnderMouseSelected(treeview, event)
-
-        if not chain_up:
-            self._draggedPaths = self.getSelectedPaths()
-        else:
-            self._draggedPaths = None
+        self._updateDraggedPaths(treeview, event)
 
         Gtk.TreeView.do_button_press_event(treeview, event)
 
@@ -858,7 +911,25 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
 
         return True
 
-    def _treeViewButtonReleaseEventCb(self, treeview, event):
+    def _updateDraggedPaths(self, view, event):
+        if event.type == getattr(Gdk.EventType, '2BUTTON_PRESS'):
+            # It is possible to double-click outside of clips:
+            if self.getSelectedPaths():
+                # Here we used to emit "play", but
+                # this is now handled by _itemOrRowActivatedCb instead.
+                pass
+            chain_up = False
+        elif not event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK):
+            chain_up = not self._rowUnderMouseSelected(view, event)
+        else:
+            chain_up = True
+
+        if not chain_up:
+            self._draggedPaths = self.getSelectedPaths()
+        else:
+            self._draggedPaths = None
+
+    def _treeViewButtonReleaseEventCb(self, unused_treeview, event):
         ts = self.treeview.get_selection()
         state = event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
         path = self.treeview.get_path_at_pos(event.x, event.y)
@@ -894,22 +965,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         self.emit('play', asset)
 
     def _iconViewButtonPressEventCb(self, iconview, event):
-        chain_up = True
-
-        if event.type == getattr(Gdk.EventType, '2BUTTON_PRESS'):
-            # It is possible to double-click outside of clips:
-            if self.getSelectedPaths() != []:
-                # Here we used to emit "play", but
-                # this is now handled by _itemOrRowActivatedCb instead.
-                pass
-            chain_up = False
-        elif not event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK):
-            chain_up = not self._rowUnderMouseSelected(iconview, event)
-
-        if not chain_up:
-            self._draggedPaths = self.getSelectedPaths()
-        else:
-            self._draggedPaths = None
+        self._updateDraggedPaths(iconview, event)
 
         Gtk.IconView.do_button_press_event(iconview, event)
 
@@ -917,7 +973,6 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
             for path in self._draggedPaths:
                 self.iconview.select_path(path)
 
-        self._ignoreRelease = chain_up
         self.iconview_cursor_pos = self.iconview.get_path_at_pos(event.x, event.y)
 
         return True
@@ -934,7 +989,7 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
                     iconview.unselect_all()
                     iconview.select_path(current_cursor_pos)
 
-    def _newProjectCreatedCb(self, app, project):
+    def _newProjectCreatedCb(self, unused_app, project):
         if not self._project is project:
             self._project = project
             self._resetErrorList()
@@ -966,37 +1021,37 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
     ## Drag and Drop
     def _dndDataReceivedCb(self, unused_widget, unused_context, unused_x,
                            unused_y, selection, targettype, unused_time):
-
-        self.debug("targettype:%d, selection.data:%r", targettype, selection.get_data())
+        self.debug("targettype: %d, selection.data: %r", targettype, selection.get_data())
 
         directories = []
-        remote_files = []
         filenames = []
 
         uris = selection.get_data().split("\r\n")
-        uris = filter(lambda x: x != "", uris)
+        # Filter out the empty uris.
+        uris = filter(lambda x: x, uris)
+        for raw_uri in uris:
+            # Strip out NULL chars first.
+            raw_uri = raw_uri.strip('\x00')
+            uri = urlparse(raw_uri)
+            if uri.scheme == 'file':
+                path = unquote(uri.path)
+                if os.path.isfile(path):
+                    filenames.append(raw_uri)
+                elif os.path.isdir(path):
+                    directories.append(raw_uri)
+                else:
+                    self.warning("Unusable file: %s, %s", raw_uri, path)
+            else:
+                self.fixme("Importing remote files is not implemented: %s", raw_uri)
 
-        for uri in uris:
-            uri = unquote(uri.strip('\x00'))
-            if os.path.isfile(uri[7:]):
-                filenames.append(uri)
-            elif os.path.isdir(uri[7:]):
-                directories.append(uri)
-            elif "://" in uri:
-                #FIXME Very dubious check.
-                remote_files.append(uri)
-
-        if len(directories):
+        if directories:
             # Recursively import from folders that were dragged into the library
             self.app.threads.addThread(PathWalker, directories, self._addUris)
-        if len(remote_files):
-            #TODO waiting for remote files downloader support to be implemented
-            self.fixme("Importing remote files is not implemented")
-        if len(filenames):
+        if filenames:
             self.app.current_project.addUris(filenames)
 
     #used with TreeView and IconView
-    def _dndDragDataGetCb(self, unused_view, context, data, info, timestamp):
+    def _dndDragDataGetCb(self, unused_view, unused_context, data, unused_info, unused_timestamp):
         paths = self.getSelectedPaths()
         uris = [self.modelFilter[path][COL_URI] for path in paths]
         data.set_uris(uris)
@@ -1006,13 +1061,13 @@ class MediaLibraryWidget(Gtk.VBox, Loggable):
         self.dragged = True
         paths = self.getSelectedPaths()
 
-        if len(paths) < 1:
+        if not paths:
             context.drag_abort(int(time.time()))
         else:
             row = self.modelFilter[paths[0]]
             Gtk.drag_set_icon_pixbuf(context, row[COL_ICON_64], 0, 0)
 
-    def _dndDragEndCb(self, unused_view, context):
+    def _dndDragEndCb(self, unused_view, unused_context):
         self.info("Drag operation ended")
         self.dragged = False
 
