@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       project.py
+#       pitivi/project.py
 #
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
@@ -41,11 +41,10 @@ from pitivi.undo.undo import UndoableAction
 from pitivi.configure import get_ui_dir
 
 from pitivi.utils.misc import quote_uri, path_from_uri, isWritable
-from pitivi.utils.pipeline import Seeker
+from pitivi.utils.pipeline import PipelineError, Seeker
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.signal import Signallable
 from pitivi.utils.pipeline import Pipeline
-from pitivi.utils.timeline import Selection
 from pitivi.utils.widgets import FractionWidget
 from pitivi.utils.ripple_update_group import RippleUpdateGroup
 from pitivi.utils.ui import frame_rates, audio_rates,\
@@ -145,16 +144,16 @@ class ProjectManager(Signallable, Loggable):
         use_backup = False
         try:
             time_diff = os.path.getmtime(backup_path) - os.path.getmtime(path)
-            self.debug('Backup file "%s" is %d secs newer' % (backup_path, time_diff))
+            self.debug('Backup file is %d secs newer: %s', time_diff, backup_path)
         except OSError:
-            self.debug('Backup file "%s" does not exist' % backup_path)
+            self.debug('Backup file does not exist: %s', backup_path)
         else:
             if time_diff > 0:
                 use_backup = self._restoreFromBackupDialog(time_diff)
 
         if use_backup:
             uri = self._makeBackupURI(uri)
-            self.debug('Loading project from backup "%s"' % uri)
+            self.debug('Loading project from backup: %s', uri)
 
         # Load the project:
         self.current_project = Project(uri=uri)
@@ -187,9 +186,9 @@ class ProjectManager(Signallable, Loggable):
 
         @param time_diff: the difference, in seconds, between file mtimes
         """
-        dialog = Gtk.Dialog("", None, 0,
-                            (_("Ignore backup"), Gtk.ResponseType.REJECT,
-                            _("Restore from backup"), Gtk.ResponseType.YES))
+        dialog = Gtk.Dialog(title="", transient_for=None)
+        dialog.add_buttons(_("Ignore backup"), Gtk.ResponseType.REJECT,
+                           _("Restore from backup"), Gtk.ResponseType.YES)
         # Even though we set the title to an empty string when creating dialog,
         # seems we really have to do it once more so it doesn't show "pitivi"...
         dialog.set_title("")
@@ -211,13 +210,12 @@ class ProjectManager(Signallable, Loggable):
         primary.props.label = message
 
         # put the text in a vbox
-        vbox = Gtk.VBox(False, SPACING * 2)
+        vbox = Gtk.VBox(homogeneous=False, spacing=SPACING * 2)
         vbox.pack_start(primary, True, True, 0)
 
         # make the [[image] text] hbox
-        image = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_QUESTION,
-                                         Gtk.IconSize.DIALOG)
-        hbox = Gtk.HBox(False, SPACING * 2)
+        image = Gtk.Image.new_from_icon_name("dialog-question", Gtk.IconSize.DIALOG)
+        hbox = Gtk.HBox(homogeneous=False, spacing=SPACING * 2)
         hbox.pack_start(image, False, True, 0)
         hbox.pack_start(vbox, True, True, 0)
         hbox.set_border_width(SPACING)
@@ -252,7 +250,7 @@ class ProjectManager(Signallable, Loggable):
         """
         if self.disable_save is True and (backup is True or uri is None):
             self.log("Read-only mode is enforced and no new URI was specified, ignoring save request")
-            return
+            return False
 
         if backup:
             if self.current_project is not None and self.current_project.uri is not None:
@@ -262,7 +260,7 @@ class ProjectManager(Signallable, Loggable):
                 # Do not try to save backup files for blank projects.
                 # It is possible that self.current_project.uri == None when the backup
                 # timer sent us an old instance of the (now closed) project.
-                return
+                return False
         elif uri is None:
             # "Normal save" scenario. The filechoosers in mainwindow ask users
             # for permission to overwrite the file (if needed), so we're safe.
@@ -276,7 +274,7 @@ class ProjectManager(Signallable, Loggable):
                 # TODO: this will not be needed when GTK+ bug #601451 is fixed
                 self.emit("save-project-failed", uri,
                           _("You do not have permissions to write to this folder."))
-                return
+                return False
 
         try:
             # "overwrite" is always True: our GTK filechooser save dialogs are
@@ -292,14 +290,14 @@ class ProjectManager(Signallable, Loggable):
                 # Do not emit the signal when autosaving a backup file
                 self.current_project.setModificationState(False)
                 self.emit("project-saved", self.current_project, uri)
-                self.debug('Saved project "%s"' % uri)
+                self.debug('Saved project: %s', uri)
                 # Update the project instance's uri,
                 # otherwise, subsequent saves will be to the old uri.
-                self.info("Setting the project instance's URI to %s" % uri)
+                self.info("Setting the project instance's URI to: %s", uri)
                 self.current_project.uri = uri
                 self.disable_save = False
             else:
-                self.debug('Saved backup "%s"' % uri)
+                self.debug('Saved backup: %s', uri)
 
         return saved
 
@@ -424,8 +422,8 @@ class ProjectManager(Signallable, Loggable):
         project.author = getpwuid(os.getuid()).pw_gecos.split(",")[0]
 
         project.createTimeline()
-        self.emit("new-project-created", project)
         self.current_project = project
+        self.emit("new-project-created", project)
 
         project.connect("project-changed", self._projectChangedCb)
         self.emit("new-project-loaded", self.current_project, emission)
@@ -463,7 +461,7 @@ class ProjectManager(Signallable, Loggable):
             if self._backup_lock < 60:
                 self._backup_lock += 5
 
-    def _saveBackupCb(self, project, uri):
+    def _saveBackupCb(self, unused_project, unused_uri):
         if self._backup_lock > 10:
             self._backup_lock -= 5
             return True
@@ -478,7 +476,7 @@ class ProjectManager(Signallable, Loggable):
         path = path_from_uri(self._makeBackupURI(uri))
         if os.path.exists(path):
             os.remove(path)
-            self.debug('Removed backup file "%s"' % path)
+            self.debug('Removed backup file: %s', path)
 
     def _makeBackupURI(self, uri):
         """
@@ -491,10 +489,10 @@ class ProjectManager(Signallable, Loggable):
         name, ext = os.path.splitext(uri)
         return name + ext + "~"
 
-    def _missingURICb(self, project, error, asset, what=None):
+    def _missingURICb(self, project, error, asset, unused_what=None):
         return self.emit("missing-uri", project, error, asset)
 
-    def _projectLoadedCb(self, project, timeline):
+    def _projectLoadedCb(self, unused_project, unused_timeline):
         self.debug("Project loaded %s", self.current_project.props.uri)
         self.emit("new-project-loaded", self.current_project, True)
         self.time_loaded = time()
@@ -511,8 +509,6 @@ class Project(Loggable, GES.Project):
     @type timeline: L{GES.Timeline}
     @ivar pipeline: The timeline's pipeline
     @type pipeline: L{Pipeline}
-    @ivar format: The format under which the project is currently stored.
-    @type format: L{FormatterClass}
     @ivar loaded: Whether the project is fully loaded or not.
     @type loaded: C{bool}
 
@@ -531,7 +527,7 @@ class Project(Loggable, GES.Project):
                                         GObject.TYPE_PYOBJECT,))
     }
 
-    def __init__(self, name="", uri=None, **kwargs):
+    def __init__(self, name="", uri=None, **unused_kwargs):
         """
         @param name: the name of the project
         @param uri: the uri of the project
@@ -685,7 +681,7 @@ class Project(Loggable, GES.Project):
 
     @videowidth.setter
     def videowidth(self, value):
-        if self.set_video_restriction_value("width", int(value)):
+        if value and self.set_video_restriction_value("width", int(value)):
             self._emitChange("rendering-settings-changed", "width", value)
 
     @property
@@ -694,8 +690,7 @@ class Project(Loggable, GES.Project):
 
     @videoheight.setter
     def videoheight(self, value):
-        value = int(value)
-        if self.set_video_restriction_value("height", int(value)):
+        if value and self.set_video_restriction_value("height", int(value)):
             self._emitChange("rendering-settings-changed", "height", value)
 
     @property
@@ -713,7 +708,8 @@ class Project(Loggable, GES.Project):
 
     @videopar.setter
     def videopar(self, value):
-        self.set_video_restriction_value("pixel-aspect-ratio", value)
+        if self.set_video_restriction_value("pixel-aspect-ratio", value):
+            self._emitChange("rendering-settings-changed", "pixel-aspect-ratio", value)
 
     @property
     def audiochannels(self):
@@ -721,7 +717,7 @@ class Project(Loggable, GES.Project):
 
     @audiochannels.setter
     def audiochannels(self, value):
-        if self.set_audio_restriction_value("channels", value):
+        if value and self.set_audio_restriction_value("channels", int(value)):
             self._emitChange("rendering-settings-changed", "channels", value)
 
     @property
@@ -733,7 +729,7 @@ class Project(Loggable, GES.Project):
 
     @audiorate.setter
     def audiorate(self, value):
-        if self.set_audio_restriction_value("rate", value):
+        if value and self.set_audio_restriction_value("rate", int(value)):
             self._emitChange("rendering-settings-changed", "rate", value)
 
     @property
@@ -749,7 +745,7 @@ class Project(Loggable, GES.Project):
             else:
                 for template in feature.get_static_pad_templates():
                     if template.name_template == "src":
-                        audiotype = template.get_caps()[0].get_name()
+                        audiotype = template.get_caps()[0].to_string()
                         break
                 self.audio_profile.set_format(Gst.Caps(audiotype))
             self.audio_profile.set_preset_name(value)
@@ -769,7 +765,7 @@ class Project(Loggable, GES.Project):
             else:
                 for template in feature.get_static_pad_templates():
                     if template.name_template == "src":
-                        videotype = template.get_caps()[0].get_name()
+                        videotype = template.get_caps()[0].to_string()
                         break
                 self.video_profile.set_format(Gst.Caps(videotype))
 
@@ -790,7 +786,7 @@ class Project(Loggable, GES.Project):
             else:
                 for template in feature.get_static_pad_templates():
                     if template.name_template == "src":
-                        muxertype = template.get_caps()[0].get_name()
+                        muxertype = template.get_caps()[0].to_string()
                         break
                 self.container_profile.set_format(Gst.Caps(muxertype))
             self.container_profile.set_preset_name(value)
@@ -809,9 +805,14 @@ class Project(Loggable, GES.Project):
     #--------------------------------------------#
     # GES.Project virtual methods implementation #
     #--------------------------------------------#
-    def _handle_asset_loaded(self, id):
+
+    def _handle_asset_loaded(self, asset=None, unused_asset_id=None):
+        if asset and not GObject.type_is_a(asset.get_extractable_type(), GES.UriClip):
+            # Ignore for example the assets producing GES.TitleClips.
+            return
         self.nb_imported_files += 1
-        self.nb_remaining_file_to_import = len([asset for asset in self.get_loading_assets() if
+        assets = self.get_loading_assets()
+        self.nb_remaining_file_to_import = len([asset for asset in assets if
                 GObject.type_is_a(asset.get_extractable_type(), GES.UriClip)])
         if self.nb_remaining_file_to_import == 0:
             self.nb_imported_files = 0
@@ -822,13 +823,13 @@ class Project(Loggable, GES.Project):
         When GES.Project emit "asset-added" this vmethod
         get calls
         """
-        self._handle_asset_loaded(asset.get_id())
+        self._handle_asset_loaded(asset=asset)
 
-    def do_loading_error(self, error, id, type):
+    def do_loading_error(self, unused_error, asset_id, unused_type):
         """ vmethod, get called on "asset-loading-error"""
-        self._handle_asset_loaded(id)
+        self._handle_asset_loaded(unused_asset_id=asset_id)
 
-    def do_loaded(self, timeline):
+    def do_loaded(self, unused_timeline):
         """ vmethod, get called on "loaded" """
         self._ensureTracks()
         #self._ensureLayer()
@@ -864,11 +865,12 @@ class Project(Loggable, GES.Project):
     #--------------------------------------------#
     #               Our API                      #
     #--------------------------------------------#
+
     def createTimeline(self):
         """
-        The pitivi.Project handle 1 timeline at a time
-        unlike GES.Project
+        Load the project.
         """
+        # In this extract call the project is loaded from the file.
         self.timeline = self.extract()
         if self.timeline is None:
             return False
@@ -876,20 +878,25 @@ class Project(Loggable, GES.Project):
             self.timeline.props.auto_transition = True
         self._calculateNbLoadingAssets()
 
-        self.timeline.selection = Selection()
         self.pipeline = Pipeline()
-        self.pipeline.add_timeline(self.timeline)
+        try:
+            self.pipeline.set_timeline(self.timeline)
+        except PipelineError, e:
+            self.warning("Failed to set the timeline to the pipeline: %s", e)
+            return False
 
         return True
 
     def update_restriction_caps(self):
         caps = Gst.Caps.new_empty_simple("video/x-raw")
+
         caps.set_value("width", self.videowidth)
         caps.set_value("height", self.videoheight)
         caps.set_value("framerate", self.videorate)
         for track in self.timeline.get_tracks():
             if isinstance(track, GES.VideoTrack):
                 track.set_restriction_caps(caps)
+        self.pipeline.flushSeek()
 
     def addUris(self, uris):
         """
@@ -1034,7 +1041,7 @@ class Project(Loggable, GES.Project):
         if not self.videorate:
             self.videorate = Gst.Fraction(25, 1)
         if not self.videopar:
-            self.videopar = Gst.Fraction(16, 15)
+            self.videopar = Gst.Fraction(1, 1)
 
     def _ensureAudioRestrictions(self):
         if not self.audiochannels:
@@ -1123,8 +1130,8 @@ class ProjectSettingsDialog():
         # Set the shading style in the contextual toolbars below presets
         video_presets_toolbar = getObj("video_presets_toolbar")
         audio_presets_toolbar = getObj("audio_presets_toolbar")
-        video_presets_toolbar.get_style_context().add_class("inline-toolbar")
-        audio_presets_toolbar.get_style_context().add_class("inline-toolbar")
+        video_presets_toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
+        audio_presets_toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
 
     def _setupUiConstraints(self):
         """
@@ -1330,12 +1337,12 @@ class ProjectSettingsDialog():
             "height": self.project.videoheight,
             "width": self.project.videowidth})
 
-    def _newPresetCb(self, model, path, iter_, column, renderer, treeview):
+    def _newPresetCb(self, unused_model, path, unused_iter_, column, renderer, treeview):
         """ Handle the addition of a preset to the model of the preset manager. """
         treeview.set_cursor_on_cell(path, column, renderer, start_editing=True)
         treeview.grab_focus()
 
-    def _presetNameEditedCb(self, renderer, path, new_text, mgr):
+    def _presetNameEditedCb(self, unused_renderer, path, new_text, mgr):
         """Handle the renaming of a preset."""
         try:
             mgr.renamePreset(path, new_text)
@@ -1343,7 +1350,7 @@ class ProjectSettingsDialog():
             error_markup = _('"%s" already exists.') % new_text
             self._showPresetManagerError(mgr, error_markup)
 
-    def _presetNameEditingStartedCb(self, renderer, editable, path, mgr):
+    def _presetNameEditingStartedCb(self, unused_renderer, unused_editable, unused_path, mgr):
         """Handle the start of a preset renaming."""
         self._hidePresetManagerError(mgr)
 
@@ -1358,7 +1365,7 @@ class ProjectSettingsDialog():
         update_preset_buttons_func()
         self._hidePresetManagerError(mgr)
 
-    def _treeviewDefocusedCb(self, widget, event, mgr):
+    def _treeviewDefocusedCb(self, unused_widget, unused_event, mgr):
         self._hidePresetManagerError(mgr)
 
     def _showPresetManagerError(self, mgr, error_markup):
@@ -1420,7 +1427,7 @@ class ProjectSettingsDialog():
             i += 1
         return preset_name
 
-    def _addAudioPresetButtonClickedCb(self, button):
+    def _addAudioPresetButtonClickedCb(self, unused_button):
         preset_name = self._getUniquePresetName(self.audio_presets)
         self.audio_presets.addPreset(preset_name, {
             "channels": get_combo_value(self.channels_combo),
@@ -1429,18 +1436,18 @@ class ProjectSettingsDialog():
         self.audio_presets.restorePreset(preset_name)
         self._updateAudioPresetButtons()
 
-    def _removeAudioPresetButtonClickedCb(self, button):
+    def _removeAudioPresetButtonClickedCb(self, unused_button):
         selection = self.audio_preset_treeview.get_selection()
         model, iter_ = selection.get_selected()
         if iter_:
             self.audio_presets.removePreset(model[iter_][0])
 
-    def _saveAudioPresetButtonClickedCb(self, button):
+    def _saveAudioPresetButtonClickedCb(self, unused_button):
         self.audio_presets.saveCurrentPreset()
         self.save_audio_preset_button.set_sensitive(False)
         self.remove_audio_preset_button.set_sensitive(True)
 
-    def _addVideoPresetButtonClickedCb(self, button):
+    def _addVideoPresetButtonClickedCb(self, unused_button):
         preset_name = self._getUniquePresetName(self.video_presets)
         self.video_presets.addPreset(preset_name, {
             "width": int(self.width_spinbutton.get_value()),
@@ -1451,13 +1458,13 @@ class ProjectSettingsDialog():
         self.video_presets.restorePreset(preset_name)
         self._updateVideoPresetButtons()
 
-    def _removeVideoPresetButtonClickedCb(self, button):
+    def _removeVideoPresetButtonClickedCb(self, unused_button):
         selection = self.video_preset_treeview.get_selection()
         model, iter_ = selection.get_selected()
         if iter_:
             self.video_presets.removePreset(model[iter_][0])
 
-    def _saveVideoPresetButtonClickedCb(self, button):
+    def _saveVideoPresetButtonClickedCb(self, unused_button):
         self.video_presets.saveCurrentPreset()
         self.save_video_preset_button.set_sensitive(False)
         self.remove_video_preset_button.set_sensitive(True)
