@@ -25,33 +25,25 @@ Rendering-related utilities and classes
 """
 
 import os
+import time
+
 from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Gst
 from gi.repository import GES
-import time
 
 from gettext import gettext as _
-from pitivi import configure
-from pitivi.utils.signal import Signallable
 
+from pitivi import configure
+
+from pitivi.check import PYCANBERRA_SOFT_DEPENDENCY
 from pitivi.utils.loggable import Loggable
-from pitivi.utils.widgets import GstElementSettingsDialog
-from pitivi.utils.ripple_update_group import RippleUpdateGroup
 from pitivi.utils.misc import show_user_manual, path_from_uri
+from pitivi.utils.ripple_update_group import RippleUpdateGroup
+from pitivi.utils.signal import Signallable
 from pitivi.utils.ui import model, frame_rates, audio_rates,\
     audio_channels, get_combo_value, set_combo_value, beautify_ETA
-try:
-    import pycanberra
-    has_canberra = True
-except ImportError:
-    has_canberra = False
-
-try:
-    from gi.repository import Notify
-    has_libnotify = True
-except ImportError:
-    has_libnotify = False
+from pitivi.utils.widgets import GstElementSettingsDialog
 
 
 class CachedEncoderList(object):
@@ -526,13 +518,13 @@ class RenderDialog(Loggable):
                                         mgr, update_buttons_func)
         treeview.connect("focus-out-event", self._treeviewDefocusedCb, mgr)
 
-    def _newPresetCb(self, model, path, iter_, column, renderer, treeview):
+    def _newPresetCb(self, unused_model, path, unused_iter_, column, renderer, treeview):
         """Handle the addition of a preset to the model of the preset manager.
         """
         treeview.set_cursor_on_cell(path, column, renderer, start_editing=True)
         treeview.grab_focus()
 
-    def _presetNameEditedCb(self, renderer, path, new_text, mgr):
+    def _presetNameEditedCb(self, unused_renderer, path, new_text, mgr):
         """Handle the renaming of a preset."""
         from pitivi.preset import DuplicatePresetNameException
 
@@ -543,11 +535,11 @@ class RenderDialog(Loggable):
             error_markup = _('"%s" already exists.') % new_text
             self._showPresetManagerError(mgr, error_markup)
 
-    def _presetNameEditingStartedCb(self, renderer, editable, path, mgr):
+    def _presetNameEditingStartedCb(self, unused_renderer, unused_editable, unused_path, mgr):
         """Handle the start of a preset renaming."""
         self._hidePresetManagerError(mgr)
 
-    def _treeviewDefocusedCb(self, widget, event, mgr):
+    def _treeviewDefocusedCb(self, unused_widget, unused_event, mgr):
         """Handle the treeview loosing the focus."""
         self._hidePresetManagerError(mgr)
 
@@ -588,7 +580,7 @@ class RenderDialog(Loggable):
             i += 1
         return preset_name
 
-    def _addRenderPresetButtonClickedCb(self, button):
+    def _addRenderPresetButtonClickedCb(self, unused_button):
         preset_name = self._getUniquePresetName(self.render_presets)
         self.render_presets.addPreset(preset_name, {
             "channels": int(get_combo_value(self.channels_combo)),
@@ -604,7 +596,7 @@ class RenderDialog(Loggable):
         self.render_presets.restorePreset(preset_name)
         self._updateRenderPresetButtons()
 
-    def _saveRenderPresetButtonClickedCb(self, button):
+    def _saveRenderPresetButtonClickedCb(self, unused_button):
         self.render_presets.saveCurrentPreset()
         self.save_render_preset_button.set_sensitive(False)
         self.remove_render_preset_button.set_sensitive(True)
@@ -615,7 +607,7 @@ class RenderDialog(Loggable):
         can_remove = self.render_presets.isRemoveButtonSensitive()
         self.remove_render_preset_button.set_sensitive(can_remove)
 
-    def _removeRenderPresetButtonClickedCb(self, button):
+    def _removeRenderPresetButtonClickedCb(self, unused_button):
         selection = self.render_preset_treeview.get_selection()
         model, iter_ = selection.get_selected()
         if iter_:
@@ -667,9 +659,9 @@ class RenderDialog(Loggable):
 
         # Set the shading style in the toolbar below presets
         presets_toolbar = builder.get_object("render_presets_toolbar")
-        presets_toolbar.get_style_context().add_class("inline-toolbar")
+        presets_toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
 
-    def _settingsChanged(self, project, key, value):
+    def _settingsChanged(self, unused_project, unused_key, unused_value):
         self.updateResolution()
 
     def _initializeComboboxModels(self):
@@ -699,7 +691,7 @@ class RenderDialog(Loggable):
         set_combo_value(self.muxercombobox,
             Gst.ElementFactory.find(self.project.muxer))
 
-    def _checkForExistingFile(self, *args):
+    def _checkForExistingFile(self, *unused_args):
         """
         Display a warning icon and tooltip if the file path already exists.
         """
@@ -707,7 +699,7 @@ class RenderDialog(Loggable):
         if not path:
             # This happens when the window is initialized.
             return
-        warning_icon = Gtk.STOCK_DIALOG_WARNING
+        warning_icon = "dialog-warning"
         filename = self.fileentry.get_text()
         if not filename:
             tooltip_text = _("A file name is required.")
@@ -718,7 +710,7 @@ class RenderDialog(Loggable):
         else:
             warning_icon = None
             tooltip_text = None
-        self.fileentry.set_icon_from_stock(1, warning_icon)
+        self.fileentry.set_icon_from_icon_name(1, warning_icon)
         self.fileentry.set_icon_tooltip_text(1, tooltip_text)
 
     def _getFilesizeEstimate(self):
@@ -801,14 +793,14 @@ class RenderDialog(Loggable):
             "file a bug report. See the details below for some basic "
             "information that may help identify the problem.")
 
-        dialog = Gtk.MessageDialog(self.window, Gtk.DialogFlags.MODAL,
-            Gtk.MessageType.ERROR, Gtk.ButtonsType.OK,
-            primary_message)
+        dialog = Gtk.MessageDialog(transient_for=self.window, modal=True,
+            message_type=Gtk.MessageType.ERROR, buttons=Gtk.ButtonsType.OK,
+            text=primary_message)
         dialog.set_property("secondary-text", secondary_message)
 
         expander = Gtk.Expander()
         expander.set_label(_("Details"))
-        details_label = Gtk.Label(str(error) + "\n\n" + str(details))
+        details_label = Gtk.Label(label=str(error) + "\n\n" + str(details))
         details_label.set_line_wrap(True)
         details_label.set_selectable(True)
         expander.add(details_label)
@@ -834,8 +826,7 @@ class RenderDialog(Loggable):
         self._destroyProgressWindow()
 
     def _shutDown(self):
-        """ The render process has been aborted, shutdown the gstreamer pipeline
-        and disconnect from its signals """
+        """Shutdown the gstreamer pipeline and disconnect from its signals."""
         self.project.set_rendering(False)
         self._is_rendering = False
         self._rendering_is_paused = False
@@ -844,13 +835,13 @@ class RenderDialog(Loggable):
         self._disconnectFromGst()
         self._pipeline.set_mode(GES.PipelineFlags.FULL_PREVIEW)
 
-    def _pauseRender(self, progress):
+    def _pauseRender(self, unused_progress):
         self._rendering_is_paused = self.progress.play_pause_button.get_active()
         if self._rendering_is_paused:
             self._last_timestamp_when_pausing = time.time()
         else:
             self._time_spent_paused += time.time() - self._last_timestamp_when_pausing
-            self.debug("Resuming render after %d seconds in pause" % self._time_spent_paused)
+            self.debug("Resuming render after %d seconds in pause", self._time_spent_paused)
         self.app.current_project.pipeline.togglePlayback()
 
     def _destroyProgressWindow(self):
@@ -872,11 +863,19 @@ class RenderDialog(Loggable):
     def destroy(self):
         self.window.destroy()
 
+    @staticmethod
+    def _maybePlayFinishedSound():
+        if not PYCANBERRA_SOFT_DEPENDENCY:
+            return
+        import pycanberra
+        canberra = pycanberra.Canberra()
+        canberra.play(1, pycanberra.CA_PROP_EVENT_ID, "complete-media", None)
+
     #------------------- Callbacks ------------------------------------------#
 
     #-- UI callbacks
     def _okButtonClickedCb(self, unused_button, settings_attr):
-        setattr(self, settings_attr, self.dialog.getSettings())
+        setattr(self.project, settings_attr, self.dialog.getSettings())
         self.dialog.window.destroy()
 
     def _renderButtonClickedCb(self, unused_button):
@@ -925,7 +924,7 @@ class RenderDialog(Loggable):
         self.debug("Render dialog's Close button clicked")
         self.destroy()
 
-    def _deleteEventCb(self, window, event):
+    def _deleteEventCb(self, unused_window, unused_event):
         self.debug("Render dialog is being deleted")
         self.destroy()
 
@@ -970,14 +969,10 @@ class RenderDialog(Loggable):
             self.progress.progressbar.set_text(_("Render complete"))
             self.progress.window.set_title(_("Render complete"))
             self.progress.setFilesizeEstimate(None)
-            if has_libnotify:
-                Notify.init("pitivi")
-                if not self.progress.window.is_active():
-                    self.notification = Notify.Notification.new(_("Render complete"), _('"%s" has finished rendering.' % self.fileentry.get_text()), "pitivi")
-                    self.notification.show()
-            if has_canberra:
-                canberra = pycanberra.Canberra()
-                canberra.play(1, pycanberra.CA_PROP_EVENT_ID, "complete-media", None)
+            if not self.progress.window.is_active():
+                notification = _('"%s" has finished rendering.' % self.fileentry.get_text())
+                self.notification = self.app.system.desktopMessage(_("Render complete"), notification, "pitivi")
+            self._maybePlayFinishedSound()
             self.progress.play_rendered_file_button.show()
             self.progress.close_button.show()
             self.progress.cancel_button.hide()
@@ -1001,7 +996,7 @@ class RenderDialog(Loggable):
                     else:
                         self.system.uninhibitSleep(RenderDialog.INHIBIT_REASON)
 
-    def _updatePositionCb(self, pipeline, position):
+    def _updatePositionCb(self, unused_pipeline, position):
         """
         Unlike other progression indicator callbacks, this one occurs every time
         the pipeline emits a position changed signal, which is *very* often.
@@ -1027,7 +1022,7 @@ class RenderDialog(Loggable):
         if not self._filesizeEstimateTimer and (fraction > 0.33 or timediff > 180):
             self._filesizeEstimateTimer = GLib.timeout_add_seconds(5, self._updateFilesizeEstimateCb)
 
-    def _elementAddedCb(self, bin, element):
+    def _elementAddedCb(self, unused_bin, element):
         """
         Setting properties on Gst.Element-s has they are added to the
         Gst.Encodebin
@@ -1044,7 +1039,7 @@ class RenderDialog(Loggable):
             self.debug("Setting %s to %s", propname, value)
 
     #-- Settings changed callbacks
-    def _scaleSpinbuttonChangedCb(self, button):
+    def _scaleSpinbuttonChangedCb(self, unused_button):
         render_scale = self.scale_spinbutton.get_value()
         self.project.render_scale = render_scale
         self.updateResolution()
@@ -1053,12 +1048,12 @@ class RenderDialog(Loggable):
         width, height = self.project.getVideoWidthAndHeight(True)
         self.resolution_label.set_text(u"%d×%d" % (width, height))
 
-    def _projectSettingsButtonClickedCb(self, button):
+    def _projectSettingsButtonClickedCb(self, unused_button):
         from pitivi.project import ProjectSettingsDialog
         dialog = ProjectSettingsDialog(self.window, self.project)
         dialog.window.run()
 
-    def _audioOutputCheckbuttonToggledCb(self, audio):
+    def _audioOutputCheckbuttonToggledCb(self, unused_audio):
         active = self.audio_output_checkbutton.get_active()
         if active:
             self.channels_combo.set_sensitive(True)
@@ -1074,7 +1069,7 @@ class RenderDialog(Loggable):
             if not self.video_output_checkbutton.get_active():
                 self.render_button.set_sensitive(False)
 
-    def _videoOutputCheckbuttonToggledCb(self, video):
+    def _videoOutputCheckbuttonToggledCb(self, unused_video):
         active = self.video_output_checkbutton.get_active()
         if active:
             self.scale_spinbutton.set_sensitive(True)
@@ -1102,7 +1097,7 @@ class RenderDialog(Loggable):
             # The user directly changed the video encoder combo.
             self.preferred_vencoder = vencoder
 
-    def _videoSettingsButtonClickedCb(self, button):
+    def _videoSettingsButtonClickedCb(self, unused_button):
         factory = get_combo_value(self.video_encoder_combo)
         self._elementSettingsDialog(factory, 'vcodecsettings')
 
@@ -1119,7 +1114,7 @@ class RenderDialog(Loggable):
             # The user directly changed the audio encoder combo.
             self.preferred_aencoder = aencoder
 
-    def _audioSettingsButtonClickedCb(self, button):
+    def _audioSettingsButtonClickedCb(self, unused_button):
         factory = get_combo_value(self.audio_encoder_combo)
         self._elementSettingsDialog(factory, 'acodecsettings')
 
