@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
 #
-#       pitivi/elements.py
+#       pitivi/timeline/elements.py
 #
 # Copyright (c) 2013, Mathieu Duponchelle <mduponchelle1@gmail.com>
 #
@@ -33,12 +33,12 @@ from datetime import datetime
 
 import weakref
 
-from gi.repository import Clutter, Gtk, GtkClutter, Cogl, GES, Gdk, Gst, GstController, GLib
-from pitivi.utils.timeline import Zoomable, EditingContext, Selection, SELECT, UNSELECT, SELECT_ADD, Selected
-from previewers import AudioPreviewer, VideoPreviewer, BORDER_WIDTH
+from gi.repository import Clutter, Gtk, GtkClutter, Cogl, GES, Gdk, Gst, GstController
+from pitivi.utils.timeline import Zoomable, EditingContext, SELECT, UNSELECT, SELECT_ADD, Selected
+from previewers import AudioPreviewer, VideoPreviewer
 
 import pitivi.configure as configure
-from pitivi.utils.ui import EXPANDED_SIZE, SPACING, KEYFRAME_SIZE, CONTROL_WIDTH
+from pitivi.utils.ui import EXPANDED_SIZE, SPACING, KEYFRAME_SIZE, CONTROL_WIDTH, create_cogl_color
 
 # Colors for keyframes and clips (RGBA)
 KEYFRAME_LINE_COLOR = (237, 212, 0, 255)  # "Tango" yellow
@@ -46,26 +46,15 @@ KEYFRAME_NORMAL_COLOR = Clutter.Color.new(0, 0, 0, 200)
 KEYFRAME_SELECTED_COLOR = Clutter.Color.new(200, 200, 200, 200)
 CLIP_SELECTED_OVERLAY_COLOR = Clutter.Color.new(60, 60, 60, 100)
 GHOST_CLIP_COLOR = Clutter.Color.new(255, 255, 255, 50)
+TRANSITION_COLOR = create_cogl_color(35, 85, 125, 125)  # light blue
 
+BORDER_NORMAL_COLOR = create_cogl_color(100, 100, 100, 255)
+BORDER_SELECTED_COLOR = create_cogl_color(200, 200, 10, 255)
 
-def get_preview_for_object(bElement, timeline):
-    # FIXME: special preview for transitions, titles
-    if not isinstance(bElement.get_parent(), GES.UriClip):
-        return Clutter.Actor()
-
-    track_type = bElement.get_track_type()
-    if track_type == GES.TrackType.AUDIO:
-        previewer = AudioPreviewer(bElement, timeline)
-        previewer.startLevelsDiscoveryWhenIdle()
-        return previewer
-    elif track_type == GES.TrackType.VIDEO:
-        if bElement.get_parent().is_image():
-            # TODO: return still image previewer
-            return Clutter.Actor()
-        else:
-            return VideoPreviewer(bElement, timeline)
-    else:
-        return Clutter.Actor()
+NORMAL_CURSOR = Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR)
+DRAG_CURSOR = Gdk.Cursor.new(Gdk.CursorType.HAND1)
+DRAG_LEFT_HANDLEBAR_CURSOR = Gdk.Cursor.new(Gdk.CursorType.LEFT_SIDE)
+DRAG_RIGHT_HANDLEBAR_CURSOR = Gdk.Cursor.new(Gdk.CursorType.RIGHT_SIDE)
 
 
 class RoundedRectangle(Clutter.Actor):
@@ -139,9 +128,6 @@ class RoundedRectangle(Clutter.Actor):
         self._border_width = width
         self.queue_redraw()
 
-    def get_border_color(color):
-        return self._border_color
-
     def set_border_color(self, color):
         self._border_color = color
         self.queue_redraw()
@@ -169,10 +155,10 @@ class Ghostclip(Clutter.Actor):
         self.props.width = width
 
     def update(self, priority, y, isControlledByBrother):
-        self.priority = priority
-        # Only tricky part of the code, can be called by the linked track element.
-        if priority < 0:
-            return
+        # Priority and y can be negative when dragging an asset to the ruler.
+        # Priority can also be negative when dragging a linked element.
+        self.priority = min(max(0, priority), self.nbrLayers)
+        y = max(0, y)
 
         # Here we make it so the calculation is the same for audio and video.
         if self.track_type == GES.TrackType.AUDIO and not isControlledByBrother:
@@ -187,22 +173,22 @@ class Ghostclip(Clutter.Actor):
                 y -= self.nbrLayers * (EXPANDED_SIZE + SPACING)
 
         # Would that be a new layer at the end or inserted ?
-        if priority == self.nbrLayers or y % (EXPANDED_SIZE + SPACING) < SPACING:
+        if self.priority == self.nbrLayers or y % (EXPANDED_SIZE + SPACING) < SPACING:
             self.shouldCreateLayer = True
             self.set_size(self.props.width, SPACING)
-            self.props.y = priority * (EXPANDED_SIZE + SPACING)
+            self.props.y = self.priority * (EXPANDED_SIZE + SPACING)
             if self.track_type == GES.TrackType.AUDIO:
                 self.props.y += self.nbrLayers * (EXPANDED_SIZE + SPACING)
             self.props.visible = True
         else:
             self.shouldCreateLayer = False
             # No need to mockup on the same layer
-            if self.bElement and priority == self.bElement.get_parent().get_layer().get_priority():
+            if self.bElement and self.priority == self.bElement.get_parent().get_layer().get_priority():
                 self.props.visible = False
             # We would be moving to an existing layer.
-            elif priority < self.nbrLayers:
+            elif self.priority < self.nbrLayers:
                 self.set_size(self.props.width, EXPANDED_SIZE)
-                self.props.y = priority * (EXPANDED_SIZE + SPACING) + SPACING
+                self.props.y = self.priority * (EXPANDED_SIZE + SPACING) + SPACING
                 if self.track_type == GES.TrackType.AUDIO:
                     self.props.y += self.nbrLayers * (EXPANDED_SIZE + SPACING)
                 self.props.visible = True
@@ -220,7 +206,6 @@ class TrimHandle(Clutter.Texture):
         Clutter.Texture.__init__(self)
 
         self.isLeft = isLeft
-        self.isSelected = False
         self.timelineElement = weakref.proxy(timelineElement)
         self.dragAction = Clutter.DragAction()
 
@@ -230,7 +215,6 @@ class TrimHandle(Clutter.Texture):
         self.set_reactive(True)
 
         self.add_action(self.dragAction)
-
         self.dragAction.connect("drag-begin", self._dragBeginCb)
         self.dragAction.connect("drag-end", self._dragEndCb)
         self.dragAction.connect("drag-progress", self._dragProgressCb)
@@ -240,15 +224,16 @@ class TrimHandle(Clutter.Texture):
 
         self.timelineElement.connect("enter-event", self._elementEnterEventCb)
         self.timelineElement.connect("leave-event", self._elementLeaveEventCb)
-        self.timelineElement.bElement.selected.connect("selected-changed", self._selectedChangedCb)
 
     def cleanup(self):
+        self.disconnect_by_func(self._enterEventCb)
+        self.disconnect_by_func(self._leaveEventCb)
         self.timelineElement.disconnect_by_func(self._elementEnterEventCb)
         self.timelineElement.disconnect_by_func(self._elementLeaveEventCb)
 
     #Callbacks
 
-    def _enterEventCb(self, actor, event):
+    def _enterEventCb(self, unused_actor, unused_event):
         self.timelineElement.set_reactive(False)
         for elem in self.timelineElement.get_children():
             elem.set_reactive(False)
@@ -256,39 +241,30 @@ class TrimHandle(Clutter.Texture):
 
         self.set_from_file(os.path.join(configure.get_pixmap_dir(), "trimbar-focused.png"))
         if self.isLeft:
-            self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.LEFT_SIDE))
+            self.timelineElement.timeline._container.embed.get_window().set_cursor(DRAG_LEFT_HANDLEBAR_CURSOR)
         else:
-            self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.RIGHT_SIDE))
+            self.timelineElement.timeline._container.embed.get_window().set_cursor(DRAG_RIGHT_HANDLEBAR_CURSOR)
 
-    def _leaveEventCb(self, actor, event):
+    def _leaveEventCb(self, unused_actor, event):
         self.timelineElement.set_reactive(True)
+        children = self.timelineElement.get_children()
 
-        actor = self.timelineElement.timeline._container.stage.get_actor_at_pos(Clutter.PickMode.ALL, event.x, event.y)
-        try:
-            element = actor.bElement
-            if element != self.timelineElement.bELement and not self.isSelected:
-                self.timelineElement.hideHandles()
-        except AttributeError:
-            if not self.isSelected:
-                self.timelineElement.hideHandles()
+        other_actor = self.timelineElement.timeline._container.stage.get_actor_at_pos(Clutter.PickMode.ALL, event.x, event.y)
+        if other_actor not in children:
+            self.timelineElement.hideHandles()
 
-        for elem in self.timelineElement.get_children():
+        for elem in children:
             elem.set_reactive(True)
         self.set_from_file(os.path.join(configure.get_pixmap_dir(), "trimbar-normal.png"))
-        self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.ARROW))
+        self.timelineElement.timeline._container.embed.get_window().set_cursor(NORMAL_CURSOR)
 
-    def _elementEnterEventCb(self, actor, event):
+    def _elementEnterEventCb(self, unused_actor, unused_event):
         self.show()
 
-    def _elementLeaveEventCb(self, actor, event):
-        if not self.isSelected:
-            self.hide()
+    def _elementLeaveEventCb(self, unused_actor, unused_event):
+        self.hide()
 
-    def _selectedChangedCb(self, selected, isSelected):
-        self.isSelected = isSelected
-        self.props.visible = isSelected
-
-    def _dragBeginCb(self, action, actor, event_x, event_y, modifiers):
+    def _dragBeginCb(self, unused_action, unused_actor, event_x, event_y, unused_modifiers):
         self.dragBeginStartX = event_x
         self.dragBeginStartY = event_y
         elem = self.timelineElement.bElement.get_parent()
@@ -312,7 +288,7 @@ class TrimHandle(Clutter.Texture):
         self._context.connect("clip-trim", self.clipTrimCb)
         self._context.connect("clip-trim-finished", self.clipTrimFinishedCb)
 
-    def _dragProgressCb(self, action, actor, delta_x, delta_y):
+    def _dragProgressCb(self, unused_action, unused_actor, delta_x, unused_delta_y):
         # We can't use delta_x here because it fluctuates weirdly.
         coords = self.dragAction.get_motion_coords()
         delta_x = coords[0] - self.dragBeginStartX
@@ -322,7 +298,7 @@ class TrimHandle(Clutter.Texture):
         self._context.editTo(new_start, self.timelineElement.bElement.get_parent().get_layer().get_priority())
         return False
 
-    def _dragEndCb(self, action, actor, event_x, event_y, modifiers):
+    def _dragEndCb(self, unused_action, unused_actor, unused_event_x, unused_event_y, unused_modifiers):
         self.timelineElement.setDragged(False)
         self._context.finish()
 
@@ -331,7 +307,7 @@ class TrimHandle(Clutter.Texture):
             elem.set_reactive(True)
 
         self.set_from_file(os.path.join(configure.get_pixmap_dir(), "trimbar-normal.png"))
-        self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.ARROW))
+        self.timelineElement.timeline._container.embed.get_window().set_cursor(NORMAL_CURSOR)
 
     def clipTrimCb(self, unused_TrimStartContext, tl_obj, position):
         # While a clip is being trimmed, ask the viewer to preview it
@@ -343,12 +319,14 @@ class TrimHandle(Clutter.Texture):
 
 
 class TimelineElement(Clutter.Actor, Zoomable):
-    def __init__(self, bElement, track, timeline):
-        """
-        @param bElement : the backend GES.TrackElement
-        @param track : the track to which the bElement belongs
-        @param timeline : the containing graphic timeline.
-        """
+    """
+    @ivar bElement: the backend element.
+    @type bElement: GES.TrackElement
+    @ivar timeline: the containing graphic timeline.
+    @type timeline: TimelineStage
+    """
+
+    def __init__(self, bElement, timeline):
         Zoomable.__init__(self)
         Clutter.Actor.__init__(self)
 
@@ -364,19 +342,32 @@ class TimelineElement(Clutter.Actor, Zoomable):
         self.source = None
         self.keyframedElement = None
         self.rightHandle = None
+        self.isSelected = False
         size = self.bElement.get_duration()
 
-        self._createBackground(track)
-        self._createPreview()
-        self._createBorder()
-        self._createMarquee()
+        self.background = self._createBackground()
+        self.background.set_position(0, 0)
+        self.add_child(self.background)
+
+        self.preview = self._createPreview()
+        self.add_child(self.preview)
+
+        self.border = self._createBorder()
+        self.add_child(self.border)
+
+        self.marquee = self._createMarquee()
+        self.add_child(self.marquee)
+
         self._createHandles()
+
+        self._linesMarker = self._createMarker()
+        self._keyframesMarker = self._createMarker()
+
         self._createGhostclip()
 
         self.update(True)
         self.set_reactive(True)
 
-        self.isSelected = False
         self._createMixingKeyframes()
 
         self._connectToEvents()
@@ -451,11 +442,10 @@ class TimelineElement(Clutter.Actor, Zoomable):
     def hideKeyframes(self):
         for keyframe in self.keyframes:
             self.remove_child(keyframe)
+        self.keyframes = []
 
         self.keyframesVisible = False
 
-        self.keyframes = []
-
         if self.isSelected:
             self.showKeyframes(self.default_element, self.default_prop)
 
@@ -463,9 +453,7 @@ class TimelineElement(Clutter.Actor, Zoomable):
 
     def setKeyframePosition(self, keyframe, value):
         x = self.nsToPixel(value.timestamp - self.bElement.props.in_point) - KEYFRAME_SIZE / 2
-        y = EXPANDED_SIZE - (value.value * EXPANDED_SIZE)
-
-        keyframe.set_z_position(2)
+        y = EXPANDED_SIZE - (value.value * EXPANDED_SIZE) - KEYFRAME_SIZE / 2
         keyframe.set_position(x, y)
 
     def drawLines(self, line=None):
@@ -491,29 +479,22 @@ class TimelineElement(Clutter.Actor, Zoomable):
             return
 
         values = self.source.get_all()
-
         if len(values) < 2 and self.bElement.props.duration > 0:
             self.source.unset_all()
             val = float(self.prop.default_value) / (self.prop.maximum - self.prop.minimum)
             self.source.set(self.bElement.props.in_point, val)
             self.source.set(self.bElement.props.duration + self.bElement.props.in_point, val)
 
-        lastPoint = None
-
         for keyframe in self.keyframes:
             self.remove_child(keyframe)
 
         self.keyframes = []
-
         values = self.source.get_all()
-
-        l = len(values)
+        values_count = len(values)
         for i, value in enumerate(values):
-            has_changable_time = True
-            if i == 0 or i == l - 1:
-                has_changable_time = False
-            self._createKeyframe(value, has_changable_time)
-            lastPoint = value
+            has_changeable_time = i > 0 and i < values_count - 1
+            keyframe = self._createKeyframe(value, has_changeable_time)
+            self.keyframes.append(keyframe)
 
         self.drawLines()
 
@@ -533,9 +514,21 @@ class TimelineElement(Clutter.Actor, Zoomable):
 
     # private API
 
+    def _createMarker(self):
+        marker = Clutter.Actor()
+        self.add_child(marker)
+        return marker
+
     def update(self, ease):
-        size = self.bElement.get_duration()
-        self.set_size(self.nsToPixel(size), EXPANDED_SIZE, ease)
+        start = self.bElement.get_start()
+        duration = self.bElement.get_duration()
+        # The calculation of the duration assumes that the start is always
+        # int(pixels_float). In that case, the rounding can add up and a pixel
+        # might be lost if we ignore the start of the clip.
+        size = self.nsToPixel(start + duration) - self.nsToPixel(start)
+        # Avoid elements to become invisible.
+        size = max(size, 1)
+        self.set_size(size, EXPANDED_SIZE, ease)
 
     def setDragged(self, dragged):
         brother = self.timeline.findBrother(self.bElement)
@@ -544,7 +537,7 @@ class TimelineElement(Clutter.Actor, Zoomable):
         self.isDragged = dragged
 
     def _createMixingKeyframes(self):
-        if self.bElement.get_track_type() == GES.TrackType.VIDEO:
+        if self.track_type == GES.TrackType.VIDEO:
             propname = "alpha"
         else:
             propname = "volume"
@@ -555,30 +548,23 @@ class TimelineElement(Clutter.Actor, Zoomable):
 
         self.hideKeyframes()
 
-    def _setKeyframePosition(self, keyframe, value):
-        x = self.nsToPixel(value.timestamp - self.bElement.props.in_point) - KEYFRAME_SIZE / 2
-        y = EXPANDED_SIZE - (value.value * EXPANDED_SIZE)
-
-        keyframe.set_z_position(2)
-        keyframe.set_position(x, y)
-
-    def _createKeyframe(self, value, has_changable_time):
-        keyframe = Keyframe(self, value, has_changable_time)
-        self.add_child(keyframe)
-        self.keyframes.append(keyframe)
+    def _createKeyframe(self, value, has_changeable_time):
+        keyframe = Keyframe(self, value, has_changeable_time)
+        self.insert_child_above(keyframe, self._keyframesMarker)
         self.setKeyframePosition(keyframe, value)
+        return keyframe
 
     def _createLine(self, keyframe, lastKeyframe, line):
         if not line:
             line = Line(self, keyframe, lastKeyframe)
             self.lines.append(line)
-            self.add_child(line)
+            self.insert_child_above(line, self._linesMarker)
 
         adj = self.nsToPixel(keyframe.value.timestamp - lastKeyframe.value.timestamp)
         opp = (lastKeyframe.value.value - keyframe.value.value) * EXPANDED_SIZE
         hyp = math.sqrt(adj ** 2 + opp ** 2)
-
-        if (hyp < 1):  # line length would be less tan one pixel
+        if hyp < 1:
+            # line length would be less than one pixel
             return
 
         sinX = opp / hyp
@@ -586,39 +572,42 @@ class TimelineElement(Clutter.Actor, Zoomable):
         line.props.height = KEYFRAME_SIZE
         line.props.rotation_angle_z = math.degrees(math.asin(sinX))
         line.props.x = self.nsToPixel(lastKeyframe.value.timestamp - self.bElement.props.in_point)
-        line.props.y = EXPANDED_SIZE - (EXPANDED_SIZE * lastKeyframe.value.value)
+        line.props.y = EXPANDED_SIZE - (EXPANDED_SIZE * lastKeyframe.value.value) - KEYFRAME_SIZE / 2
         line.canvas.invalidate()
 
     def _createGhostclip(self):
         pass
 
     def _createBorder(self):
-        self.border = RoundedRectangle(0, 0, 0, 0)
-        self.border.bElement = self.bElement
-        color = Cogl.Color()
-        color.init_from_4ub(100, 100, 100, 255)
-        self.border.set_border_color(color)
-        self.border.set_border_width(1)
-        self.border.set_position(0, 0)
-        self.add_child(self.border)
+        border = RoundedRectangle(0, 0, 0, 0)
+        border.bElement = self.bElement
+        border.set_border_color(BORDER_NORMAL_COLOR)
+        border.set_border_width(1)
+        border.set_position(0, 0)
+        return border
 
-    def _createBackground(self, track):
-        pass
+    def _createBackground(self):
+        raise NotImplementedError()
 
     def _createHandles(self):
         pass
 
     def _createPreview(self):
-        self.preview = get_preview_for_object(self.bElement, self.timeline)
-        self.add_child(self.preview)
+        if isinstance(self.bElement, GES.AudioUriSource):
+            previewer = AudioPreviewer(self.bElement, self.timeline)
+            previewer.startLevelsDiscoveryWhenIdle()
+            return previewer
+        if isinstance(self.bElement, GES.VideoUriSource):
+            return VideoPreviewer(self.bElement, self.timeline)
+        # TODO: GES.AudioTransition, GES.VideoTransition, GES.ImageSource, GES.TitleSource
+        return Clutter.Actor()
 
     def _createMarquee(self):
-        # TODO: difference between Actor.new() and Actor()?
-        self.marquee = Clutter.Actor()
-        self.marquee.bElement = self.bElement
-        self.marquee.set_background_color(CLIP_SELECTED_OVERLAY_COLOR)
-        self.marquee.props.visible = False
-        self.add_child(self.marquee)
+        marquee = Clutter.Actor()
+        marquee.bElement = self.bElement
+        marquee.set_background_color(CLIP_SELECTED_OVERLAY_COLOR)
+        marquee.props.visible = False
+        return marquee
 
     def _connectToEvents(self):
         self.dragAction = Clutter.DragAction()
@@ -634,7 +623,7 @@ class TimelineElement(Clutter.Actor, Zoomable):
         self.connect("button-press-event", self._clickedCb)
 
     def _getLayerForY(self, y):
-        if self.bElement.get_track_type() == GES.TrackType.AUDIO:
+        if self.track_type == GES.TrackType.AUDIO:
             y -= self.nbrLayers * (EXPANDED_SIZE + SPACING)
         priority = int(y / (EXPANDED_SIZE + SPACING))
         return priority
@@ -654,25 +643,27 @@ class TimelineElement(Clutter.Actor, Zoomable):
     def _dragBeginCb(self, action, actor, event_x, event_y, modifiers):
         pass
 
-    def _dragProgressCb(self, action, actor, delta_x, delta_y):
+    def _dragProgressCb(self, unused_action, unused_actor, unused_delta_x, unused_delta_y):
         return False
 
     def _dragEndCb(self, action, actor, event_x, event_y, modifiers):
         pass
 
-    def _durationChangedCb(self, element, duration):
+    def _durationChangedCb(self, unused_element, unused_duration):
         if self.keyframesVisible:
             self.updateKeyframes()
 
-    def _inpointChangedCb(self, element, inpoint):
+    def _inpointChangedCb(self, unused_element, unused_inpoint):
         if self.keyframesVisible:
             self.updateKeyframes()
 
-    def _selectedChangedCb(self, selected, isSelected):
+    def _selectedChangedCb(self, unused_selected, isSelected):
         self.isSelected = isSelected
         if not isSelected:
             self.hideKeyframes()
         self.marquee.props.visible = isSelected
+        color = BORDER_SELECTED_COLOR if isSelected else BORDER_NORMAL_COLOR
+        self.border.set_border_color(color)
 
 
 class Gradient(Clutter.Actor):
@@ -692,7 +683,7 @@ class Gradient(Clutter.Actor):
         self.set_content(self.canvas)
         self.canvas.invalidate()
 
-    def _drawCb(self, canvas, cr, width, height):
+    def _drawCb(self, unused_canvas, cr, unused_width, unused_height):
         cr.set_operator(cairo.OPERATOR_CLEAR)
         cr.paint()
         cr.set_operator(cairo.OPERATOR_OVER)
@@ -732,7 +723,7 @@ class Line(Clutter.Actor):
         self.previousKeyframe = lastKeyframe
         self.nextKeyframe = keyframe
 
-    def _drawCb(self, canvas, cr, width, unused_height):
+    def _drawCb(self, unused_canvas, cr, width, unused_height):
         """
         This is where we actually create the line segments for keyframe curves.
         We draw multiple lines (one-third of the height each) to add a "shadow"
@@ -777,37 +768,45 @@ class Line(Clutter.Actor):
 
     def _ungrab(self):
         self.timelineElement.set_reactive(True)
-        self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.ARROW))
+        self.timelineElement.timeline._container.embed.get_window().set_cursor(NORMAL_CURSOR)
 
-    def _clickedCb(self, actor, event):
+    def _clickedCb(self, unused_actor, event):
         if self.gotDragged:
             self.gotDragged = False
             return
-        x, y = self.transposeXY(event.x, event.y)
-        value = 1.0 - (y / EXPANDED_SIZE)
-        value = max(0.0, value)
-        value = min(1.0, value)
+        x, unused_y = self.transposeXY(event.x, event.y)
         timestamp = Zoomable.pixelToNs(x)
+        value = self._valueAtTimestamp(timestamp)
         self.timelineElement.addKeyframe(value, timestamp)
 
-    def _enterEventCb(self, actor, event):
-        self.timelineElement.set_reactive(False)
-        self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.HAND1))
+    def _valueAtTimestamp(self, timestamp):
+        timestamp_left = self.previousKeyframe.value.timestamp
+        value_left = self.previousKeyframe.value.value
+        timestamp_right = self.nextKeyframe.value.timestamp
+        value_right = self.nextKeyframe.value.value
+        height = value_right - value_left
+        duration = timestamp_right - timestamp_left
+        value = value_right - (timestamp_right - timestamp) * height / duration
+        return max(0.0, min(value, 1.0))
 
-    def _leaveEventCb(self, actor, event):
+    def _enterEventCb(self, unused_actor, unused_event):
+        self.timelineElement.set_reactive(False)
+        self.timelineElement.timeline._container.embed.get_window().set_cursor(DRAG_CURSOR)
+
+    def _leaveEventCb(self, unused_actor, unused_event):
         self._ungrab()
 
     def _motionEventCb(self, actor, event):
         pass
 
-    def _dragBeginCb(self, action, actor, event_x, event_y, modifiers):
+    def _dragBeginCb(self, unused_action, unused_actor, event_x, event_y, unused_modifiers):
         self.dragBeginStartX = event_x
         self.dragBeginStartY = event_y
         self.origY = self.props.y
         self.previousKeyframe.startDrag(event_x, event_y, self)
         self.nextKeyframe.startDrag(event_x, event_y, self)
 
-    def _dragProgressCb(self, action, actor, delta_x, delta_y):
+    def _dragProgressCb(self, unused_action, unused_actor, unused_delta_x, delta_y):
         self.gotDragged = True
         coords = self.dragAction.get_motion_coords()
         delta_x = coords[0] - self.dragBeginStartX
@@ -818,7 +817,7 @@ class Line(Clutter.Actor):
 
         return False
 
-    def _dragEndCb(self, action, actor, event_x, event_y, modifiers):
+    def _dragEndCb(self, unused_action, unused_actor, unused_event_x, unused_event_y, unused_modifiers):
         self.previousKeyframe.endDrag()
         self.nextKeyframe.endDrag()
         if self.timelineElement.timeline.getActorUnderPointer() != self:
@@ -849,21 +848,22 @@ class KeyframeMenu(GtkClutter.Actor):
         GtkClutter.Actor.hide(self)
         self.vbox.hide()
 
-    def _removeClickedCb(self, button):
+    def _removeClickedCb(self, unused_button):
         self.keyframe.remove()
 
 
 class Keyframe(Clutter.Actor):
     """
-    If has_changable_time is False, it means this is an edge keyframe.
+    @ivar has_changeable_time: if False, it means this is an edge keyframe.
+    @type has_changeable_time: bool
     """
 
-    def __init__(self, timelineElement, value, has_changable_time):
+    def __init__(self, timelineElement, value, has_changeable_time):
         Clutter.Actor.__init__(self)
 
         self.value = value
         self.timelineElement = weakref.proxy(timelineElement)
-        self.has_changable_time = has_changable_time
+        self.has_changeable_time = has_changeable_time
         self.lastClick = datetime.now()
 
         self.set_size(KEYFRAME_SIZE, KEYFRAME_SIZE)
@@ -875,7 +875,6 @@ class Keyframe(Clutter.Actor):
         self.dragAction.connect("drag-begin", self._dragBeginCb)
         self.dragAction.connect("drag-end", self._dragEndCb)
         self.dragAction.connect("drag-progress", self._dragProgressCb)
-        self.connect("key-press-event", self._keyPressEventCb)
         self.connect("enter-event", self._enterEventCb)
         self.connect("leave-event", self._leaveEventCb)
         self.connect("button-press-event", self._clickedCb)
@@ -892,11 +891,11 @@ class Keyframe(Clutter.Actor):
     def _unselect(self):
         self.timelineElement.set_reactive(True)
         self.set_background_color(KEYFRAME_NORMAL_COLOR)
-        self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.ARROW))
+        self.timelineElement.timeline._container.embed.get_window().set_cursor(NORMAL_CURSOR)
 
     def remove(self):
         # Can't remove edge keyframes !
-        if not self.has_changable_time:
+        if not self.has_changeable_time:
             return
 
         self.timelineElement.timeline.remove_child(self.menu)
@@ -908,7 +907,7 @@ class Keyframe(Clutter.Actor):
         if actor != self.menu:
             self.menu.hide()
 
-    def _clickedCb(self, actor, event):
+    def _clickedCb(self, unused_actor, event):
         if (event.modifier_state & Clutter.ModifierType.CONTROL_MASK):
             self.remove()
         elif (datetime.now() - self.lastClick).total_seconds() < 0.5:
@@ -916,15 +915,12 @@ class Keyframe(Clutter.Actor):
 
         self.lastClick = datetime.now()
 
-    def _keyPressEventCb(self, actor, event):
-        print event, dir(event)
-
-    def _enterEventCb(self, actor, event):
+    def _enterEventCb(self, unused_actor, unused_event):
         self.timelineElement.set_reactive(False)
         self.set_background_color(KEYFRAME_SELECTED_COLOR)
-        self.timelineElement.timeline._container.embed.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.HAND1))
+        self.timelineElement.timeline._container.embed.get_window().set_cursor(DRAG_CURSOR)
 
-    def _leaveEventCb(self, actor, event):
+    def _leaveEventCb(self, unused_actor, unused_event):
         self._unselect()
 
     def startDrag(self, event_x, event_y, line=None):
@@ -955,7 +951,7 @@ class Keyframe(Clutter.Actor):
 
         newValue = min(max(newValue, 0.0), 1.0)
 
-        if not self.has_changable_time:
+        if not self.has_changeable_time:
             newTs = self.lastTs
 
         self.timelineElement.source.unset(self.lastTs)
@@ -974,11 +970,11 @@ class Keyframe(Clutter.Actor):
             if not self.line:
                 self.timelineElement.timeline._container.seekInPosition(newTs + self.start)
 
-    def _dragBeginCb(self, action, actor, event_x, event_y, modifiers):
+    def _dragBeginCb(self, unused_action, unused_actor, event_x, event_y, unused_modifiers):
         self.dragProgressed = False
         self.startDrag(event_x, event_y)
 
-    def _dragProgressCb(self, action, actor, delta_x, delta_y):
+    def _dragProgressCb(self, unused_action, unused_actor, delta_x, delta_y):
         self.dragProgressed = True
         coords = self.dragAction.get_motion_coords()
         delta_x = coords[0] - self.dragBeginStartX
@@ -986,15 +982,15 @@ class Keyframe(Clutter.Actor):
         self.updateValue(delta_x, delta_y)
         return False
 
-    def _dragEndCb(self, action, actor, event_x, event_y, modifiers):
+    def _dragEndCb(self, unused_action, unused_actor, unused_event_x, unused_event_y, unused_modifiers):
         self.endDrag()
         if self.timelineElement.timeline.getActorUnderPointer() != self:
             self._unselect()
 
 
 class URISourceElement(TimelineElement):
-    def __init__(self, bElement, track, timeline):
-        TimelineElement.__init__(self, bElement, track, timeline)
+    def __init__(self, bElement, timeline):
+        TimelineElement.__init__(self, bElement, timeline)
         self.gotDragged = False
 
     # public API
@@ -1018,39 +1014,34 @@ class URISourceElement(TimelineElement):
         self.add_child(self.leftHandle)
         self.add_child(self.rightHandle)
 
-    def _createBackground(self, track):
-        if track.type == GES.TrackType.AUDIO:
+    def _createBackground(self):
+        if self.track_type == GES.TrackType.AUDIO:
             # Audio clips go from dark green to light green
             # (27, 46, 14, 255) to (73, 108, 33, 255)
-            self.background = Gradient(27, 46, 14, 73, 108, 33)
+            background = Gradient(27, 46, 14, 73, 108, 33)
         else:
             # Video clips go from almost black to gray
             # (15, 15, 15, 255) to (45, 45, 45, 255)
-            self.background = Gradient(15, 15, 15, 45, 45, 45)
-
-        self.background.bElement = self.bElement
-
-        self.add_child(self.background)
-        self.background.set_position(0, 0)
+            background = Gradient(15, 15, 15, 45, 45, 45)
+        background.bElement = self.bElement
+        return background
 
     # Callbacks
-    def _clickedCb(self, action, actor):
+    def _clickedCb(self, unused_action, unused_actor):
         #TODO : Let's be more specific, masks etc ..
-        children = self.bElement.get_toplevel_parent().get_children(True)
-        selection = filter(lambda elem: isinstance(elem, GES.Source), children)
-
         mode = SELECT
-
-        if self.timeline._container._controlMask and not self.bElement.selected:
-            mode = SELECT_ADD
-            self.timeline.current_group.add(self.bElement.get_toplevel_parent())
-        elif self.timeline._container._controlMask:
-            self.timeline.current_group.remove(self.bElement.get_toplevel_parent())
-            mode = UNSELECT
-        elif not self.bElement.selected.selected:
+        if self.timeline._container._controlMask:
+            if not self.bElement.selected:
+                mode = SELECT_ADD
+                self.timeline.current_group.add(self.bElement.get_toplevel_parent())
+            else:
+                self.timeline.current_group.remove(self.bElement.get_toplevel_parent())
+                mode = UNSELECT
+        elif not self.bElement.selected:
             GES.Container.ungroup(self.timeline.current_group, False)
             self.timeline.current_group = GES.Group()
             self.timeline.current_group.add(self.bElement.get_toplevel_parent())
+            self.timeline._container.gui.switchContextTab(self.bElement)
 
         children = self.bElement.get_toplevel_parent().get_children(True)
         selection = filter(lambda elem: isinstance(elem, GES.Source), children)
@@ -1134,18 +1125,15 @@ class URISourceElement(TimelineElement):
             self.brother.ghostclip.props.visible = False
 
         if self.ghostclip.shouldCreateLayer:
-            self.timeline.insertLayer(self.ghostclip)
+            self.timeline.createLayerForGhostClip(self.ghostclip)
 
         if self.gotDragged:
             self._context.editTo(new_start, priority)
             self._context.finish()
 
     def cleanup(self):
-        if self.preview:
-            try:
-                self.preview.cleanup()
-            except AttributeError:  # preview was just an actor.
-                pass
+        if self.preview and not type(self.preview) is Clutter.Actor:
+            self.preview.cleanup()
         self.leftHandle.cleanup()
         self.leftHandle = None
         self.rightHandle.cleanup()
@@ -1154,19 +1142,16 @@ class URISourceElement(TimelineElement):
 
 
 class TransitionElement(TimelineElement):
-    def __init__(self, bElement, track, timeline):
-        TimelineElement.__init__(self, bElement, track, timeline)
+    def __init__(self, bElement, timeline):
+        TimelineElement.__init__(self, bElement, timeline)
         self.isDragged = True
         self.set_reactive(True)
 
-    def _createBackground(self, track):
-        self.background = RoundedRectangle(0, 0, 0, 0)
-        color = Cogl.Color()
-        color.init_from_4ub(35, 85, 125, 125)  # light blue
-        self.background.set_color(color)
-        self.background.set_border_width(1)
-        self.background.set_position(0, 0)
-        self.add_child(self.background)
+    def _createBackground(self):
+        background = RoundedRectangle(0, 0, 0, 0)
+        background.set_color(TRANSITION_COLOR)
+        background.set_border_width(1)
+        return background
 
     def _selectedChangedCb(self, selected, isSelected):
         TimelineElement._selectedChangedCb(self, selected, isSelected)
@@ -1177,6 +1162,6 @@ class TransitionElement(TimelineElement):
             self.timeline._container.app.gui.trans_list.deactivate()
 
     def _clickedCb(self, action, actor):
-        selection = set([self.bElement])
+        selection = {self.bElement}
         self.timeline.selection.setSelection(selection, SELECT)
         return False
