@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
 #
-#       pitivi/previewers
+#       pitivi/timeline/previewers.py
 #
 # Copyright (c) 2013, Daniel Thul <daniel.thul@gmail.com>
 #
@@ -24,12 +24,9 @@ from datetime import datetime, timedelta
 from gi.repository import Clutter, Gst, GLib, GdkPixbuf, Cogl, GES
 from random import randrange
 import cairo
-import hashlib
-import multiprocessing
 import numpy
 import os
 import pickle
-import resource
 import sqlite3
 
 # Our C module optimizing waveforms rendering
@@ -38,19 +35,20 @@ import renderer
 from pitivi.settings import get_dir, xdg_cache_home
 from pitivi.utils.signal import Signallable
 from pitivi.utils.loggable import Loggable
+from pitivi.utils.misc import binary_search, filename_from_uri, quantize, quote_uri, hash_file, format_ns
+from pitivi.utils.system import CPUUsageTracker
 from pitivi.utils.timeline import Zoomable
-from pitivi.utils.ui import EXPANDED_SIZE
-from pitivi.utils.misc import filename_from_uri, quote_uri, hash_file
 from pitivi.utils.ui import CONTROL_WIDTH
+from pitivi.utils.ui import EXPANDED_SIZE
 
 
-WAVEFORMS_CPU_USAGE = 30 * multiprocessing.cpu_count()
+WAVEFORMS_CPU_USAGE = 30
 
 # A little lower as it's more fluctuating
-THUMBNAILS_CPU_USAGE = 20 * multiprocessing.cpu_count()
+THUMBNAILS_CPU_USAGE = 20
 
-INTERVAL = 500000  # For the waveform update interval.
-BORDER_WIDTH = 3  # For the timeline elements
+THUMB_MARGIN_PX = 3
+WAVEFORM_UPDATE_INTERVAL = timedelta(microseconds=500000)
 MARGIN = 500  # For the waveforms, ensures we always have a little extra surface when scrolling while playing.
 
 """
@@ -109,9 +107,9 @@ class PreviewGenerator(Signallable):
     those classes
     """
 
-    # We only wan 1 instance of PipelineQueue to be used for all the
-    # generators
-    manager = PreviewGeneratorManager()
+    # We only want one instance of PreviewGeneratorManager to be used for
+    # all the generators.
+    __manager = PreviewGeneratorManager()
 
     __signals__ = {
         "done": [],
@@ -119,6 +117,9 @@ class PreviewGenerator(Signallable):
     }
 
     def __init__(self, track_type):
+        """
+        @param track_type : GES.TrackType.*
+        """
         Signallable.__init__(self)
         self.track_type = track_type
 
@@ -132,7 +133,7 @@ class PreviewGenerator(Signallable):
         """
         Let the PreviewGeneratorManager control our execution
         """
-        self.manager.addPipeline(self)
+        PreviewGenerator.__manager.addPipeline(self)
 
 
 class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
@@ -159,18 +160,17 @@ class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
         self._thumb_cb_id = None
         self._allAnimated = False
         self._running = False
-        self.thumb_period = long(0.5 * Gst.SECOND)  # TODO: get this from user settings
-        self.thumb_margin = BORDER_WIDTH
-        self.thumb_height = EXPANDED_SIZE - 2 * self.thumb_margin
+        # We should have one thumbnail per thumb_period.
+        # TODO: get this from the user settings
+        self.thumb_period = long(0.5 * Gst.SECOND)
+        self.thumb_height = EXPANDED_SIZE - 2 * THUMB_MARGIN_PX
         self.thumb_width = None  # will be set by self._setupPipeline()
 
         # Maps (quantized) times to Thumbnail objects
         self.thumbs = {}
         self.thumb_cache = get_cache_for_uri(self.uri)
 
-        # For CPU management
-        self.lastMoment = datetime.now()
-        self.lastUsage = resource.getrusage(resource.RUSAGE_SELF)
+        self.cpu_usage_tracker = CPUUsageTracker()
         self.interval = 500  # Every 0.5 second, reevaluate the situation
 
         # Connect signals and fire things up
@@ -247,41 +247,35 @@ class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
         which the next thumbnail will be generated. Even then, it will only
         happen when the gobject loop is idle to avoid blocking the UI.
         """
-        deltaTime = (datetime.now() - self.lastMoment).total_seconds()
-        deltaUsage = resource.getrusage(resource.RUSAGE_SELF).ru_utime - self.lastUsage.ru_utime
-        usage_percent = float(deltaUsage) / deltaTime * 100
-
+        usage_percent = self.cpu_usage_tracker.usage()
         if usage_percent < THUMBNAILS_CPU_USAGE:
             self.interval *= 0.9
             self.log('Thumbnailing sped up (+10%%) to a %.1f ms interval for "%s"' % (self.interval, filename_from_uri(self.uri)))
         else:
             self.interval *= 1.1
             self.log('Thumbnailing slowed down (-10%%) to a %.1f ms interval for "%s"' % (self.interval, filename_from_uri(self.uri)))
-
-        self.lastMoment = datetime.now()
-        self.lastUsage = resource.getrusage(resource.RUSAGE_SELF)
+        self.cpu_usage_tracker.reset()
         self._thumb_cb_id = GLib.timeout_add(self.interval, self._create_next_thumb)
 
-        return False
-
     def _startThumbnailingWhenIdle(self):
-        self.debug('Waiting for UI to become idle for "%s"' % filename_from_uri(self.uri))
+        self.debug('Waiting for UI to become idle for: %s', filename_from_uri(self.uri))
         GLib.idle_add(self._startThumbnailing, priority=GLib.PRIORITY_LOW)
 
     def _startThumbnailing(self):
-        self.debug('Now generating thumbnails for "%s"' % filename_from_uri(self.uri))
-        self.queue = []
+        if not self.pipeline:
+            # Can happen if stopGeneration is called because the clip has been
+            # removed from the timeline after the PreviewGeneratorManager
+            # started this job.
+            return
+        self.debug('Now generating thumbnails for: %s', filename_from_uri(self.uri))
         query_success, duration = self.pipeline.query_duration(Gst.Format.TIME)
         if not query_success or duration == -1:
-            self.debug("Could not determine duration of %s" % self.uri)
+            self.debug("Could not determine duration of: %s", self.uri)
             duration = self.duration
         else:
             self.duration = duration
 
-        current_time = 0
-        while current_time < duration:
-            self.queue.append(current_time)
-            current_time += self.thumb_period
+        self.queue = range(0, duration, self.thumb_period)
 
         self._checkCPU()
 
@@ -331,35 +325,32 @@ class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
         else:
             return False  # Stop the timer
 
+    def _get_thumb_duration(self):
+        thumb_duration_tmp = Zoomable.pixelToNs(self.thumb_width + THUMB_MARGIN_PX)
+        # quantize thumb length to thumb_period
+        thumb_duration = quantize(thumb_duration_tmp, self.thumb_period)
+        # make sure that the thumb duration after the quantization isn't smaller than before
+        if thumb_duration < thumb_duration_tmp:
+            thumb_duration += self.thumb_period
+        # make sure that we don't show thumbnails more often than thumb_period
+        return max(thumb_duration, self.thumb_period)
+
     def _addVisibleThumbnails(self):
         """
         Get the thumbnails to be displayed in the currently visible clip portion
         """
         self.remove_all_children()
-        old_thumbs = self.thumbs.copy()
+        old_thumbs = self.thumbs
         self.thumbs = {}
         self.wishlist = []
 
-        thumb_duration_tmp = Zoomable.pixelToNs(self.thumb_width + self.thumb_margin)
-
-        # quantize thumb length to thumb_period
-        # TODO: replace with a call to utils.misc.quantize:
-        thumb_duration = (thumb_duration_tmp // self.thumb_period) * self.thumb_period
-        # make sure that the thumb duration after the quantization isn't smaller than before
-        if thumb_duration < thumb_duration_tmp:
-            thumb_duration += self.thumb_period
-
-        # make sure that we don't show thumbnails more often than thumb_period
-        thumb_duration = max(thumb_duration, self.thumb_period)
-
+        thumb_duration = self._get_thumb_duration()
         element_left, element_right = self._get_visible_range()
-        # TODO: replace with a call to utils.misc.quantize:
-        element_left = (element_left // thumb_duration) * thumb_duration
+        element_left = quantize(element_left, thumb_duration)
 
-        current_time = element_left
-        while current_time < element_right:
+        for current_time in range(element_left, element_right, thumb_duration):
             thumb = Thumbnail(self.thumb_width, self.thumb_height)
-            thumb.set_position(Zoomable.nsToPixel(current_time), self.thumb_margin)
+            thumb.set_position(Zoomable.nsToPixel(current_time), THUMB_MARGIN_PX)
             self.add_child(thumb)
             self.thumbs[current_time] = thumb
             if current_time in self.thumb_cache:
@@ -370,7 +361,6 @@ class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
                     self.thumbs[current_time].set_from_gdkpixbuf(gdkpixbuf)
             else:
                 self.wishlist.append(current_time)
-            current_time += thumb_duration
         self._allAnimated = False
 
     def _get_wish(self):
@@ -384,19 +374,36 @@ class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
             if wish in self.queue:
                 return wish
 
-    def _setThumbnail(self, time, thumbnail):
-        # TODO: is "time" guaranteed to be nanosecond precise?
+    def _setThumbnail(self, time, pixbuf):
+        # Q: Is "time" guaranteed to be nanosecond precise?
+        # A: Not always.
         # => __tim says: "that's how it should be"
         # => also see gst-plugins-good/tests/icles/gdkpixbufsink-test
         # => Daniel: It is *not* nanosecond precise when we remove the videorate
         #            element from the pipeline
+        # => thiblahute: not the case with mpegts
+        original_time = time
+        if time in self.thumbs:
+            thumb = self.thumbs[time]
+        else:
+            sorted_times = sorted(self.thumbs.keys())
+            index = binary_search(sorted_times, time)
+            time = sorted_times[index]
+            thumb = self.thumbs[time]
+            if thumb.has_pixel_data:
+                # If this happens, it means the precision of the thumbnail
+                # generator is not good enough for the current thumbnail
+                # interval.
+                # We could consider shifting the thumbnails, but seems like
+                # too much trouble for something which does not happen in
+                # practice. My last words..
+                self.fixme("Thumbnail is already set for time: %s, %s",
+                           format_ns(time), format_ns(original_time))
+                return
+        thumb.set_from_gdkpixbuf_animated(pixbuf)
         if time in self.queue:
             self.queue.remove(time)
-
-        self.thumb_cache[time] = thumbnail
-
-        if time in self.thumbs:
-            self.thumbs[time].set_from_gdkpixbuf_animated(thumbnail)
+        self.thumb_cache[time] = pixbuf
 
     # Interface (Zoomable)
 
@@ -467,13 +474,15 @@ class VideoPreviewer(Clutter.ScrollActor, PreviewGenerator, Zoomable, Loggable):
             struct = message.get_structure()
             struct_name = struct.get_name()
             if struct_name == "preroll-pixbuf":
-                self._setThumbnail(struct.get_value("stream-time"), struct.get_value("pixbuf"))
+                stream_time = struct.get_value("stream-time")
+                pixbuf = struct.get_value("pixbuf")
+                self._setThumbnail(stream_time, pixbuf)
         elif message.type == Gst.MessageType.ASYNC_DONE and \
                 message.src == self.pipeline:
             self._checkCPU()
         return Gst.BusSyncReply.PASS
 
-    def _autoplugSelectCb(self, decode, pad, caps, factory):
+    def _autoplugSelectCb(self, unused_decode, unused_pad, unused_caps, factory):
         # Don't plug audio decoders / parsers.
         if "Audio" in factory.get_klass():
             return True
@@ -527,11 +536,13 @@ class Thumbnail(Clutter.Actor):
         #self.set_background_color(Clutter.Color.new(0, 100, 150, 100))
         self.set_opacity(0)
         self.set_size(self.width, self.height)
+        self.has_pixel_data = False
 
     def set_from_gdkpixbuf(self, gdkpixbuf):
         row_stride = gdkpixbuf.get_rowstride()
         pixel_data = gdkpixbuf.get_pixels()
         alpha = gdkpixbuf.get_has_alpha()
+        self.has_pixel_data = True
         if alpha:
             self.props.content.set_data(pixel_data, Cogl.PixelFormat.RGBA_8888,
                                         self.width, self.height, row_stride)
@@ -588,15 +599,15 @@ class ThumbnailCache(Loggable):
     def __getitem__(self, key):
         self._cur.execute("SELECT * FROM Thumbs WHERE Time = ?", (key,))
         row = self._cur.fetchone()
-        if row:
-            jpeg = row[1]
-            loader = GdkPixbuf.PixbufLoader.new()
-            # TODO: what do to if any of the following calls fails?
-            loader.write(jpeg)
-            loader.close()
-            pixbuf = loader.get_pixbuf()
-            return pixbuf
-        raise KeyError(key)
+        if not row:
+            raise KeyError(key)
+        jpeg = row[1]
+        loader = GdkPixbuf.PixbufLoader.new()
+        # TODO: what do to if any of the following calls fails?
+        loader.write(jpeg)
+        loader.close()
+        pixbuf = loader.get_pixbuf()
+        return pixbuf
 
     def __setitem__(self, key, value):
         success, jpeg = value.save_to_bufferv("jpeg", ["quality", None], ["90"])
@@ -607,10 +618,9 @@ class ThumbnailCache(Loggable):
         #Replace if the key already existed
         self._cur.execute("DELETE FROM Thumbs WHERE  time=?", (key,))
         self._cur.execute("INSERT INTO Thumbs VALUES (?,?)", (key, blob,))
-        #self._db.commit()
 
     def commit(self):
-        self.debug('Saving thumbnail cache file to disk for "%s"' % self._filename)
+        self.debug('Saving thumbnail cache file to disk for: %s', self._filename)
         self._db.commit()
         self.log("Saved thumbnail cache file: %s" % self._filehash)
 
@@ -627,8 +637,7 @@ class PipelineCpuAdapter(Loggable):
         self.pipeline = pipeline
         self.bus = self.pipeline.get_bus()
 
-        self.lastMoment = datetime.now()
-        self.lastUsage = resource.getrusage(resource.RUSAGE_SELF)
+        self.cpu_usage_tracker = CPUUsageTracker()
         self.rate = 1.0
         self.done = False
         self.ready = False
@@ -654,24 +663,20 @@ class PipelineCpuAdapter(Loggable):
         if self.done:
             return False
 
-        deltaTime = (datetime.now() - self.lastMoment).total_seconds()
-        deltaUsage = resource.getrusage(resource.RUSAGE_SELF).ru_utime - self.lastUsage.ru_utime
-        usage_percent = float(deltaUsage) / deltaTime * 100
+        usage_percent = self.cpu_usage_tracker.usage()
+        self.cpu_usage_tracker.reset()
+        if usage_percent >= WAVEFORMS_CPU_USAGE:
+            if self.rate < 0.1:
+                if not self.ready:
+                    self.ready = True
+                    self.pipeline.set_state(Gst.State.READY)
+                    res, self.lastPos = self.pipeline.query_position(Gst.Format.TIME)
+                return True
 
-        self.lastMoment = datetime.now()
-        self.lastUsage = resource.getrusage(resource.RUSAGE_SELF)
-
-        if usage_percent >= WAVEFORMS_CPU_USAGE and self.rate < 0.1:
-            if not self.ready:
-                self.ready = True
-                self.pipeline.set_state(Gst.State.READY)
-                res, self.lastPos = self.pipeline.query_position(Gst.Format.TIME)
-            return True
-
-        if usage_percent >= WAVEFORMS_CPU_USAGE and self.rate > 0.0:
-            self.rate *= 0.9
-            self.log('Pipeline rate slowed down (-10%%) to %.3f' % self.rate)
-        elif usage_percent < WAVEFORMS_CPU_USAGE:
+            if self.rate > 0.0:
+                self.rate *= 0.9
+                self.log('Pipeline rate slowed down (-10%%) to %.3f' % self.rate)
+        else:
             self.rate *= 1.1
             self.log('Pipeline rate sped up (+10%%) to %.3f' % self.rate)
 
@@ -695,7 +700,7 @@ class PipelineCpuAdapter(Loggable):
         # Keep the glib timer running:
         return True
 
-    def _messageCb(self, bus, message):
+    def _messageCb(self, unused_bus, message):
         if not self.ready:
             return
         if message.type == Gst.MessageType.STATE_CHANGED:
@@ -721,6 +726,7 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
         Zoomable.__init__(self)
         Loggable.__init__(self)
         PreviewGenerator.__init__(self, GES.TrackType.AUDIO)
+        self.pipeline = None
         self.discovered = False
         self.bElement = bElement
         self._uri = quote_uri(bElement.props.uri)  # Guard against malformed URIs
@@ -734,8 +740,6 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
         self._num_failures = 0
         self.lastUpdate = datetime.now()
 
-        self.interval = timedelta(microseconds=INTERVAL)
-
         self.current_geometry = (-1, -1)
 
         self.adapter = None
@@ -747,7 +751,7 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
         self._callback_id = 0
 
     def startLevelsDiscoveryWhenIdle(self):
-        self.debug('Waiting for UI to become idle for "%s"' % filename_from_uri(self._uri))
+        self.debug('Waiting for UI to become idle for: %s', filename_from_uri(self._uri))
         GLib.idle_add(self._startLevelsDiscovery, priority=GLib.PRIORITY_LOW)
 
     def _startLevelsDiscovery(self):
@@ -764,35 +768,32 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
             self._launchPipeline()
 
     def _launchPipeline(self):
-        self.debug('Now generating waveforms for "%s"' % filename_from_uri(self._uri))
+        self.debug('Now generating waveforms for: %s', filename_from_uri(self._uri))
         self.peaks = None
         self.pipeline = Gst.parse_launch("uridecodebin name=decode uri=" + self._uri + " ! audioconvert ! level name=wavelevel interval=10000000 post-messages=true ! fakesink qos=false name=faked")
         faked = self.pipeline.get_by_name("faked")
         faked.props.sync = True
-        self._level = self.pipeline.get_by_name("wavelevel")
+        self._wavelevel = self.pipeline.get_by_name("wavelevel")
         decode = self.pipeline.get_by_name("decode")
         decode.connect("autoplug-select", self._autoplugSelectCb)
         bus = self.pipeline.get_bus()
         bus.add_signal_watch()
 
         self.nSamples = self.bElement.get_parent().get_asset().get_duration() / 10000000
-        bus.connect("message", self._messageCb)
+        bus.connect("message", self._busMessageCb)
         self.becomeControlled()
 
-    def set_size(self, width, height):
+    def set_size(self, unused_width, unused_height):
         if self.discovered:
             self._maybeUpdate()
 
-    def updateOffset(self):
-        print self.timeline.get_scroll_point().x
-
     def zoomChanged(self):
         self._maybeUpdate()
 
     def _maybeUpdate(self):
         if self.discovered:
             self.log('Checking if the waveform for "%s" needs to be redrawn' % self._uri)
-            if datetime.now() - self.lastUpdate > self.interval:
+            if datetime.now() - self.lastUpdate > WAVEFORM_UPDATE_INTERVAL:
                 self.lastUpdate = datetime.now()
                 self._compute_geometry()
             else:
@@ -802,44 +803,42 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
 
     def _compute_geometry(self):
         self.log("Computing the clip's geometry for waveforms")
+        width_px = self.nsToPixel(self.bElement.props.duration)
+        if width_px <= 0:
+            return
         start = self.timeline.get_scroll_point().x - self.nsToPixel(self.bElement.props.start)
         start = max(0, start)
-        end = min(self.timeline.get_scroll_point().x + self.timeline._container.get_allocation().width - CONTROL_WIDTH + MARGIN,
-                  self.nsToPixel(self.bElement.props.duration))
-
-        pixelWidth = self.nsToPixel(self.bElement.props.duration)
-
-        if pixelWidth <= 0:
-            return
-
-        real_duration = self.bElement.get_parent().get_asset().get_duration()
-
-        # We need to take duration and inpoint into account.
-
-        nbSamples = self.nbSamples
-        startOffsetSamples = 0
-
-        if self.bElement.props.duration != 0:
-            nbSamples = self.nbSamples / (float(real_duration) / float(self.bElement.props.duration))
-        if self.bElement.props.in_point != 0:
-            startOffsetSamples = self.nbSamples / (float(real_duration) / float(self.bElement.props.in_point))
-
-        self.start = int(start / pixelWidth * nbSamples + startOffsetSamples)
-        self.end = int(end / pixelWidth * nbSamples + startOffsetSamples)
-
+        # Take into account the timeline width, to avoid building
+        # huge clips when the timeline is zoomed in a lot.
+        timeline_width = self.timeline._container.get_allocation().width - CONTROL_WIDTH
+        end = min(width_px,
+                  self.timeline.get_scroll_point().x + timeline_width + MARGIN)
         self.width = int(end - start)
-
         if self.width < 0:  # We've been called at a moment where size was updated but not scroll_point.
             return
 
-        self.canvas.set_size(self.width, 65)
+        # We need to take duration and inpoint into account.
+        asset_duration = self.bElement.get_parent().get_asset().get_duration()
+        if self.bElement.props.duration:
+            nbSamples = self.nbSamples / (float(asset_duration) / float(self.bElement.props.duration))
+        else:
+            nbSamples = self.nbSamples
+        if self.bElement.props.in_point:
+            startOffsetSamples = self.nbSamples / (float(asset_duration) / float(self.bElement.props.in_point))
+        else:
+            startOffsetSamples = 0
+
+        self.start = int(start / width_px * nbSamples + startOffsetSamples)
+        self.end = int(end / width_px * nbSamples + startOffsetSamples)
+
+        self.canvas.set_size(self.width, EXPANDED_SIZE)
         Clutter.Actor.set_size(self, self.width, EXPANDED_SIZE)
         self.set_position(start, self.props.y)
         self.canvas.invalidate()
 
     def _prepareSamples(self):
         # Let's go mono.
-        if (len(self.peaks) > 1):
+        if len(self.peaks) > 1:
             samples = (numpy.array(self.peaks[0]) + numpy.array(self.peaks[1])) / 2
         else:
             samples = numpy.array(self.peaks[0])
@@ -857,8 +856,8 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
         if self.adapter:
             self.adapter.stop()
 
-    def _messageCb(self, bus, message):
-        if message.src == self._level:
+    def _busMessageCb(self, bus, message):
+        if message.src == self._wavelevel:
             s = message.get_structure()
             p = None
             if s:
@@ -901,7 +900,7 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
                              " for the %ith time, trying again with no rate "
                              " modulation", message.parse_error(),
                              self._num_failures)
-                bus.disconnect_by_func(self._messageCb)
+                bus.disconnect_by_func(self._busMessageCb)
                 self._launchPipeline()
                 self.becomeControlled()
             else:
@@ -926,15 +925,15 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
                     self.adapter = PipelineCpuAdapter(self.pipeline)
                     self.adapter.start()
 
-    def _autoplugSelectCb(self, decode, pad, caps, factory):
+    def _autoplugSelectCb(self, unused_decode, unused_pad, unused_caps, factory):
         # Don't plug video decoders / parsers.
         if "Video" in factory.get_klass():
             return True
         return False
 
-    def _drawContentCb(self, canvas, cr, surf_w, surf_h):
-        cr.set_operator(cairo.OPERATOR_CLEAR)
-        cr.paint()
+    def _drawContentCb(self, unused_canvas, context, unused_surf_w, unused_surf_h):
+        context.set_operator(cairo.OPERATOR_CLEAR)
+        context.paint()
         if not self.discovered:
             return
 
@@ -943,9 +942,9 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
 
         self.surface = renderer.fill_surface(self.samples[self.start:self.end], int(self.width), int(EXPANDED_SIZE))
 
-        cr.set_operator(cairo.OPERATOR_OVER)
-        cr.set_source_surface(self.surface, 0, 0)
-        cr.paint()
+        context.set_operator(cairo.OPERATOR_OVER)
+        context.set_source_surface(self.surface, 0, 0)
+        context.paint()
 
     def _scrolledCb(self, unused):
         self._maybeUpdate()
@@ -959,8 +958,11 @@ class AudioPreviewer(Clutter.Actor, PreviewGenerator, Zoomable, Loggable):
         if self.adapter is not None:
             self.adapter.stop()
             self.adapter = None
-        self.pipeline.set_state(Gst.State.NULL)
-        self.pipeline.get_state(Gst.CLOCK_TIME_NONE)
+
+        if self.pipeline:
+            self.pipeline.set_state(Gst.State.NULL)
+            self.pipeline.get_state(Gst.CLOCK_TIME_NONE)
+
         PreviewGenerator.emit(self, "done")
 
     def cleanup(self):
