@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       pitivi/medialibrary.py
+#       pitivi/titleeditor.py
 #
 # Copyright (c) 2012, Matas Brazdeikis <matas@brazdeikis.lt>
 #
@@ -96,7 +96,7 @@ class AttrIterator():
 def get_iterator(self):
     tmplist = []
 
-    def fil(val, data):
+    def fil(val, unused_data):
         tmplist.append(val)
         return False
 
@@ -364,7 +364,7 @@ class PangoBuffer(Gtk.TextBuffer):
             intervals.extend(tmpint)
         return tagdict
 
-    def get_text(self, start=None, end=None, include_hidden_chars=True):
+    def get_text(self, start=None, end=None, unused_include_hidden_chars=True):
         tagdict = self.get_tags()
         if not start:
             start = self.get_start_iter()
@@ -570,7 +570,11 @@ class InteractivePangoBuffer(PangoBuffer):
 
 
 class TitleEditor(Loggable):
-    def __init__(self, instance, uimap):
+    """
+    Widget for configuring the selected title.
+    """
+
+    def __init__(self, instance, unused_uimap):
         Loggable.__init__(self)
         Signallable.__init__(self)
         self.app = instance
@@ -606,7 +610,7 @@ class TitleEditor(Loggable):
         self.textarea = builder.get_object("textview")
         self.markup_button = builder.get_object("markupToggle")
         toolbar = builder.get_object("toolbar")
-        toolbar.get_style_context().add_class("inline-toolbar")
+        toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
 
         buttons = ["bold", "italic", "font", "font_fore_color", "back_color"]
         for button in buttons:
@@ -726,7 +730,7 @@ class TitleEditor(Loggable):
             else:
                 self.bt["back_color"].set_visible(False)
 
-    def _updateSourceText(self, updated_obj):
+    def _updateSourceText(self, unused_updated_obj):
         if self.source is not None:
             if self.markup_button.get_active():
                 text = self.textbuffer.get_text(self.textbuffer.get_start_iter(),
@@ -775,7 +779,7 @@ class TitleEditor(Loggable):
         This can be called either from the title editor in _createCb, or by
         track.py to set the source to None.
         """
-        self.debug("Source set to %s", str(source))
+        self.debug("Source set to %s", source)
         self.source = source
         self._reset()
         self.created = created
@@ -820,7 +824,7 @@ class TitleEditor(Loggable):
         self.app.gui.viewer.target.disconnect_by_func(self.drag_release_event)
         self._signals_connected = False
 
-    def drag_press_event(self, widget, event):
+    def drag_press_event(self, unused_widget, event):
         if event.button == 1:
             self._drag_events = [(event.x, event.y)]
             #Update drag by drag event change, but not too often
@@ -858,18 +862,18 @@ class TitleEditor(Loggable):
         else:
             return False
 
-    def drag_notify_event(self, widget, event):
+    def drag_notify_event(self, unused_widget, event):
         if len(self._drag_events) > 0 and event.get_state() & Gdk.ModifierType.BUTTON1_MASK:
             self._drag_updated = True
             self._drag_events.append((event.x, event.y))
             st = self._drag_events[0]
             e = self._drag_events[-1]
 
-    def drag_release_event(self, widget, event):
+    def drag_release_event(self, unused_widget, unused_event):
         self._drag_events = []
 
-    def tab_switched(self, unused_notebook, arg1, arg2):
-        if arg2 == 2:
+    def tabSwitchedCb(self, unused_notebook, page_widget, unused_page_index):
+        if self.widget == page_widget:
             self._connect_signals()
         else:
             self._disconnect_signals()
