@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
 #
-#       transitions.py
+#       pitivi/transitions.py
 #
 # Copyright (c) 2012, Jean-François Fortin Tam <nekohayo@gmail.com>
 #
@@ -41,8 +41,11 @@ from pitivi.utils.ui import SPACING, PADDING
 
 
 class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
+    """
+    Widget for configuring the selected transition.
+    """
 
-    def __init__(self, instance, uiman):
+    def __init__(self, instance, unused_uiman):
         Gtk.VBox.__init__(self)
         Loggable.__init__(self)
         Signallable.__init__(self)
@@ -66,7 +69,7 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
         self.searchbar.pack_end(self.searchEntry, True, True, 0)
 
         self.props_widgets = Gtk.VBox()
-        borderTable = Gtk.Table(rows=2, columns=3)
+        borderTable = Gtk.Table(n_rows=2, n_columns=3)
 
         self.border_mode_normal = Gtk.RadioButton(group=None, label=_("Normal"))
         self.border_mode_loop = Gtk.RadioButton(group=self.border_mode_normal, label=_("Loop"))
@@ -79,7 +82,7 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
         # The ypadding is a hack to make the slider widget align with the radiobuttons.
         borderTable.attach(self.borderScale, 2, 3, 0, 2, ypadding=SPACING * 2)
 
-        self.invert_checkbox = Gtk.CheckButton(_("Reverse direction"))
+        self.invert_checkbox = Gtk.CheckButton(label=_("Reverse direction"))
         self.invert_checkbox.set_border_width(SPACING)
 
         self.props_widgets.add(borderTable)
@@ -107,7 +110,7 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
         # See also http://python.6.n6.nabble.com/Cannot-shrink-width-of-scrolled-textview-tp1945060.html
         #self.iconview_scrollwin.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
 
-        self.iconview = Gtk.IconView(self.storemodel)
+        self.iconview = Gtk.IconView(model=self.storemodel)
         self.iconview.set_pixbuf_column(COL_ICON)
         # We don't show text because we have a searchbar and the names are ugly
         #self.iconview.set_text_column(COL_NAME_TEXT)
@@ -145,13 +148,13 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
 
 # UI callbacks
 
-    def _transitionSelectedCb(self, event):
+    def _transitionSelectedCb(self, unused_event):
         transition_asset = self.getSelectedItem()
         if not transition_asset:
             # The user clicked between icons
             return False
 
-        self.debug("New transition type selected: %s" % transition_asset.get_id())
+        self.debug("New transition type selected: %s", transition_asset.get_id())
         if transition_asset.get_id() == "crossfade":
             self.props_widgets.set_sensitive(False)
         else:
@@ -164,13 +167,13 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
 
     def _borderScaleCb(self, range_changed):
         value = range_changed.get_value()
-        self.debug("User changed the border property to %s" % value)
+        self.debug("User changed the border property to %s", value)
         self.element.set_border(int(value))
         self.app.current_project.seeker.flush(True)
 
     def _invertCheckboxCb(self, widget):
         value = widget.get_active()
-        self.debug("User changed the invert property to %s" % value)
+        self.debug("User changed the invert property to %s", value)
         self.element.set_inverted(value)
         self.app.current_project.seeker.flush()
 
@@ -191,10 +194,10 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
             self.borderScale.add_mark(0, Gtk.PositionType.BOTTOM, _("Sharp"))
             self.borderScale.add_mark(25000, Gtk.PositionType.BOTTOM, _("Smooth"))
 
-    def _searchEntryChangedCb(self, entry):
+    def _searchEntryChangedCb(self, unused_entry):
         self.modelFilter.refilter()
 
-    def _searchEntryIconClickedCb(self, entry, unused, unsed1):
+    def _searchEntryIconClickedCb(self, entry, unused, unused_1):
         entry.set_text("")
 
 # GES callbacks
@@ -269,7 +272,7 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
         self.props_widgets.show_all()
         self.searchbar.show_all()
         self.selectTransition(transition_asset)
-        self.app.gui.switchContextTab("transitions")
+        self.app.gui.switchContextTab(element)
         # We REALLY want the infobar to be hidden as space is really constrained
         # and yet GTK 3.10 seems to be racy in showing/hiding infobars, so
         # this must happen *after* the tab has been made visible/switched to:
@@ -346,7 +349,7 @@ class TransitionsListWidget(Signallable, Gtk.VBox, Loggable):
             return None
         return self.modelFilter[path[0]][COL_TRANSITION_ASSET]
 
-    def _setRowVisible(self, model, iter, data):
+    def _setRowVisible(self, model, iter, unused_data):
         """
         Filters the icon view depending on the search results
         """
