@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       pitivi/log/loggable.py
+#       pitivi/utils/loggable.py
 #
 # Copyright (c) 2009, Alessandro Decina <alessandro.decina@collabora.co.uk>
 #
@@ -280,7 +280,7 @@ def getLevelName(level):
     @return: The name of the level
     @rtype: str
     """
-    assert isinstance(level, int) and level > 0 and level < 6, \
+    assert isinstance(level, int) and level > 0 and level < 7, \
         TypeError("Bad debug level")
     return getLevelNames()[level - 1]
 
@@ -427,8 +427,8 @@ def getFileLine(where=-1):
     @param where: how many frames to go back up, or function
     @type  where: int (negative) or function
 
-    @return: tuple of (file, line)
-    @rtype:  tuple of (str, int)
+    @return: tuple of (file, line, function_name)
+    @rtype:  tuple of (str, int, str)
     """
     co = None
     lineno = None
@@ -458,7 +458,7 @@ def getFileLine(where=-1):
             stackFrame = stackFrame.f_back
 
     if not co:
-        return "<unknown file>", 0
+        return "<unknown file>", 0, None
 
     return scrubFilename(co.co_filename), lineno, name
 
@@ -519,41 +519,28 @@ def doLog(level, object, category, format, args, where=-1, filePath=None, line=N
         message = format % args
     else:
         message = format
+    funcname = None
 
-    # first all the unlimited ones
-    if _log_handlers:
+    if level > getCategoryLevel(category):
+        handlers = _log_handlers
+    else:
+        handlers = _log_handlers + _log_handlers_limited
+
+    if handlers:
         if filePath is None and line is None:
             (filePath, line, funcname) = getFileLine(where=where)
         ret['filePath'] = filePath
         ret['line'] = line
         if funcname:
             message = "\033[00m\033[32;01m%s:\033[00m %s" % (funcname, message)
-        for handler in _log_handlers:
+        for handler in handlers:
             try:
-                handler(level, object, category, file, line, message)
+                handler(level, object, category, filePath, line, message)
             except TypeError, e:
                 raise SystemError("handler %r raised a TypeError: %s" % (
                     handler, getExceptionMessage(e)))
 
-    if level > getCategoryLevel(category):
-        return ret
-
-    if _log_handlers_limited:
-        if filePath is None and line is None:
-            (filePath, line, funcname) = getFileLine(where=where)
-        ret['filePath'] = filePath
-        ret['line'] = line
-        if funcname:
-            message = "\033[00m\033[32;01m%s:\033[00m %s" % (funcname, message)
-        for handler in _log_handlers_limited:
-            # set this a second time, just in case there weren't unlimited
-            # loggers there before
-            try:
-                handler(level, object, category, filePath, line, message)
-            except TypeError:
-                raise SystemError("handler %r raised a TypeError" % handler)
-
-        return ret
+    return ret
 
 
 def errorObject(object, cat, format, *args):
@@ -563,9 +550,6 @@ def errorObject(object, cat, format, *args):
     """
     doLog(ERROR, object, cat, format, args)
 
-    # we do the import here because having it globally causes weird import
-    # errors if our gstreactor also imports .log, which brings in errors
-    # and pb stuff
     if args:
         raise SystemExit(format % args)
     else:
@@ -660,13 +644,10 @@ def stderrHandler(level, object, category, file, line, message):
     sys.stderr.flush()
 
 
-def _preformatLevels(noColorEnvVarName):
+def _preformatLevels(enableColorOutput):
     format = '%-5s'
 
-    if (noColorEnvVarName is not None
-        and (noColorEnvVarName not in os.environ
-             or not os.environ[noColorEnvVarName])):
-
+    if enableColorOutput:
         t = TerminalController()
         formatter = lambda level: ''.join((t.BOLD, getattr(t, COLORS[level]),
                             format % (_LEVEL_NAMES[level - 1], ), t.NORMAL))
@@ -681,7 +662,7 @@ def _preformatLevels(noColorEnvVarName):
 # setup functions
 
 
-def init(envVarName, enableColorOutput=False, enableCrackOutput=True):
+def init(envVarName, enableColorOutput=True, enableCrackOutput=True):
     """
     Initialize the logging system and parse the environment variable
     of the given name.
@@ -697,10 +678,7 @@ def init(envVarName, enableColorOutput=False, enableCrackOutput=True):
     global _ENV_VAR_NAME
     _ENV_VAR_NAME = envVarName
 
-    if enableColorOutput:
-        _preformatLevels(envVarName + "_NO_COLOR")
-    else:
-        _preformatLevels(None)
+    _preformatLevels(enableColorOutput)
 
     if envVarName in os.environ:
         # install a log handler that uses the value of the environment var
@@ -951,8 +929,7 @@ class BaseLoggable(object):
         marker to multiple elements at a time helps debugging.
         @param marker: A string write to the log.
         @type marker: str
-        @param level: The log level. It can be log.WARN, log.INFO,
-        log.DEBUG, log.ERROR or log.LOG.
+        @param level: The log level. It can be log.ERROR, etc.
         @type  level: int
         """
         logHandlers = {WARN: self.warning,
@@ -1024,23 +1001,6 @@ class BaseLoggable(object):
         return doLog(level, self.logObjectName(), self.logCategory,
                   format, args, where=where, **kwargs)
 
-    def warningFailure(self, failure, swallow=True):
-        """
-        Log a warning about a Twisted Failure. Useful as an errback handler:
-        d.addErrback(self.warningFailure)
-
-        @param swallow: whether to swallow the failure or not
-        @type  swallow: bool
-        """
-        if _canShortcutLogging(self.logCategory, WARN):
-            if swallow:
-                return
-            return failure
-        warningObject(self.logObjectName(), self.logCategory,
-            *self.logFunction(getFailureMessage(failure)))
-        if not swallow:
-            return failure
-
     def logFunction(self, *args):
         """Overridable log function.  Default just returns passed message."""
         return args
@@ -1057,145 +1017,6 @@ class BaseLoggable(object):
     def handleException(self, exc):
         self.warning(getExceptionMessage(exc))
 
-# Twisted helper stuff
-
-# private stuff
-_initializedTwisted = False
-
-# make a singleton
-__theTwistedLogObserver = None
-
-
-def _getTheTwistedLogObserver():
-    # used internally and in test
-    global __theTwistedLogObserver
-
-    if not __theTwistedLogObserver:
-        __theTwistedLogObserver = TwistedLogObserver()
-
-    return __theTwistedLogObserver
-
-
-# public helper methods
-
-
-def getFailureMessage(failure):
-    """
-    Return a short message based on L{twisted.python.failure.Failure}.
-    Tries to find where the exception was triggered.
-    """
-    exc = str(failure.type)
-    msg = failure.getErrorMessage()
-    if len(failure.frames) == 0:
-        return "failure %(exc)s: %(msg)s" % locals()
-
-    (func, filename, line, some, other) = failure.frames[-1]
-    filename = scrubFilename(filename)
-    return "failure %(exc)s at %(filename)s:%(line)s: %(func)s(): %(msg)s" % locals()
-
-
-def warningFailure(failure, swallow=True):
-    """
-    Log a warning about a Failure. Useful as an errback handler:
-    d.addErrback(warningFailure)
-
-    @param swallow: whether to swallow the failure or not
-    @type  swallow: bool
-    """
-    warning('', getFailureMessage(failure))
-    if not swallow:
-        return failure
-
-
-def logTwisted():
-    """
-    Integrate twisted's logger with our logger.
-
-    This is done in a separate method because calling this imports and sets
-    up a reactor.  Since we want basic logging working before choosing a
-    reactor, we need to separate these.
-    """
-    global _initializedTwisted
-
-    if _initializedTwisted:
-        return
-
-    debug('log', 'Integrating twisted logger')
-
-    # integrate twisted's logging with us
-    from twisted.python import log as tlog
-
-    # this call imports the reactor
-    # that is why we do this in a separate method
-    from twisted.spread import pb
-
-    # we don't want logs for pb.Error types since they
-    # are specifically raised to be handled on the other side
-    observer = _getTheTwistedLogObserver()
-    observer.ignoreErrors([pb.Error, ])
-    tlog.startLoggingWithObserver(observer.emit, False)
-
-    _initializedTwisted = True
-
-
-# we need an object as the observer because startLoggingWithObserver
-# expects a bound method
-
-
-class TwistedLogObserver(BaseLoggable):
-    """
-    Twisted log observer that integrates with our logging.
-    """
-    logCategory = "logobserver"
-
-    def __init__(self):
-        self._ignoreErrors = []  # Failure types
-
-    def emit(self, eventDict):
-        method = log  # by default, lowest level
-        edm = eventDict['message']
-        if not edm:
-            if eventDict['isError'] and 'failure' in eventDict:
-                f = eventDict['failure']
-                for failureType in self._ignoreErrors:
-                    r = f.check(failureType)
-                    if r:
-                        self.debug("Failure of type %r, ignoring" % failureType)
-                        return
-
-                self.log("Failure %r" % f)
-
-                method = debug  # tracebacks from errors at debug level
-                msg = "A twisted traceback occurred."
-                if getCategoryLevel("twisted") < WARN:
-                    msg += "  Run with debug level >= 2 to see the traceback."
-                # and an additional warning
-                warning('twisted', msg)
-                text = f.getTraceback()
-                safeprintf(sys.stderr, "\nTwisted traceback:\n")
-                safeprintf(sys.stderr, text + '\n')
-            elif 'format' in eventDict:
-                text = eventDict['format'] % eventDict
-            else:
-                # we don't know how to log this
-                return
-        else:
-            text = ' '.join(map(str, edm))
-
-        fmtDict = {'system': eventDict['system'],
-                   'text': text.replace("\n", "\n\t")}
-        msgStr = " [%(system)s] %(text)s\n" % fmtDict
-        # because msgstr can contain %, as in a backtrace, make sure we
-        # don't try to splice it
-        method('twisted', msgStr)
-
-    def ignoreErrors(self, *types):
-        for failureType in types:
-            self._ignoreErrors.append(failureType)
-
-    def clearIgnores(self):
-        self._ignoreErrors = []
-
 
 class Loggable(BaseLoggable):
     def __init__(self, logCategory=None):
