@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       utils.py
+#       pitivi/utils/misc.py
 #
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 # Copyright (c) 2009, Alessandro Decina <alessandro.d@gmail.com>
@@ -27,8 +27,9 @@ from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gst
 from gi.repository import Gtk
-from urllib import quote, unquote
-from urlparse import urlsplit, urlunsplit, urlparse
+from urllib import unquote
+from urlparse import urlsplit, urlparse
+import bisect
 import hashlib
 import os
 import struct
@@ -60,7 +61,9 @@ def between(a, b, c):
     return (a <= b) and (b <= c)
 
 
-def print_ns(time):
+def format_ns(time):
+    if time is None:
+        return None
     if time == Gst.CLOCK_TIME_NONE:
         return "CLOCK_TIME_NONE"
 
@@ -98,18 +101,6 @@ def bin_contains(bin, element):
     return False
 
 
-def in_devel():
-    """
-    Returns True if the current Pitivi instance is run from a git checkout
-    """
-    try:
-        # This code is the same as in the configure files
-        root_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-        return os.path.exists(os.path.join(root_dir, '.git'))
-    except:
-        return False
-
-
 #------------------------------ URI helpers   --------------------------------#
 def isWritable(path):
     """Check if the file/path is writable"""
@@ -124,7 +115,6 @@ def isWritable(path):
         # open(path, "rw") will IOError if the file doesn't already exist.
         # Therefore, simply check the directory permissions instead:
         return os.access(os.path.dirname(path), os.W_OK)
-    return True
 
 
 def uri_is_valid(uri):
@@ -133,7 +123,7 @@ def uri_is_valid(uri):
     Will also check if the size is valid (> 0).
 
     @param uri: The location to check
-    @type uri: C{URI}
+    @type uri: C{str}
     """
     return (Gst.uri_is_valid(uri) and
             Gst.uri_get_protocol(uri) == "file" and
@@ -144,7 +134,7 @@ def uri_is_reachable(uri):
     """ Check whether the given uri is reachable by GStreamer.
 
     @param uri: The location to check
-    @type uri: C{URI}
+    @type uri: C{str}
     @return: C{True} if the uri is reachable.
     @rtype: C{bool}
     """
@@ -160,13 +150,13 @@ def get_filesystem_encoding():
     return sys.getfilesystemencoding() or "utf-8"
 
 
-def path_from_uri(uri):
+def path_from_uri(raw_uri):
     """
-    Return a human-readable path that can be used with python's os.path
+    Return a path that can be used with Python's os.path.
     """
-    foo = urlparse(uri)
-    path = foo.netloc + foo.path
-    return unquote(path)
+    uri = urlparse(raw_uri)
+    assert uri.scheme == "file"
+    return unquote(uri.path)
 
 
 def filename_from_uri(uri):
@@ -258,7 +248,7 @@ def get_controllable_properties(element):
 
 def linkDynamic(element, target):
 
-    def pad_added(bin, pad, target):
+    def pad_added(unused_bin, pad, target):
         compatpad = target.get_compatible_pad(pad)
         if compatpad:
             pad.link_full(compatpad, Gst.PAD_LINK_CHECK_NOTHING)
@@ -337,14 +327,13 @@ infinity = Infinity()
 
 
 def profile(func, profiler_filename="result.prof"):
-    import os.path
     counter = 1
     output_filename = profiler_filename
     while os.path.exists(output_filename):
         output_filename = profiler_filename + str(counter)
         counter += 1
 
-    def _wrapper(*args, **kwargs):
+    def _wrapper(*unused_args, **kwargs):
         local_func = func
         cProfile.runctx("result = local_func(*args, **kwargs)", globals(), locals(),
                         filename=output_filename)
@@ -361,30 +350,26 @@ def quantize(input, interval):
     return (input // interval) * interval
 
 
-# Python re-implementation of binary search algorithm found here:
-# http://en.wikipedia.org/wiki/Binary_search
-#
-# This is the iterative version without the early termination branch, which
-# also tells us the element of A that are nearest to Value, if the element we
-# want is not found. This is useful for implementing edge snaping in the UI,
-# where we repeatedly search through a list of control points for the one
-# closes to the cursor. Because we don't care whether the cursor position
-# matches the list, this function returns the index of the lement closest to
-# value in the array.
+def binary_search(elements, value):
+    """Returns the index of the element closest to value.
 
-
-def binary_search(col, value):
-    low = 0
-    high = len(col)
-    while (low < high):
-        mid = (low + high) / 2
-        if (col[mid] < value):
-            low = mid + 1
-        else:
-            #can't be high = mid-1: here col[mid] >= value,
-            #so high can't be < mid if col[mid] == value
-            high = mid
-    return low
+    @param elements: A sorted list.
+    """
+    if not elements:
+        return -1
+    closest_index = bisect.bisect_left(elements, value, 0, len(elements) - 1)
+    element = elements[closest_index]
+    closest_distance = abs(element - value)
+    if closest_distance == 0:
+        return closest_index
+    for index in (closest_index - 1,):
+        if index < 0:
+            continue
+        distance = abs(elements[index] - value)
+        if closest_distance > distance:
+            closest_index = index
+            closest_distance = distance
+    return closest_index
 
 
 def argmax(func, seq):
