@@ -1,8 +1,8 @@
 #!/usr/bin/env python2
 #
-#       pipeline.py
+#       pitivi/utils/pipeline.py
 #
-# Copyright (C) 2012 Thibault Saunier <thibaul.saunier@collabora.com>
+# Copyright (C) 2012 Thibault Saunier <thibault.saunier@collabora.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -15,9 +15,7 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3, or (at your option)
@@ -28,7 +26,7 @@ High-level pipelines
 """
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.signal import Signallable
-from pitivi.utils.misc import print_ns
+from pitivi.utils.misc import format_ns
 
 from gi.repository import GLib
 from gi.repository import GObject
@@ -38,7 +36,6 @@ from gi.repository import GES
 MAX_RECOVERIES = 5
 
 
-# FIXME : define/document a proper hierarchy
 class PipelineError(Exception):
     pass
 
@@ -123,7 +120,7 @@ class Seeker(Signallable, Loggable):
                 self.emit('seek', position, format)
             except PipelineError as e:
                 self.error("Error while seeking to position:%s format: %r, reason: %s",
-                          print_ns(position), format, e)
+                          format_ns(position), format, e)
                 # if an exception happened while seeking, properly
                 # reset ourselves
                 return False
@@ -158,9 +155,10 @@ class SimplePipeline(Signallable, Loggable):
         "error": ["message", "details"]
     }
 
-    def __init__(self, pipeline, video_overlay):
+    def __init__(self, pipeline):
         Loggable.__init__(self)
         Signallable.__init__(self)
+
         self._pipeline = pipeline
         self._bus = self._pipeline.get_bus()
         self._bus.add_signal_watch()
@@ -169,7 +167,6 @@ class SimplePipeline(Signallable, Loggable):
         self._listeningInterval = 300  # default 300ms
         self._listeningSigId = 0
         self._duration = Gst.CLOCK_TIME_NONE
-        self.video_overlay = video_overlay
         self.lastPosition = long(0 * Gst.SECOND)
         self.pendingRecovery = False
         self._attempted_recoveries = 0
@@ -177,6 +174,20 @@ class SimplePipeline(Signallable, Loggable):
         self._next_seek = None
         self._timeout_async_id = 0
 
+        # Create a cluttersink element used for display. Subclasses must connect
+        # it to self._pipeline themselves
+        self._clutter_sink = Gst.ElementFactory.make("cluttersink", None)
+        if isinstance(pipeline, GES.Pipeline):
+            self._pipeline.preview_set_video_sink(self._clutter_sink)
+        else:
+            self._pipeline.set_property("video_sink", self._clutter_sink)
+
+    def connectWithViewer(self, viewer):
+        """
+        Connect the specified ViewerWidget so this pipeline can be displayed.
+        """
+        self._clutter_sink.props.texture = viewer.texture
+
     def release(self):
         """
         Release the L{Pipeline} and all used L{ObjectFactory} and
@@ -208,7 +219,7 @@ class SimplePipeline(Signallable, Loggable):
         @raises PipelineError: If the C{Gst.Pipeline} could not be changed to
         the requested state.
         """
-        self.debug("state:%r" % state)
+        self.debug("state set to: %r", state)
         res = self._pipeline.set_state(state)
         if res == Gst.StateChangeReturn.FAILURE:
             # reset to NULL
@@ -225,8 +236,8 @@ class SimplePipeline(Signallable, Loggable):
         @return: The current state.
         @rtype: C{State}
         """
-        change, state, pending = self._pipeline.get_state(0)
-        self.debug("change:%r, state:%r, pending:%r" % (change, state, pending))
+        change, state, pending = self._pipeline.get_state(timeout=0)  # No timeout
+        self.debug("change: %r, state: %r, pending: %r", change, state, pending)
         return state
 
     def play(self):
@@ -246,10 +257,12 @@ class SimplePipeline(Signallable, Loggable):
         # is only emitted every 300ms and the playhead jumps
         # during the playback.
         try:
-            self.emit("position", self.getPosition())
+            position = self.getPosition()
         except PipelineError:
             # Getting the position failed
-            pass
+            return
+        if position != Gst.CLOCK_TIME_NONE and position >= 0:
+            self.emit("position", position)
 
     def stop(self):
         """
@@ -275,7 +288,6 @@ class SimplePipeline(Signallable, Loggable):
         @rtype: L{long}
         @raise PipelineError: If the position couldn't be obtained.
         """
-        self.log("format %r" % format)
         try:
             res, cur = self._pipeline.query_position(format)
         except Exception, e:
@@ -285,20 +297,17 @@ class SimplePipeline(Signallable, Loggable):
         if not res:
             raise PipelineError("Couldn't get position")
 
-        self.log("Got position %s" % print_ns(cur))
+        self.log("Got position %s", format_ns(cur))
         return cur
 
     def getDuration(self, format=Gst.Format.TIME):
         """
         Get the duration of the C{Pipeline}.
         """
-        self.log("format %r" % format)
+        self.log("format %r", format)
 
         dur = self._getDuration(format)
-        if dur is None:
-            self.info("Invalid duration: None")
-        else:
-            self.log("Got duration %s" % print_ns(dur))
+        self.log("Got duration %s", format_ns(dur))
         if self._duration != dur:
             self.emit("duration-changed", dur)
 
@@ -306,7 +315,7 @@ class SimplePipeline(Signallable, Loggable):
 
         return dur
 
-    def activatePositionListener(self, interval=30):
+    def activatePositionListener(self, interval=500):
         """
         Activate the position listener.
 
@@ -375,13 +384,13 @@ class SimplePipeline(Signallable, Loggable):
         """
         if self._waiting_for_async_done is True:
             self._next_seek = (position, format)
-            self.info("Setting next seek to %s" % (str(self._next_seek)))
+            self.info("Setting next seek to %s", self._next_seek)
             return
 
         if format == Gst.Format.TIME:
-            self.debug("position : %s" % print_ns(position))
+            self.debug("position: %s", format_ns(position))
         else:
-            self.debug("position : %d , format:%d" % (position, format))
+            self.debug("position: %d, format: %d", position, format)
 
         # clamp between [0, duration]
         if format == Gst.Format.TIME:
@@ -422,7 +431,7 @@ class SimplePipeline(Signallable, Loggable):
             prev, new, pending = message.parse_state_changed()
 
             if message.src == self._pipeline:
-                self.debug("Pipeline change state prev:%r, new:%r, pending:%r" % (prev, new, pending))
+                self.debug("Pipeline change state prev: %r, new: %r, pending: %r", prev, new, pending)
 
                 emit_state_change = pending == Gst.State.VOID_PENDING
                 if prev == Gst.State.READY and new == Gst.State.PAUSED:
@@ -463,7 +472,7 @@ class SimplePipeline(Signallable, Loggable):
                 GLib.source_remove(self._timeout_async_id)
                 self._timeout_async_id = 0
         else:
-            self.log("%s [%r]" % (message.type, message.src))
+            self.log("%s [%r]", message.type, message.src)
 
     def _recover(self):
         if self._attempted_recoveries > MAX_RECOVERIES:
@@ -475,7 +484,7 @@ class SimplePipeline(Signallable, Loggable):
         self._pipeline.set_state(Gst.State.PAUSED)
         self._attempted_recoveries += 1
 
-    def _queryDurationAsync(self, *args, **kwargs):
+    def _queryDurationAsync(self, *unused_args, **unused_kwargs):
         try:
             self.getDuration()
         except:
@@ -490,12 +499,29 @@ class SimplePipeline(Signallable, Loggable):
         try:
             res, dur = self._pipeline.query_duration(format)
         except Exception, e:
-
             self.handleException(e)
             raise PipelineError("Couldn't get duration: %s" % e)
 
         if not res:
             raise PipelineError("Couldn't get duration: Returned None")
+        return dur
+
+
+class AssetPipeline(SimplePipeline):
+    """
+    Pipeline for playing a single clip.
+    """
+
+    def __init__(self, clip=None, name=None):
+        bPipeline = Gst.ElementFactory.make("playbin", name)
+        SimplePipeline.__init__(self, bPipeline)
+
+        self.clip = clip
+        if self.clip:
+            self.setClipUri(self.clip.props.uri)
+
+    def setClipUri(self, uri):
+        self._pipeline.set_property("uri", uri)
 
 
 class Pipeline(GES.Pipeline, SimplePipeline):
@@ -524,8 +550,9 @@ class Pipeline(GES.Pipeline, SimplePipeline):
 
     def __init__(self, pipeline=None):
         GES.Pipeline.__init__(self)
-        SimplePipeline.__init__(self, self, self)
+        SimplePipeline.__init__(self, self)
 
+        self._timeline = None
         self._seeker = Seeker()
         self._seeker.connect("seek", self._seekCb)
         self._seeker.connect("seek-relative", self._seekRelativeCb)
@@ -533,12 +560,10 @@ class Pipeline(GES.Pipeline, SimplePipeline):
     def _getDuration(self, format=Gst.Format.TIME):
         return self._timeline.get_duration()
 
-    def add_timeline(self, timeline):
-        if GES.Pipeline.add_timeline(self, timeline):
-            self._timeline = timeline
-            return True
-
-        return False
+    def set_timeline(self, timeline):
+        if not GES.Pipeline.set_timeline(self, timeline):
+            raise PipelineError("Cannot set the timeline to the pipeline")
+        self._timeline = timeline
 
     def release(self):
         """
@@ -571,12 +596,14 @@ class Pipeline(GES.Pipeline, SimplePipeline):
         cur_frame = int(round(position * framerate.num / float(Gst.SECOND * framerate.denom), 2))
         new_frame = cur_frame + frames_offset
         new_pos = long(new_frame * Gst.SECOND * framerate.denom / framerate.num)
-        Loggable.info(self, "From frame %d to %d at %f fps, seek to %s s" % (cur_frame,
-                    new_frame, framerate.num / framerate.denom,
-                    new_pos / float(Gst.SECOND)))
+        Loggable.info(self, "From frame %d to %d at %f fps, seek to %s s",
+                    cur_frame,
+                    new_frame,
+                    framerate.num / framerate.denom,
+                    new_pos / float(Gst.SECOND))
         self.simple_seek(new_pos)
 
-    def _seekCb(self, ruler, position, format):
+    def _seekCb(self, unused_ruler, position, unused_format):
         """
         The app's main seek method used when the user seeks manually.
 
