@@ -27,18 +27,25 @@ classes that help with UI drawing around the application
 """
 
 
-from decimal import Decimal
+import cairo
+import decimal
+import os
+import urllib
+
 from gettext import ngettext, gettext as _
+
+from gi.repository import Clutter
+from gi.repository import Cogl
 from gi.repository import GLib
 from gi.repository import GES
+from gi.repository import Gdk
+from gi.repository import Gio
 from gi.repository import Gst
 from gi.repository import Gtk
 from gi.repository.GstPbutils import DiscovererVideoInfo, DiscovererAudioInfo,\
-    DiscovererStreamInfo, DiscovererSubtitleInfo
-from itertools import izip
-from urllib import unquote
-import cairo
-import os
+    DiscovererStreamInfo, DiscovererSubtitleInfo, DiscovererInfo
+
+from pitivi.utils.misc import path_from_uri
 
 from pitivi.utils.loggable import doLog, ERROR
 
@@ -47,6 +54,7 @@ from pitivi.utils.loggable import doLog, ERROR
 ##
 # UI pixels information constants
 ##
+
 LAYER_HEIGHT_EXPANDED = 50
 LAYER_HEIGHT_COLLAPSED = 15
 TRACK_SPACING = 8
@@ -56,10 +64,12 @@ CONTROL_WIDTH = 250
 PADDING = 6
 SPACING = 10
 
-PLAYHEAD_WIDTH = 2
+PLAYHEAD_WIDTH = 1
 CANVAS_SPACING = 21
 KEYFRAME_SIZE = 8
 
+PLAYHEAD_COLOR = Clutter.Color.new(200, 0, 0, 255)
+
 # Layer creation blocking time in s
 LAYER_CREATION_BLOCK_TIME = 0.2
 
@@ -91,6 +101,26 @@ VIDEO_TRANSITION_TARGET_ENTRY = Gtk.TargetEntry.new("pitivi/video-transition", 0
 LAYER_CONTROL_TARGET_ENTRY = Gtk.TargetEntry.new("pitivi/layer-control", 0, TYPE_PITIVI_LAYER_CONTROL)
 
 
+def _get_settings(schema):
+    if schema not in Gio.Settings.list_schemas():
+        return None
+    return Gio.Settings(schema=schema)
+
+
+def _get_font(font_spec, default):
+    raw_font = default
+    settings = _get_settings("org.gnome.desktop.interface")
+    if settings:
+        if font_spec in settings.list_keys():
+            raw_font = settings.get_string(font_spec)
+    face = raw_font.rsplit(" ", 1)[0]
+    return cairo.ToyFontFace(face)
+
+NORMAL_FONT = _get_font("font-name", "Cantarell")
+DOCUMENT_FONT = _get_font("document-font-name", "Sans")
+MONOSPACE_FONT = _get_font("monospace-font-name", "Monospace")
+
+
 # ---------------------- ARGB color helper-------------------------------------#
 def pack_color_32(red, green, blue, alpha=0xFFFF):
     """Packs the specified 16bit color values in a 32bit RGBA value."""
@@ -169,8 +199,32 @@ def hex_to_rgb(value):
     return tuple(float(int(value[i:i + 2], 16)) / 255.0 for i in range(0, 6, 2))
 
 
-#------ Helper to help beatify indos so they can be displayed in the UI -----#
+def create_cogl_color(red, green, blue, alpha):
+    color = Cogl.Color()
+    color.init_from_4ub(red, green, blue, alpha)
+    return color
+
+
+def set_cairo_color(context, color):
+    if type(color) is Clutter.Color:
+        color = (color.red, color.green, color.blue)
+
+    if type(color) is Gdk.RGBA:
+        cairo_color = (float(color.red), float(color.green), float(color.blue))
+    elif type(color) is tuple:
+        # Cairo's set_source_rgb function expects values from 0.0 to 1.0
+        cairo_color = map(lambda x: max(0, min(1, x / 255.0)), color)
+    else:
+        raise Exception("Unexpected color parameter: %s, %s" % (type(color), color))
+    context.set_source_rgb(*cairo_color)
+
+
 def beautify_info(info):
+    """
+    Formats the specified info for display.
+
+    @type info: L{DiscovererInfo}
+    """
     ranks = {
         DiscovererVideoInfo: 0,
         DiscovererAudioInfo: 1,
@@ -190,20 +244,26 @@ def beautify_info(info):
             beautified_string = beautify_stream(stream)
         except NotImplementedError:
             doLog(ERROR, "Beautify", "None", "Cannot beautify %s", stream)
-        else:
-            if beautified_string is not None:
-                nice_streams_txts.append(beautified_string)
+            continue
+        if beautified_string:
+            nice_streams_txts.append(beautified_string)
 
-    return ("<b>" + info_name(info) + "</b>\n" +
-        "\n".join((nice for nice in nice_streams_txts)))
+    return ("<b>" + path_from_uri(info.get_uri()) + "</b>\n" +
+        "\n".join(nice_streams_txts))
 
 
 def info_name(info):
-    """Return a human-readable filename (without the path and quoting)."""
+    """
+    Return a human-readable filename (without the path and quoting).
+
+    @type info: L{GES.Asset} or L{DiscovererInfo}
+    """
     if isinstance(info, GES.Asset):
-        filename = unquote(os.path.basename(info.get_id()))
+        filename = urllib.unquote(os.path.basename(info.get_id()))
+    elif isinstance(info, DiscovererInfo):
+        filename = urllib.unquote(os.path.basename(info.get_uri()))
     else:
-        filename = unquote(os.path.basename(info.get_uri()))
+        raise Exception("Unsupported argument type: %s" % type(info))
     return GLib.markup_escape_text(filename)
 
 
@@ -335,30 +395,6 @@ def beautify_ETA(length):
     return ", ".join(parts)
 
 
-#--------------------- UI drawing helper -------------------------------------#
-# from http://cairographics.org/cookbook/roundedrectangles/
-def roundedrec(context, x, y, w, h, r=10):
-    "Draw a rounded rectangle"
-    #   A****BQ
-    #  H      C
-    #  *      *
-    #  G      D
-    #   F****E
-
-    context.move_to(x + r, y)      # Move to A
-    context.line_to(x + w - r, y)  # Straight line to B
-
-    # Curve to C, Control points are both at Q
-    context.curve_to(x + w, y, x + w, y, x + w, y + r)
-    context.line_to(x + w, y + h - r)                               # Move to D
-    context.curve_to(x + w, y + h, x + w, y + h, x + w - r, y + h)  # Curve to E
-    context.line_to(x + r, y + h)                                   # Line to F
-    context.curve_to(x, y + h, x, y + h, x, y + h - r)              # Curve to G
-    context.line_to(x, y + r)                                       # Line to H
-    context.curve_to(x, y, x, y, x + r, y)                          # Curve to A
-    return
-
-
 #--------------------- Gtk widget helpers ------------------------------------#
 def model(columns, data):
     ret = Gtk.ListStore(*columns)
@@ -391,9 +427,18 @@ def get_value_from_model(model, key):
         if row[1] == key:
             return str(row[0])
     if isinstance(key, Gst.Fraction):
-        return "%.3f" % Decimal(float(key.num) / key.denom)
+        return "%.3f" % decimal.Decimal(float(key.num) / key.denom)
     return str(key)
 
+
+def alter_style_class(style_class, target_widget, css_style):
+    css_provider = Gtk.CssProvider()
+    toolbar_css = "%s { %s }" % (style_class, css_style)
+    css_provider.load_from_data(toolbar_css.encode('UTF-8'))
+    style_context = target_widget.get_style_context()
+    style_context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+
+
 #------------------------ encoding datas ----------------------------------------#
 # FIXME This should into a special file
 frame_rates = model((str, object), (
@@ -451,49 +496,3 @@ display_aspect_ratios = model((str, object), (
     (_("Anamorphic (2.39)"), Gst.Fraction(239, 100)),
     (_("Anamorphic (2.4)"), Gst.Fraction(24, 10)),
 ))
-
-
-# ---------------------- Classes ---------------------------------------------#
-class Point(tuple):
-
-    def __new__(cls, x, y):
-        return tuple.__new__(cls, (x, y))
-
-    def __pow__(self, scalar):
-        """Returns the scalar multiple self, scalar"""
-        return Point(self[0] * scalar, self[1] * scalar)
-
-    def __rpow__(self, scalar):
-        """Returns the scalar multiple of self, scalar"""
-        return self ** scalar
-
-    def __mul__(self, p2):
-        return Point(*(a * b for a, b in izip(self, p2)))
-
-    def __div__(self, other):
-        return Point(*(a / b for a, b in izip(self, p2)))
-
-    def __floordiv__(self, scalar):
-        """Returns the scalar division of self and scalar"""
-        return Point(self[0] / scalar, self[1] / scalar)
-
-    def __add__(self, p2):
-        """Returns the 2d vector sum self + p2"""
-        return Point(*(a + b for a, b in izip(self, p2)))
-
-    def __sub__(self, p2):
-        """Returns the 2-dvector difference self - p2"""
-        return Point(*(a - b for a, b in izip(self, p2)))
-
-    def __abs__(self):
-        return Point(*(abs(a) for a in self))
-
-    @classmethod
-    def from_item_bounds(self, item):
-        bounds = item.get_bounds()
-        return Point(bounds.x1, bounds.y1), Point(bounds.x2, bounds.y2)
-
-    @classmethod
-    def from_widget_bounds(self, widget):
-        x1, y1, x2, y2 = widget.get_bounds()
-        return Point(x1, y1), Point(x2, y2)
