@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
 #
-#       ui/gstwidget.py
+#       pitivi/utils/widgets.py
 #
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
@@ -26,6 +26,7 @@ A collection of helper classes and routines for:
     * Creating UI from GstElement-s
 """
 
+import math
 import os
 import re
 import sys
@@ -41,12 +42,16 @@ from gettext import gettext as _
 
 from pitivi.utils.loggable import Loggable
 from pitivi.configure import get_ui_dir
-from pitivi.utils.ui import unpack_color, pack_color_32, pack_color_64, \
-    time_to_string, SPACING
+from pitivi.utils.ui import beautify_length, \
+    unpack_color, pack_color_32, pack_color_64, \
+    time_to_string, SPACING, CONTROL_WIDTH
 from pitivi.utils.timeline import Zoomable
 
+
 ZOOM_FIT = _("Zoom Fit")
 
+ZOOM_SLIDER_PADDING = SPACING * 4 / 5
+
 
 class DynamicWidget(object):
 
@@ -67,30 +72,16 @@ class DynamicWidget(object):
     def getWidgetDefault(self):
         return self.default
 
-    def setWidgetDefault(self, value):
-        self.default = value
-
     def setWidgetToDefault(self):
         if self.default is not None:
             self.setWidgetValue(self.default)
 
 
-class DefaultWidget(Gtk.Label, DynamicWidget):
-
+class DefaultWidget(Gtk.Label):
     """When all hope fails...."""
 
-    def __init__(self, default=None, *unused, **kw_unused):
+    def __init__(self, *unused, **unused_kwargs):
         Gtk.Label.__init__(self, _("Implement Me"))
-        DynamicWidget.__init__(self, default)
-
-    def connectValueChanged(self, callback, *args):
-        pass
-
-    def setWidgetValue(self, value):
-        self.set_text(str(value))
-
-    def getWidgetValue(self):
-        return self.get_text()
 
 
 class TextWidget(Gtk.HBox, DynamicWidget):
@@ -127,13 +118,13 @@ class TextWidget(Gtk.HBox, DynamicWidget):
             self.combo = Gtk.ComboBoxText.new_with_entry()
             self.text = self.combo.get_child()
             self.combo.show()
-            self.pack_start(self.combo, True, True, 0)
+            self.pack_start(self.combo, expand=True, fill=True, padding=0)
             for choice in choices:
                 self.combo.append_text(choice)
         else:
             self.text = Gtk.Entry()
             self.text.show()
-            self.pack_start(self.text, True, True, 0)
+            self.pack_start(self.text, expand=True, fill=True, padding=0)
         self.matches = None
         self.last_valid = None
         self.valid = False
@@ -171,11 +162,11 @@ class TextWidget(Gtk.HBox, DynamicWidget):
                 if self.send_signal:
                     self.emit("value-changed")
                 if not self.valid:
-                    self.text.set_icon_from_stock(1, None)
+                    self.text.set_icon_from_icon_name(1, None)
                 self.valid = True
             else:
                 if self.valid:
-                    self.text.set_icon_from_stock(1, Gtk.STOCK_DIALOG_WARNING)
+                    self.text.set_icon_from_icon_name(1, "dialog-warning")
                 self.valid = False
         elif self.send_signal:
             self.emit("value-changed")
@@ -221,7 +212,7 @@ class NumericWidget(Gtk.HBox, DynamicWidget):
         self._type = None
         if (lower is not None and upper is not None) and (lower > -5000 and upper < 5000):
             self.slider = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, self.adjustment)
-            self.pack_start(self.slider, fill=True, expand=True, padding=0)
+            self.pack_start(self.slider, expand=True, fill=True, padding=0)
             self.slider.show()
             self.slider.props.draw_value = False
             # Abuse GTK3's progressbar "fill level" feature to provide
@@ -281,12 +272,13 @@ class TimeWidget(TextWidget, DynamicWidget):
     # The "frame number" match rule is ^([0-9]+)$ (with a + to require 1 digit)
     # The "timecode" rule is ^([0-9]:[0-5][0-9]:[0-5][0-9])\.[0-9][0-9][0-9]$"
     # Combining the two, we get:
-    regex = re.compile("^([0-9]+)$|^([0-9]:[0-5][0-9]:[0-5][0-9])\.[0-9][0-9][0-9]$")
+    VALID_REGEX = re.compile("^([0-9]+)$|^([0-9]:)?([0-5][0-9]:[0-5][0-9])\.[0-9][0-9][0-9]$")
+
     __gtype_name__ = 'TimeWidget'
 
     def __init__(self, default=None):
         DynamicWidget.__init__(self, default)
-        TextWidget.__init__(self, self.regex)
+        TextWidget.__init__(self, self.VALID_REGEX)
         TextWidget.set_width_chars(self, 10)
         self._framerate = None
 
@@ -294,12 +286,17 @@ class TimeWidget(TextWidget, DynamicWidget):
         timecode = TextWidget.getWidgetValue(self)
 
         if ":" in timecode:
-            hh, mm, end = timecode.split(":")
-            ss, xxx = end.split(".")
+            parts = timecode.split(":")
+            if len(parts) == 2:
+                hh = 0
+                mm, end = parts
+            else:
+                hh, mm, end = parts
+            ss, millis = end.split(".")
             nanosecs = int(hh) * 3.6 * 10e12 \
                 + int(mm) * 6 * 10e10 \
                 + int(ss) * 10e9 \
-                + int(xxx) * 10e6
+                + int(millis) * 10e6
             nanosecs = nanosecs / 10  # Compensate the 10 factor of e notation
         else:
             # We were given a frame number. Convert from the project framerate.
@@ -308,11 +305,12 @@ class TimeWidget(TextWidget, DynamicWidget):
         # The seeker won't like floating point nanoseconds!
         return int(nanosecs)
 
-    def setWidgetValue(self, value, send_signal=True):
-        TextWidget.setWidgetValue(self, time_to_string(value),
-                                send_signal=send_signal)
+    def setWidgetValue(self, timeNanos, send_signal=True):
+        timecode = time_to_string(timeNanos)
+        if timecode.startswith("0:"):
+            timecode = timecode[2:]
+        TextWidget.setWidgetValue(self, timecode, send_signal=send_signal)
 
-    # No need to define connectValueChanged as it is inherited from DynamicWidget
     def connectActivateEvent(self, activateCb):
         return self.connect("activate", activateCb)
 
@@ -442,7 +440,7 @@ class ChoiceWidget(Gtk.HBox, DynamicWidget):
         self.choices = None
         self.values = None
         self.contents = Gtk.ComboBoxText()
-        self.pack_start(self.contents, True, True, 0)
+        self.pack_start(self.contents, expand=True, fill=True, padding=0)
         self.setChoices(choices)
         self.contents.show()
         cell = self.contents.get_cells()[0]
@@ -473,132 +471,6 @@ class ChoiceWidget(Gtk.HBox, DynamicWidget):
             self.contents.set_sensitive(True)
 
 
-class PresetChoiceWidget(Gtk.HBox, DynamicWidget):
-
-    """A popup which manages preset settings on a group of widgets supporting
-    the dynamic interface"""
-
-    class WidgetMap(object):
-
-        """A helper class for mapping data from a preset to a set of
-        widgets"""
-
-        def __init__(self):
-            raise NotImplementedError
-
-        def getWidgets(self):
-            raise NotImplementedError
-
-        def map(self, preset):
-            raise NotImplementedError
-
-    class SeqWidgetMap(WidgetMap):
-
-        """Maps widgets positionally to a sequence of values. None can be used
-        if the given position should not map to a widget"""
-
-        def __init__(self, *widgets):
-            self.widgets = widgets
-
-        def getWidgets(self):
-            return (w for w in self.widgets if w)
-
-        def map(self, preset):
-            for w, p in zip(self.widgets, preset):
-                if w:
-                    w.setWidgetValue(p)
-
-        def unmap(self):
-            return [w.getWidgetValue() for w in self.widgets if w]
-
-    def __init__(self, presets, default=None):
-        Gtk.HBox.__init__(self)
-        DynamicWidget.__init__(self, default)
-        self._block_update = False
-        self._widget_map = None
-
-        self.presets = presets
-        presets.connect("preset-added", self._presetAdded)
-        presets.connect("preset-removed", self._presetRemoved)
-
-        self.combo = Gtk.ComboBoxText()
-        self.combo.set_row_separator_func(self._sep_func)
-        for preset in presets:
-            self.combo.append_text(preset[0])
-        self.combo.append_text("-")
-        self.combo.append_text(_("Custom"))
-        self.pack_start(self.combo, True, True, 0)
-        self._custom_row = len(presets) + 1
-
-        save_button = Gtk.Button(stock=Gtk.STOCK_SAVE)
-        self._save_button = save_button
-        self.pack_start(save_button, False, False, 0)
-        save_button.connect("clicked", self._savePreset)
-        self.show_all()
-
-    def _sep_func(self, model, iter):
-        return model[iter][0] == "-"
-
-    def _presetAdded(self, presetlist, preset):
-        row = self._custom_row - 1
-        self._custom_row += 1
-        self.combo.insert_text(row, preset[0])
-        self.combo.set_active(row)
-
-    def _presetRemoved(self, presetlist, preset, index):
-        self.combo.remove_text(index)
-        self._custom_row -= 1
-
-    def _savePreset(self, unused_button):
-        d = Gtk.Dialog(_("Save Preset"), None, Gtk.DialogFlags.MODAL,
-            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE,
-                Gtk.ResponseType.OK))
-        input = Gtk.Entry()
-        ca = d.get_content_area()
-        ca.pack_start(input, True, True, 0)
-        input.show()
-        response = d.run()
-
-        if response == Gtk.ResponseType.OK:
-            name = input.get_text()
-            values = self._widget_map.unmap()
-            self.presets.addPreset(name, values)
-        d.destroy()
-
-    def setWidgetMap(self, map):
-        self._widget_map = map
-        for widget in self._widget_map.getWidgets():
-            widget.connectValueChanged(self._slaveWidgetValueChanged)
-        self.combo.connect("changed", self._comboChanged)
-
-    def mapWidgetsToSeq(self, *args):
-        self.setWidgetMap(self.SeqWidgetMap(*args))
-
-    def _slaveWidgetValueChanged(self, unused_widget):
-        # gtk isn't very friendly to this sort of thing
-        if not self._block_update:
-            self.combo.set_active(self._custom_row)
-
-    def _comboChanged(self, combo):
-        active = combo.get_active()
-        if active > len(self.presets):
-            self._save_button.set_sensitive(True)
-            return
-        preset = self.presets[active][1]
-
-        self._save_button.set_sensitive(False)
-
-        self._block_update = True
-        self._widget_map.map(preset)
-        self._block_update = False
-
-    def connectValueChanged(self, callback, *args):
-        self.combo.connect("changed", callback, *args)
-
-    def setWidgetValue(self, preset_index):
-        self.combo.set_active(preset_index)
-
-
 class PathWidget(Gtk.FileChooserButton, DynamicWidget):
 
     """A Gtk.FileChooserButton which supports the DynamicWidget interface."""
@@ -613,10 +485,8 @@ class PathWidget(Gtk.FileChooserButton, DynamicWidget):
 
     def __init__(self, action=Gtk.FileChooserAction.OPEN, default=None):
         DynamicWidget.__init__(self, default)
-        self.dialog = Gtk.FileChooserDialog(
-            action=action,
-            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_CLOSE,
-             Gtk.ResponseType.CLOSE))
+        self.dialog = Gtk.FileChooserDialog(action=action)
+        self.dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
         self.dialog.set_default_response(Gtk.ResponseType.OK)
         Gtk.FileChooserButton.__init__(self, self.dialog)
         self.dialog.connect("response", self._responseCb)
@@ -696,123 +566,6 @@ class FontWidget(Gtk.FontButton, DynamicWidget):
         return self.get_font_name()
 
 
-class ResolutionWidget(Gtk.HBox, DynamicWidget):
-
-    def __init__(self, default=None):
-        Gtk.HBox.__init__(self)
-        DynamicWidget.__init__(self, default)
-        self.props.spacing = SPACING
-
-        self.dwidth = 0
-        self.dheight = 0
-        self.dwidthWidget = NumericWidget(lower=0)
-        self.dheightWidget = NumericWidget(lower=0)
-        self.pack_start(self.dwidthWidget, True, True, 0)
-        self.pack_start(Gtk.Label("x", True, True, 0))
-        self.pack_start(self.dheightWidget, True, True, 0)
-        self.setWidgetValue((320, 240))
-        self.show_all()
-
-    def connectValueChanged(self, callback, *args):
-        self.dwidthWidget.connectValueChanged(callback, *args)
-        self.dheightWidget.connectValueChanged(callback, *args)
-
-    def setWidgetValue(self, value):
-        width, height = value
-
-        self.dwidthWidget.setWidgetValue(width)
-        self.dheightWidget.setWidgetValue(height)
-
-    def getWidgetValue(self):
-        return self.dwidthWidget.getWidgetValue(),\
-            self.dheightWidget.getWidgetValue()
-
-if __name__ == '__main__':
-
-    def valueChanged(unused_widget, widget, target):
-        target.set_text(str(widget.getWidgetValue()))
-
-    widgets = (
-        (PathWidget, "file:///home/", ()),
-        (TextWidget, "banana", ()),
-        (TextWidget, "words only", ("^([a-zA-Z]+\s*)+$",)),
-        (TextWidget, "numbers only", ("^\d+$",
-            ("12", "14"))),
-        (NumericWidget, 42, (100, 1)),
-        (ToggleWidget, True, ()),
-        (ChoiceWidget, "banana", ((
-            ("banana", "banana"),
-            ("apple", "apple"),
-            ("pear", "pear")),)),
-        (ColorWidget, 0x336699FF, (int,)),
-        (FontWidget, "Sans 9", ()),
-        (FractionWidget, "30M", (
-            Gst.FractionRange(Gst.Fraction(1, 1),
-            Gst.Fraction(30000, 1001)),
-        )),
-        (FractionWidget, Gst.Fraction(25000, 1001), (
-            Gst.FractionRange(
-                Gst.Fraction(1, 1),
-                Gst.Fraction(30000, 1001)
-            ),
-            ("25:1", Gst.Fraction(30, 1), "30M", ),
-        )),
-    )
-
-    W = Gtk.Window()
-    v = Gtk.VBox()
-    t = Gtk.Table()
-
-    for y, (klass, default, args) in enumerate(widgets):
-        w = klass(*args)
-        w.setWidgetValue(default)
-        l = Gtk.Label(label=str(w.getWidgetValue()))
-        w.connectValueChanged(valueChanged, w, l)
-        w.show()
-        l.show()
-        t.attach(w, 0, 1, y, y + 1)
-        t.attach(l, 1, 2, y, y + 1)
-    t.show()
-
-    W.add(t)
-    W.show()
-    Gtk.main()
-
-
-def make_property_widget(unused_element, prop, value=None):
-    """ Creates a Widget for the given element property """
-    # FIXME : implement the case for flags
-    type_name = GObject.type_name(prop.value_type.fundamental)
-
-    if value is None:
-        value = prop.default_value
-    if type_name == "gchararray":
-        widget = TextWidget(default=prop.default_value)
-    elif type_name in ['guint64', 'gint64', 'guint', 'gint', 'gfloat', 'gulong', 'gdouble']:
-        maximum, minimum = None, None
-        if hasattr(prop, "minimum"):
-            minimum = prop.minimum
-        if hasattr(prop, "maximum"):
-            maximum = prop.maximum
-        widget = NumericWidget(default=prop.default_value, upper=maximum, lower=minimum)
-    elif type_name == "gboolean":
-        widget = ToggleWidget(default=prop.default_value)
-    elif type_name == "GEnum":
-        choices = []
-        for key, val in prop.enum_class.__enum_values__.iteritems():
-            choices.append([val.value_name, int(val)])
-        widget = ChoiceWidget(choices, default=prop.default_value)
-    elif type_name == "GstFraction":
-        widget = FractionWidget(None, presets=["0:1"], default=prop.default_value)
-    else:
-        widget = DefaultWidget(type_name)
-
-    if value is not None and type_name != "GFlags":
-        widget.setWidgetValue(value)
-
-    return widget
-
-
 class GstElementSettingsWidget(Gtk.VBox, Loggable):
     """
     Widget to view/modify properties of a Gst.Element
@@ -859,7 +612,7 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
         """
         Set given element on Widget, with optional properties
         """
-        self.info("element:%s, use properties:%s", element, properties)
+        self.info("element: %s, use properties: %s", element, properties)
         self.element = element
         self.ignore = ignore
         self.properties = {}
@@ -883,18 +636,18 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
         else:
             props = [prop for prop in GObject.list_properties(self.element) if not prop.name in self.ignore]
         if not props:
-            table = Gtk.Table(rows=1, columns=1)
+            table = Gtk.Table(n_rows=1, n_columns=1)
             widget = Gtk.Label(label=_("No properties."))
             widget.set_sensitive(False)
             table.attach(widget, 0, 1, 0, 1, yoptions=Gtk.AttachOptions.FILL)
-            self.pack_start(table, True, True, 0)
+            self.pack_start(table, expand=True, fill=True, padding=0)
             self.show_all()
             return
 
         if default_btn:
-            table = Gtk.Table(rows=len(props), columns=4)
+            table = Gtk.Table(n_rows=len(props), n_columns=4)
         else:
-            table = Gtk.Table(rows=len(props), columns=3)
+            table = Gtk.Table(n_rows=len(props), n_columns=3)
 
         table.set_row_spacings(SPACING)
         table.set_col_spacings(SPACING)
@@ -912,14 +665,14 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
             if is_effect:
                 result, prop_value = self.element.get_child_property(prop.name)
                 if result is False:
-                    self.debug("Could not get property %s value", prop.name)
+                    self.debug("Could not get value for property: %s", prop.name)
             else:
                 if use_element_props:
                     prop_value = self.element.get_property(prop.name)
                 else:
                     prop_value = properties.get(prop.name)
 
-            widget = make_property_widget(self.element, prop, prop_value)
+            widget = self._makePropertyWidget(prop, prop_value)
             if isinstance(widget, ToggleWidget):
                 widget.set_label(prop.nick)
                 table.attach(widget, 0, 2, y, y + 1, yoptions=Gtk.AttachOptions.FILL)
@@ -958,10 +711,10 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
 
             y += 1
 
-        self.pack_start(table, True, True, 0)
+        self.pack_start(table, expand=True, fill=True, padding=0)
         self.show_all()
 
-    def _propertyChangedCb(self, element, pspec, widget):
+    def _propertyChangedCb(self, unused_element, pspec, widget):
         widget.setWidgetValue(self.element.get_property(pspec.name))
 
     def _getKeyframeToggleButton(self, prop):
@@ -972,7 +725,7 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
         button.connect('toggled', self._showKeyframesToggledCb, prop)
         return button
 
-    def _getResetToDefaultValueButton(self, prop, widget):
+    def _getResetToDefaultValueButton(self, unused_prop, widget):
         icon = Gtk.Image()
         icon.set_from_icon_name("edit-clear-all-symbolic", Gtk.IconSize.MENU)
         button = Gtk.Button()
@@ -1018,7 +771,7 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
             elif hasattr(track_element, "ui_element") and track_type == track_element.get_track_type():
                 track_element.ui_element.hideKeyframes()
 
-    def _defaultBtnClickedCb(self, button, widget):
+    def _defaultBtnClickedCb(self, unused_button, widget):
         try:
             binding = self.bindings[widget]
         except KeyError:
@@ -1041,23 +794,57 @@ class GstElementSettingsWidget(Gtk.VBox, Loggable):
         """
         d = {}
         for prop, widget in self.properties.iteritems():
-            if (not prop.flags & GObject.PARAM_WRITABLE
-            or isinstance(widget, DefaultWidget)):
+            if not prop.flags & GObject.PARAM_WRITABLE:
+                continue
+            if isinstance(widget, DefaultWidget):
                 continue
             value = widget.getWidgetValue()
             if value is not None and (value != prop.default_value or with_default):
                 d[prop.name] = value
         return d
 
+    def _makePropertyWidget(self, prop, value=None):
+        """ Creates a Widget for the specified element property """
+        type_name = GObject.type_name(prop.value_type.fundamental)
+        if type_name == "gchararray":
+            widget = TextWidget(default=prop.default_value)
+        elif type_name in ['guint64', 'gint64', 'guint', 'gint', 'gfloat', 'gulong', 'gdouble']:
+            maximum, minimum = None, None
+            if hasattr(prop, "minimum"):
+                minimum = prop.minimum
+            if hasattr(prop, "maximum"):
+                maximum = prop.maximum
+            widget = NumericWidget(default=prop.default_value, upper=maximum, lower=minimum)
+        elif type_name == "gboolean":
+            widget = ToggleWidget(default=prop.default_value)
+        elif type_name == "GEnum":
+            choices = []
+            for key, val in prop.enum_class.__enum_values__.iteritems():
+                choices.append([val.value_name, int(val)])
+            widget = ChoiceWidget(choices, default=prop.default_value)
+        elif type_name == "GstFraction":
+            widget = FractionWidget(None, presets=["0:1"], default=prop.default_value)
+        else:
+            # TODO: implement widgets for: GBoxed, GFlags
+            self.fixme("Unsupported property type: %s", type_name)
+            widget = DefaultWidget()
+
+        if value is None:
+            value = prop.default_value
+        if value is not None and not isinstance(widget, DefaultWidget):
+            widget.setWidgetValue(value)
+
+        return widget
+
 
 class GstElementSettingsDialog(Loggable):
     """
     Dialog window for viewing/modifying properties of a Gst.Element
     """
 
-    def __init__(self, elementfactory, properties={}, parent_window=None, isControllable=True):
+    def __init__(self, elementfactory, properties, parent_window=None, isControllable=True):
         Loggable.__init__(self)
-        self.debug("factory:%s, properties:%s", elementfactory, properties)
+        self.debug("factory: %s, properties: %s", elementfactory, properties)
 
         self.builder = Gtk.Builder()
         self.builder.add_from_file(os.path.join(get_ui_dir(), "elementsettingsdialog.ui"))
@@ -1153,7 +940,7 @@ class BaseTabs(Gtk.Notebook):
         if self._hide_hpaned:
             self._showSecondHpanedInMainWindow()
 
-    def _createWindowCb(self, from_notebook, child, x, y):
+    def _createWindowCb(self, unused_from_notebook, child, x, y):
         original_position = self.child_get_property(child, "position")
         label = self.child_get_property(child, "tab-label")
         window = Gtk.Window()
@@ -1194,10 +981,13 @@ class BaseTabs(Gtk.Notebook):
 
 
 class ZoomBox(Gtk.HBox, Zoomable):
+    """
+    Container holding the widgets for zooming.
+
+    @type timeline: TimelineContainer
+    """
+
     def __init__(self, timeline):
-        """
-        This will hold the widgets responsible for zooming.
-        """
         Gtk.HBox.__init__(self)
         Zoomable.__init__(self)
 
@@ -1206,51 +996,69 @@ class ZoomBox(Gtk.HBox, Zoomable):
         zoom_fit_btn = Gtk.Button()
         zoom_fit_btn.set_relief(Gtk.ReliefStyle.NONE)
         zoom_fit_btn.set_tooltip_text(ZOOM_FIT)
-        zoom_fit_icon = Gtk.Image()
-        zoom_fit_icon.set_from_stock(Gtk.STOCK_ZOOM_FIT, Gtk.IconSize.BUTTON)
+        zoom_fit_icon = Gtk.Image.new_from_icon_name("zoom-best-fit", Gtk.IconSize.BUTTON)
         zoom_fit_btn_hbox = Gtk.HBox()
-        zoom_fit_btn_hbox.pack_start(zoom_fit_icon, False, True, 0)
-        zoom_fit_btn_hbox.pack_start(Gtk.Label(_("Zoom")), False, True, 0)
+        zoom_fit_btn_hbox.pack_start(zoom_fit_icon, expand=False, fill=True, padding=0)
+        zoom_fit_btn_hbox.pack_start(Gtk.Label(label=_("Zoom")), expand=False, fill=True, padding=0)
         zoom_fit_btn.add(zoom_fit_btn_hbox)
         zoom_fit_btn.connect("clicked", self._zoomFitCb)
 
-        self.pack_start(zoom_fit_btn, False, True, 0)
+        self.pack_start(zoom_fit_btn, expand=False, fill=True, padding=0)
 
         # zooming slider
         self._zoomAdjustment = Gtk.Adjustment()
-        self._zoomAdjustment.set_value(Zoomable.getCurrentZoomLevel())
-        self._zoomAdjustment.connect("value-changed", self._zoomAdjustmentChangedCb)
         self._zoomAdjustment.props.lower = 0
         self._zoomAdjustment.props.upper = Zoomable.zoom_steps
         zoomslider = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, adjustment=self._zoomAdjustment)
+        # Setting _zoomAdjustment's value must be done after we create the
+        # zoom slider, otherwise the slider remains at the leftmost position.
+        self._zoomAdjustment.set_value(Zoomable.getCurrentZoomLevel())
         zoomslider.props.draw_value = False
-        zoomslider.set_tooltip_text(_("Zoom Timeline"))
         zoomslider.connect("scroll-event", self._zoomSliderScrollCb)
+        zoomslider.connect("value-changed", self._zoomAdjustmentChangedCb)
+        zoomslider.connect("query-tooltip", self._sliderTooltipCb)
+        zoomslider.set_has_tooltip(True)
         zoomslider.set_size_request(100, 0)  # At least 100px wide for precision
-        self.pack_start(zoomslider, True, True, 0)
+        self.pack_start(zoomslider, expand=True, fill=True, padding=ZOOM_SLIDER_PADDING)
 
+        self.set_size_request(CONTROL_WIDTH, -1)
         self.show_all()
 
-        self._updateZoomSlider = True
-
     def _zoomAdjustmentChangedCb(self, adjustment):
-        # GTK crack
-        self._updateZoomSlider = False
-        Zoomable.setZoomLevel(int(adjustment.get_value()))
-        self.timeline._scrollToPlayhead()
-        self.zoomed_fitted = False
-        self._updateZoomSlider = True
+        Zoomable.setZoomLevel(adjustment.get_value())
 
-    def _zoomFitCb(self, button):
+    def _zoomFitCb(self, unused_button):
         self.timeline.zoomFit()
 
     def _zoomSliderScrollCb(self, unused, event):
-        value = self._zoomAdjustment.get_value()
+        delta = 0
         if event.direction in [Gdk.ScrollDirection.UP, Gdk.ScrollDirection.RIGHT]:
-            self._zoomAdjustment.set_value(value + 1)
+            delta = 1
         elif event.direction in [Gdk.ScrollDirection.DOWN, Gdk.ScrollDirection.LEFT]:
-            self._zoomAdjustment.set_value(value - 1)
+            delta = -1
+        elif event.direction in [Gdk.ScrollDirection.SMOOTH]:
+            unused_res, delta_x, delta_y = event.get_scroll_deltas()
+            if delta_x:
+                delta = math.copysign(1, delta_x)
+            elif delta_y:
+                delta = math.copysign(1, -delta_y)
+        if delta:
+            Zoomable.setZoomLevel(Zoomable.getCurrentZoomLevel() + delta)
 
     def zoomChanged(self):
-        if self._updateZoomSlider:
-            self._zoomAdjustment.set_value(self.getCurrentZoomLevel())
+        zoomLevel = self.getCurrentZoomLevel()
+        if int(self._zoomAdjustment.get_value()) != zoomLevel:
+            self._zoomAdjustment.set_value(zoomLevel)
+
+    def _sliderTooltipCb(self, unused_slider, unused_x, unused_y, unused_keyboard_mode, tooltip):
+        # We assume the width of the ruler is exactly the width of the timeline.
+        width_px = self.timeline.ruler.get_allocated_width()
+        timeline_width_ns = Zoomable.pixelToNs(width_px)
+        if timeline_width_ns >= Gst.SECOND:
+            # Translators: %s represents a duration, for example "10 minutes"
+            tip = _("%s displayed") % beautify_length(timeline_width_ns)
+        else:
+            # Translators: This is a tooltip
+            tip = _("%d nanoseconds displayed, because we can") % timeline_width_ns
+        tooltip.set_text(tip)
+        return True
