@@ -1,6 +1,6 @@
 # Pitivi video editor
 #
-#       ui/viewer.py
+#       pitivi/viewer.py
 #
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
@@ -19,16 +19,13 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-import platform
+from gi.repository import Clutter
 from gi.repository import Gtk
+from gi.repository import GtkClutter
 from gi.repository import Gdk
 from gi.repository import Gst
-from gi.repository import GdkX11
 from gi.repository import GObject
 from gi.repository import GES
-GdkX11  # pyflakes
-from gi.repository import GstVideo
-GstVideo    # pyflakes
 import cairo
 
 from gettext import gettext as _
@@ -36,11 +33,11 @@ from time import time
 from math import pi
 
 from pitivi.settings import GlobalSettings
-from pitivi.utils.pipeline import Seeker, SimplePipeline
+from pitivi.utils.loggable import Loggable
+from pitivi.utils.misc import format_ns
+from pitivi.utils.pipeline import AssetPipeline, Seeker
 from pitivi.utils.ui import SPACING, hex_to_rgb
 from pitivi.utils.widgets import TimeWidget
-from pitivi.utils.loggable import Loggable
-from pitivi.utils.misc import print_ns
 
 GlobalSettings.addConfigSection("viewer")
 GlobalSettings.addConfigOption("viewerDocked", section="viewer",
@@ -69,16 +66,11 @@ GlobalSettings.addConfigOption("pointColor", section="viewer",
     default='49a0e0')
 
 
-class PitiviViewer(Gtk.VBox, Loggable):
+class ViewerContainer(Gtk.VBox, Loggable):
     """
-    A Widget to control and visualize a Pipeline
-
-    @ivar pipeline: The current pipeline
-    @type pipeline: L{Pipeline}
-    @ivar action: The action controlled by this Pipeline
-    @type action: L{ViewAction}
+    A wiget holding a viewer and the controls.
     """
-    __gtype_name__ = 'PitiviViewer'
+    __gtype_name__ = 'ViewerContainer'
     __gsignals__ = {
         "activate-playback-controls": (GObject.SignalFlags.RUN_LAST,
             None, (GObject.TYPE_BOOLEAN,)),
@@ -86,7 +78,7 @@ class PitiviViewer(Gtk.VBox, Loggable):
 
     INHIBIT_REASON = _("Currently playing")
 
-    def __init__(self, app, undock_action=None):
+    def __init__(self, app):
         Gtk.VBox.__init__(self)
         self.set_border_width(SPACING)
         self.app = app
@@ -94,13 +86,11 @@ class PitiviViewer(Gtk.VBox, Loggable):
         self.system = app.system
 
         Loggable.__init__(self)
-        self.log("New PitiviViewer")
+        self.log("New ViewerContainer")
 
         self.pipeline = None
-        self._tmp_pipeline = None  # Used for displaying a preview when trimming
-
-        self.sink = None
         self.docked = True
+        self.seeker = Seeker()
 
         # Only used for restoring the pipeline position after a live clip trim preview:
         self._oldTimelinePos = None
@@ -108,13 +98,16 @@ class PitiviViewer(Gtk.VBox, Loggable):
         self._haveUI = False
 
         self._createUi()
-        self.target = self.internal
-        self.undock_action = undock_action
-        if undock_action:
-            self.undock_action.connect("activate", self._toggleDocked)
 
-            if not self.settings.viewerDocked:
-                self.undock()
+        if not self.settings.viewerDocked:
+            self.undock()
+
+    @property
+    def target(self):
+        if self.docked:
+            return self.internal
+        else:
+            return self.external
 
     def setPipeline(self, pipeline, position=None):
         """
@@ -126,29 +119,22 @@ class PitiviViewer(Gtk.VBox, Loggable):
         @type pipeline: L{Pipeline}.
         @param position: Optional position to seek to initially.
         """
-        self.debug("self.pipeline:%r", self.pipeline)
-
-        self.seeker = Seeker()
         self._disconnectFromPipeline()
-        if self.pipeline:
-            self.pipeline.set_state(Gst.State.NULL)
 
+        self.debug("New pipeline: %r", pipeline)
         self.pipeline = pipeline
-        if self.pipeline:
-            self.pipeline.pause()
-            self.seeker.seek(position)
+        self.pipeline.pause()
+        self.seeker.seek(position)
 
-            self.pipeline.connect("state-change", self._pipelineStateChangedCb)
-            self.pipeline.connect("position", self._positionCb)
-            self.pipeline.connect("duration-changed", self._durationChangedCb)
+        self.pipeline.connect("state-change", self._pipelineStateChangedCb)
+        self.pipeline.connect("position", self._positionCb)
+        self.pipeline.connect("duration-changed", self._durationChangedCb)
 
-        self.sink = pipeline.video_overlay
-        self.target.sink = self.sink
         self._switch_output_window()
         self._setUiActive()
 
     def _disconnectFromPipeline(self):
-        self.debug("pipeline:%r", self.pipeline)
+        self.debug("Previous pipeline: %r", self.pipeline)
         if self.pipeline is None:
             # silently return, there's nothing to disconnect from
             return
@@ -170,38 +156,34 @@ class PitiviViewer(Gtk.VBox, Loggable):
         if active:
             self.emit("activate-playback-controls", True)
 
-    def _externalWindowDeleteCb(self, window, event):
+    def _externalWindowDeleteCb(self, unused_window, unused_event):
         self.dock()
         return True
 
-    def _externalWindowConfigureCb(self, window, event):
+    def _externalWindowConfigureCb(self, unused_window, event):
         self.settings.viewerWidth = event.width
         self.settings.viewerHeight = event.height
         self.settings.viewerX = event.x
         self.settings.viewerY = event.y
 
-    def _videoRealized(self, widget):
-        if widget == self.target:
+    def _videoRealizedCb(self, unused_drawing_area, viewer):
+        if viewer == self.target:
+            self.log("Viewer widget realized: %s", viewer)
             self._switch_output_window()
 
     def _createUi(self):
         """ Creates the Viewer GUI """
         # Drawing area
-        # The aspect ratio gets overridden on startup by setDisplayAspectRatio
-        self.aframe = Gtk.AspectFrame(xalign=0.5, yalign=1.0, ratio=4.0 / 3.0,
-                                      obey_child=False)
-
-        self.internal = ViewerWidget(self.app.settings)
+        self.internal = ViewerWidget(self.app.settings, realizedCb=self._videoRealizedCb)
         # Transformation boxed DISABLED
         # self.internal.init_transformation_events()
-        self.aframe.add(self.internal)
-        self.pack_start(self.aframe, True, True, 0)
+        self.pack_start(self.internal, True, True, 0)
 
         self.external_window = Gtk.Window()
         vbox = Gtk.VBox()
         vbox.set_spacing(SPACING)
         self.external_window.add(vbox)
-        self.external = ViewerWidget(self.app.settings)
+        self.external = ViewerWidget(self.app.settings, realizedCb=self._videoRealizedCb)
         vbox.pack_start(self.external, True, True, 0)
         self.external_window.connect("delete-event", self._externalWindowDeleteCb)
         self.external_window.connect("configure-event", self._externalWindowConfigureCb)
@@ -211,15 +193,17 @@ class PitiviViewer(Gtk.VBox, Loggable):
         bbox = Gtk.HBox()
         boxalign = Gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0.0)
         boxalign.add(bbox)
-        self.pack_start(boxalign, False, True, 0)
+        self.pack_start(boxalign, False, True, SPACING)
 
-        self.goToStart_button = Gtk.ToolButton(Gtk.STOCK_MEDIA_PREVIOUS)
+        self.goToStart_button = Gtk.ToolButton()
+        self.goToStart_button.set_icon_name("media-skip-backward")
         self.goToStart_button.connect("clicked", self._goToStartCb)
         self.goToStart_button.set_tooltip_text(_("Go to the beginning of the timeline"))
         self.goToStart_button.set_sensitive(False)
         bbox.pack_start(self.goToStart_button, False, True, 0)
 
-        self.back_button = Gtk.ToolButton(Gtk.STOCK_MEDIA_REWIND)
+        self.back_button = Gtk.ToolButton()
+        self.back_button.set_icon_name("media-seek-backward")
         self.back_button.connect("clicked", self._backCb)
         self.back_button.set_tooltip_text(_("Go back one second"))
         self.back_button.set_sensitive(False)
@@ -230,24 +214,32 @@ class PitiviViewer(Gtk.VBox, Loggable):
         bbox.pack_start(self.playpause_button, False, True, 0)
         self.playpause_button.set_sensitive(False)
 
-        self.forward_button = Gtk.ToolButton(Gtk.STOCK_MEDIA_FORWARD)
+        self.forward_button = Gtk.ToolButton()
+        self.forward_button.set_icon_name("media-seek-forward")
         self.forward_button.connect("clicked", self._forwardCb)
         self.forward_button.set_tooltip_text(_("Go forward one second"))
         self.forward_button.set_sensitive(False)
         bbox.pack_start(self.forward_button, False, True, 0)
 
-        self.goToEnd_button = Gtk.ToolButton(Gtk.STOCK_MEDIA_NEXT)
+        self.goToEnd_button = Gtk.ToolButton()
+        self.goToEnd_button.set_icon_name("media-skip-forward")
         self.goToEnd_button.connect("clicked", self._goToEndCb)
         self.goToEnd_button.set_tooltip_text(_("Go to the end of the timeline"))
         self.goToEnd_button.set_sensitive(False)
         bbox.pack_start(self.goToEnd_button, False, True, 0)
 
-        # current time
         self.timecode_entry = TimeWidget()
         self.timecode_entry.setWidgetValue(0)
         self.timecode_entry.set_tooltip_text(_('Enter a timecode or frame number\nand press "Enter" to go to that position'))
         self.timecode_entry.connectActivateEvent(self._entryActivateCb)
         bbox.pack_start(self.timecode_entry, False, 10, 0)
+
+        self.undock_button = Gtk.ToolButton()
+        self.undock_button.set_icon_name("view-restore")
+        self.undock_button.connect("clicked", self.undock)
+        self.undock_button.set_tooltip_text(_("Detach the viewer\nYou can re-attach it by closing the newly created window."))
+        bbox.pack_start(self.undock_button, False, True, 0)
+
         self._haveUI = True
 
         # Identify widgets for AT-SPI, making our test suite easier to develop
@@ -258,6 +250,7 @@ class PitiviViewer(Gtk.VBox, Loggable):
         self.forward_button.get_accessible().set_name("forward_button")
         self.goToEnd_button.get_accessible().set_name("goToEnd_button")
         self.timecode_entry.get_accessible().set_name("timecode_entry")
+        self.undock_button.get_accessible().set_name("undock_button")
 
         screen = Gdk.Screen.get_default()
         height = screen.get_height()
@@ -269,37 +262,20 @@ class PitiviViewer(Gtk.VBox, Loggable):
             width = req.width
             height = req.height
             width += 110
-            height = int(width / self.aframe.props.ratio)
-            self.aframe.set_size_request(width, height)
+            height = int(width / self.internal.props.ratio)
+            self.internal.set_size_request(width, height)
 
         self.buttons = bbox
         self.buttons_container = boxalign
-        # Prevent black frames and flickering while resizing or changing focus:
-        self.internal.set_double_buffered(False)
-        self.external.set_double_buffered(False)
-        # We keep the ViewerWidget hidden initially, or the desktop wallpaper
-        # would show through the non-double-buffered widget!
-        self.internal.set_no_show_all(True)
-        self.external.set_no_show_all(True)
-        self.internal.connect("realize", self._videoRealized)
-        self.external.connect("realize", self._videoRealized)
         self.show_all()
         self.external_vbox.show_all()
 
     def setDisplayAspectRatio(self, ratio):
-        """
-        Sets the DAR of the Viewer to the given ratio.
+        self.debug("Setting aspect ratio to %f [%r]", float(ratio), ratio)
+        self.internal.setDisplayAspectRatio(ratio)
+        self.external.setDisplayAspectRatio(ratio)
 
-        @arg ratio: The aspect ratio to set on the viewer
-        @type ratio: L{float}
-        """
-        self.debug("Setting ratio of %f [%r]", float(ratio), ratio)
-        try:
-            self.aframe.set_property("ratio", float(ratio))
-        except:
-            self.warning("could not set ratio !")
-
-    def _entryActivateCb(self, entry):
+    def _entryActivateCb(self, unused_entry):
         self._seekFromTimecodeWidget()
 
     def _seekFromTimecodeWidget(self):
@@ -330,91 +306,78 @@ class PitiviViewer(Gtk.VBox, Loggable):
             self.sink.set_render_rectangle(*area)
             self.target.box.update_size(area)
             self.target.zoom = zoom
-            self.target.sink = self.sink
             self.target.renderbox()
 
-    def _playButtonCb(self, unused_button, playing):
+    def _playButtonCb(self, unused_button, unused_playing):
         self.app.current_project.pipeline.togglePlayback()
+        self.app.gui.focusTimeline()
 
     def _goToStartCb(self, unused_button):
         self.seeker.seek(0)
+        self.app.gui.focusTimeline()
 
     def _backCb(self, unused_button):
         # Seek backwards one second
         self.seeker.seekRelative(0 - Gst.SECOND)
+        self.app.gui.focusTimeline()
 
     def _forwardCb(self, unused_button):
         # Seek forward one second
         self.seeker.seekRelative(Gst.SECOND)
+        self.app.gui.focusTimeline()
 
     def _goToEndCb(self, unused_button):
-        try:
-            end = self.app.current_project.pipeline.getDuration()
-        except:
-            self.warning("Couldn't get timeline duration")
-        try:
-            self.seeker.seek(end)
-        except:
-            self.warning("Couldn't seek to the end of the timeline")
+        end = self.app.current_project.pipeline.getDuration()
+        self.seeker.seek(end)
+        self.app.gui.focusTimeline()
 
     ## public methods for controlling playback
 
-    def undock(self):
-        if not self.undock_action:
-            self.error("Cannot undock because undock_action is missing.")
-            return
+    def undock(self, *unused_widget):
         if not self.docked:
+            self.warning("The viewer is already undocked")
             return
 
         self.docked = False
         self.settings.viewerDocked = False
-        self.undock_action.set_label(_("Dock Viewer"))
-        self.target = self.external
 
         self.remove(self.buttons_container)
         self.external_vbox.pack_end(self.buttons_container, False, False, 0)
         self.external_window.set_type_hint(Gdk.WindowTypeHint.UTILITY)
         self.external_window.show()
 
-        self.fullscreen_button = Gtk.ToggleToolButton(Gtk.STOCK_FULLSCREEN)
+        self.undock_button.hide()
+        self.fullscreen_button = Gtk.ToggleToolButton()
+        self.fullscreen_button.set_icon_name("view-fullscreen")
         self.fullscreen_button.set_tooltip_text(_("Show this window in fullscreen"))
         self.buttons.pack_end(self.fullscreen_button, expand=False, fill=False, padding=6)
         self.fullscreen_button.show()
         self.fullscreen_button.connect("toggled", self._toggleFullscreen)
 
         # if we are playing, switch output immediately
-        if self.sink:
+        if self.pipeline:
             self._switch_output_window()
         self.hide()
         self.external_window.move(self.settings.viewerX, self.settings.viewerY)
         self.external_window.resize(self.settings.viewerWidth, self.settings.viewerHeight)
 
     def dock(self):
-        if not self.undock_action:
-            self.error("Cannot dock because undock_action is missing.")
-            return
         if self.docked:
+            self.warning("The viewer is already docked")
             return
         self.docked = True
         self.settings.viewerDocked = True
-        self.undock_action.set_label(_("Undock Viewer"))
-        self.target = self.internal
 
+        self.undock_button.show()
         self.fullscreen_button.destroy()
         self.external_vbox.remove(self.buttons_container)
         self.pack_end(self.buttons_container, False, False, 0)
         self.show()
         # if we are playing, switch output immediately
-        if self.sink:
+        if self.pipeline:
             self._switch_output_window()
         self.external_window.hide()
 
-    def _toggleDocked(self, action):
-        if self.docked:
-            self.undock()
-        else:
-            self.dock()
-
     def _toggleFullscreen(self, widget):
         if widget.get_active():
             self.external_window.hide()
@@ -449,38 +412,36 @@ class PitiviViewer(Gtk.VBox, Loggable):
 
         clip_uri = tl_obj.props.uri
         cur_time = time()
-        if not self._tmp_pipeline:
+        if self.pipeline == self.app.current_project.pipeline:
             self.debug("Creating temporary pipeline for clip %s, position %s",
-                clip_uri, print_ns(position))
-
+                clip_uri, format_ns(position))
             self._oldTimelinePos = self.pipeline.getPosition()
-            self._tmp_pipeline = Gst.ElementFactory.make("playbin", None)
-            self._tmp_pipeline.set_property("uri", clip_uri)
-            self.setPipeline(SimplePipeline(self._tmp_pipeline, self._tmp_pipeline))
+            self.setPipeline(AssetPipeline(tl_obj))
             self._lastClipTrimTime = cur_time
+
         if (cur_time - self._lastClipTrimTime) > 0.2 and self.pipeline.getState() == Gst.State.PAUSED:
             # Do not seek more than once every 200 ms (for performance)
-            self._tmp_pipeline.seek_simple(Gst.Format.TIME, Gst.SeekFlags.FLUSH, position)
+            self.pipeline.simple_seek(position)
             self._lastClipTrimTime = cur_time
 
     def clipTrimPreviewFinished(self):
         """
         After trimming a clip, reset the project pipeline into the viewer.
         """
-        if self._tmp_pipeline is not None:
-            self._tmp_pipeline.set_state(Gst.State.NULL)
-            self._tmp_pipeline = None  # Free the memory
+        if self.pipeline is not self.app.current_project.pipeline:
+            self.pipeline.setState(Gst.State.NULL)
+            # Using pipeline.getPosition() here does not work because for some
+            # reason it's a bit off, that's why we need self._oldTimelinePos.
             self.setPipeline(self.app.current_project.pipeline, self._oldTimelinePos)
-            self.debug("Back to old pipeline")
+            self.debug("Back to the project's pipeline")
 
-    def _pipelineStateChangedCb(self, pipeline, state):
+    def _pipelineStateChangedCb(self, unused_pipeline, state):
         """
         When playback starts/stops, update the viewer widget,
         play/pause button and (un)inhibit the screensaver.
 
         This is meant to be called by mainwindow.
         """
-        self.info("current state changed : %s", state)
         if int(state) == int(Gst.State.PLAYING):
             self.playpause_button.setPause()
             self.system.inhibitScreensaver(self.INHIBIT_REASON)
@@ -488,25 +449,20 @@ class PitiviViewer(Gtk.VBox, Loggable):
             self.playpause_button.setPlay()
             self.system.uninhibitScreensaver(self.INHIBIT_REASON)
         else:
-            self.sink = None
             self.system.uninhibitScreensaver(self.INHIBIT_REASON)
         self.internal._currentStateCb(self.pipeline, state)
 
     def _switch_output_window(self):
         # Don't do anything if we don't have a pipeline
-        if self.sink is None:
+        if self.pipeline is None:
             return
 
         if self.target.get_realized():
-            if platform.system() == 'Windows':
-                xid = self.target.get_window().get_handle()
-            else:
-                xid = self.target.get_window().get_xid()
-
-            self.sink.set_window_handle(xid)
-            self.sink.expose()
+            self.debug("Connecting the pipeline to the viewer's texture")
+            self.pipeline.connectWithViewer(self.target)
         else:
             # Show the widget and wait for the realized callback
+            self.log("Target is not realized, showing the widget")
             self.target.show()
 
 
@@ -823,9 +779,9 @@ class TransformationBox():
             self.transformation_properties.connectSpinButtonsToFlush()
 
 
-class ViewerWidget(Gtk.DrawingArea, Loggable):
+class ViewerWidget(Gtk.AspectFrame, Loggable):
     """
-    Widget for displaying properly GStreamer video sink
+    Widget for displaying a GStreamer video sink.
 
     @ivar settings: The settings of the application.
     @type settings: L{GlobalSettings}
@@ -833,9 +789,31 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
 
     __gsignals__ = {}
 
-    def __init__(self, settings=None):
-        Gtk.DrawingArea.__init__(self)
+    def __init__(self, settings=None, realizedCb=None):
+        # Prevent black frames and flickering while resizing or changing focus:
+        # The aspect ratio gets overridden by setDisplayAspectRatio.
+        Gtk.AspectFrame.__init__(self, xalign=0.5, yalign=1.0,
+                                 ratio=4.0 / 3.0, obey_child=False)
         Loggable.__init__(self)
+
+        self.drawing_area = GtkClutter.Embed()
+        self.drawing_area.set_double_buffered(False)
+        # We keep the ViewerWidget hidden initially, or the desktop wallpaper
+        # would show through the non-double-buffered widget!
+        if realizedCb:
+            self.drawing_area.connect("realize", realizedCb, self)
+        self.add(self.drawing_area)
+
+        layout_manager = Clutter.BinLayout(x_align=Clutter.BinAlignment.FILL, y_align=Clutter.BinAlignment.FILL)
+        self.drawing_area.get_stage().set_layout_manager(layout_manager)
+        self.texture = Clutter.Texture()
+        # This is a trick to make the viewer appear darker at the start.
+        self.texture.set_from_rgb_data(data=[0] * 3, has_alpha=False,
+                width=1, height=1, rowstride=3, bpp=3,
+                flags=Clutter.TextureFlags.NONE)
+        self.drawing_area.get_stage().add_child(self.texture)
+        self.drawing_area.show()
+
         self.seeker = Seeker()
         self.settings = settings
         self.box = None
@@ -850,6 +828,9 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
         #for state in range(Gtk.StateType.INSENSITIVE + 1):
             #self.modify_bg(state, self.style.black)
 
+    def setDisplayAspectRatio(self, ratio):
+        self.set_property("ratio", float(ratio))
+
     def init_transformation_events(self):
         self.fixme("TransformationBox disabled")
         """
@@ -874,7 +855,7 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
             self.renderbox()
         """
 
-    def _sizeCb(self, widget, area):
+    def _sizeCb(self, unused_widget, unused_area):
         # The transformation box is cleared when using regular rendering
         # so we need to flush the pipeline
         self.seeker.flush()
@@ -914,7 +895,7 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
 
         self.stored = True
 
-    def button_release_event(self, widget, event):
+    def button_release_event(self, unused_widget, event):
         if event.button == 1:
             self.box.update_effect_properties()
             self.box.release_point()
@@ -922,12 +903,12 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
             self.stored = False
         return True
 
-    def button_press_event(self, widget, event):
+    def button_press_event(self, unused_widget, event):
         if event.button == 1:
             self.box.select_point(event)
         return True
 
-    def _currentStateCb(self, pipeline, state):
+    def _currentStateCb(self, unused_pipeline, unused_state):
         self.fixme("TransformationBox disabled")
         """
         self.pipeline = pipeline
@@ -936,7 +917,7 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
         self.renderbox()
         """
 
-    def motion_notify_event(self, widget, event):
+    def motion_notify_event(self, unused_widget, event):
         if event.get_state() & Gdk.ModifierType.BUTTON1_MASK:
             if self.box.transform(event):
                 if self.stored:
@@ -991,7 +972,7 @@ class ViewerWidget(Gtk.DrawingArea, Loggable):
                 if self.box.area.width != self.pixbuf.get_width():
                     cr.restore()
 
-            if self.pipeline and self.pipeline.get_state()[1] == Gst.State.PAUSED:
+            if self.pipeline and self.pipeline.getState() == Gst.State.PAUSED:
                 self.box.draw(cr)
             cr.pop_group_to_source()
             cr.paint()
@@ -1022,17 +1003,15 @@ class PlayPauseButton(Gtk.Button, Loggable):
         self.emit("play", self.playing)
 
     def setPlay(self):
-        """ display the play image """
-        self.log("setPlay")
+        self.log("Displaying the play image")
         self.playing = True
-        self.set_image(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_PLAY, Gtk.IconSize.BUTTON))
+        self.set_image(Gtk.Image.new_from_icon_name("media-playback-start", Gtk.IconSize.BUTTON))
         self.set_tooltip_text(_("Play"))
         self.playing = False
 
     def setPause(self):
-        self.log("setPause")
-        """ display the pause image """
+        self.log("Displaying the pause image")
         self.playing = False
-        self.set_image(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_PAUSE, Gtk.IconSize.BUTTON))
+        self.set_image(Gtk.Image.new_from_icon_name("media-playback-pause", Gtk.IconSize.BUTTON))
         self.set_tooltip_text(_("Pause"))
         self.playing = True
