@@ -1,4 +1,4 @@
-# PiTiVi , Non-linear video editor
+# Pitivi video editor
 #
 #       tests/test_undo.py
 #
@@ -21,8 +21,8 @@
 
 from unittest import TestCase
 
-from pitivi.undo import UndoError, UndoWrongStateError, UndoableAction, \
-        UndoableActionStack, UndoableActionLog
+from pitivi.undo.undo import UndoError, UndoWrongStateError, UndoableAction, \
+    UndoableActionStack, UndoableActionLog
 
 
 class DummyUndoableAction(UndoableAction):
